@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2014 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # Copyright (C) 2008 Noah Kantrowitz <noah@coderanger.net>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 # Trac documentation build configuration file, created by
 # sphinx-quickstart on Wed May 14 09:05:13 2008.
@@ -25,29 +25,30 @@
 # All configuration values have a default value; values that are
 # commented out serve to show the default value.
 
-import sys, os
+import os
+import sys
+from datetime import datetime
+
+from trac.util import get_pkginfo
+
+pkg_info = get_pkginfo(sys.modules['trac'])
 
 # General substitutions.
 project = 'Trac'
-copyright = '2015, Edgewall Software'
-url = 'http://trac.edgewall.org'
+copyright = '%s, Edgewall Software' % datetime.now().year
+url = pkg_info['home_page']
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '1.0.3'
+version = pkg_info['version'].split('dev')[0]
 # The full version, including alpha/beta/rc tags.
-release = '1.0.3'
+release = pkg_info['version']
 
 # Devel or Release mode for the documentation (if devel, include TODOs,
 # can also be used in conditionals: .. ifconfig :: devel)
-devel = True
-
-if devel:
-    release += 'dev'
-
-
+devel = 'dev' in pkg_info['version']
 
 # If your extensions are in another directory, add it here. If the
 # directory is relative to the documentation root, use os.path.abspath
@@ -76,7 +77,7 @@ extensions.append('sphinx.ext.ifconfig')
 # -- Link to other Sphinx documentations
 extensions.append('sphinx.ext.intersphinx')
 
-intersphinx_mapping = {'python': ('http://docs.python.org/2.7', None)}
+intersphinx_mapping = {'python': ('https://docs.python.org/2.7', None)}
 
 # -- Keep track of :todo: items
 extensions.append('sphinx.ext.todo')
@@ -90,6 +91,11 @@ try:
 except ImportError:
     pass
 
+# -- Documentation coverage (`make apidoc-coverage`)
+extensions.append('sphinx.ext.coverage')
+
+coverage_skip_undoc_in_source = True
+
 
 # Add any paths that contain templates here, relative to this directory.
 #templates_path = ['utils/templates']
@@ -361,4 +367,4 @@ def setup(app):
     roles.register_canonical_role('extensionpoints', extensionpoints_role)
 
     # ifconfig variables
-    app.add_config_value('devel', '', True)
+    app.add_config_value('devel', True, '')
