@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2019 Edgewall Software
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 """Sample Wiki syntax extension plugin."""
 
@@ -22,8 +22,8 @@ from trac.versioncontrol.api import NoSuchChangeset, RepositoryManager
 from trac.versioncontrol.web_ui import ChangesetModule
 from trac.wiki.api import IWikiSyntaxProvider
 
-revision = "$Rev: 12501 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.2.2/sample-plugins/revision_links.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 class RevisionLinks(Component):
     """Adds a few more ways to refer to changesets."""
