@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2019 Edgewall Software
 # Copyright (C) 2007 Eli Carter <retracile@gmail.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 from genshi.builder import tag
 
@@ -19,8 +19,8 @@ from trac.perm import IPermissionRequestor
 from trac.ticket.api import ITicketActionController
 from trac.ticket.default_workflow import ConfigurableTicketWorkflow
 
-revision = "$Rev: 13432 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.2.2/sample-plugins/workflow/CodeReview.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 class CodeReviewActionController(Component):
     """Support for simple code reviews.
@@ -32,7 +32,6 @@ class CodeReviewActionController(Component):
     Example (from the enterprise-review-workflow.ini):
     {{{
     review = in_review -> *
-    review.label = review as
     review.operations = code_review
     review.code_review =
       approve -> in_QA,
