@@ -1,11 +1,23 @@
-from genshi.builder import tag
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007-2019 Edgewall Software
+# Copyright (C) 2007 Eli Carter <retracile@gmail.com>
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at https://trac.edgewall.org/.
 
 from trac.core import implements,Component
 from trac.ticket.api import ITicketActionController
 from trac.perm import IPermissionRequestor
 
-revision = "$Rev: 11075 $"
-url = "$URL: file:///var/svn/trac/tags/trac-1.0/sample-plugins/workflow/DeleteTicket.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 class DeleteTicketActionController(Component):
     """Provides the admin with a way to delete a ticket.
@@ -31,7 +43,8 @@ class DeleteTicketActionController(Component):
 
     def get_ticket_actions(self, req, ticket):
         actions = []
-        if 'TICKET_DELETE' in req.perm(ticket.resource):
+        if 'TICKET_DELETE' in req.perm(ticket.resource) \
+                and ticket.exists:
             actions.append((0,'delete'))
         return actions
 
@@ -47,5 +60,5 @@ class DeleteTicketActionController(Component):
     def apply_action_side_effects(self, req, ticket, action):
         # Be paranoid here, as this should only be called when
         # action is delete...
-        if action == 'delete': 
+        if action == 'delete':
             ticket.delete()
