@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2019 Edgewall Software
 # Copyright (C) 2007 Eli Carter <retracile@gmail.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 
 from genshi.builder import tag
 
@@ -18,8 +18,8 @@ from trac.core import Component, implements
 from trac.ticket.api import ITicketActionController, TicketSystem
 from trac.perm import IPermissionRequestor
 
-revision = "$Rev: 14531 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/workflow/StatusFixer.py $"
+revision = "$Rev$"
+url = "$URL$"
 
 class StatusFixerActionController(Component):
     """Provides the admin with a way to correct a ticket's status.
@@ -47,7 +47,7 @@ class StatusFixerActionController(Component):
 
     def get_ticket_actions(self, req, ticket):
         actions = []
-        if 'TICKET_STATUSFIX' in req.perm(ticket.resource):
+        if ticket.exists and 'TICKET_STATUSFIX' in req.perm(ticket.resource):
             actions.append((0, 'force_status'))
         return actions
 
