@@ -1,16 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2010 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from __future__ import print_function
 
@@ -135,10 +135,10 @@ class TracAdmin(cmd.Cmd):
         self.interactive = True
         printout(_("""Welcome to trac-admin %(version)s
 Interactive Trac administration console.
-Copyright (C) 2003-2013 Edgewall Software
+Copyright (C) %(year)s Edgewall Software
 
 Type:  '?' or 'help' for help on commands.
-        """, version=TRAC_VERSION))
+        """, version=TRAC_VERSION, year='2003-2019'))
         self.cmdloop()
 
     ##
@@ -523,11 +523,11 @@ Warning: couldn't index the default repository.
 
 This can happen for a variety of reasons: wrong repository type,
 no appropriate third party library for this repository type,
-no actual repository at the specified repository path...
+no repository at the specified repository path...
 
 You can nevertheless start using your Trac environment, but
-you'll need to check again your trac.ini file and the [trac]
-repository_type and repository_path settings.
+you'll need to check your `.type` and `.dir` option values in
+the [repositories] section of your trac.ini file.
 """))
         except Exception as e:
             initenv_error(to_unicode(e))
@@ -555,12 +555,12 @@ deploying Trac to a real web server).
 The latest documentation can also always be found on the project
 website:
 
-  http://trac.edgewall.org/
+  https://trac.edgewall.org/
 
 Congratulations!
 """, project_name=project_name, project_path=self.envname,
            project_dir=os.path.basename(self.envname),
-           config_path=os.path.join(self.envname, 'conf', 'trac.ini')))
+           config_path=self.__env.config_file_path))
 
     def _resync_feedback(self, rev):
         sys.stdout.write(' [%s]\r' % rev)
