@@ -1,26 +1,28 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2013 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Tim Moloney <t.moloney@verizon.net>
 
-from __future__ import with_statement
-
+import copy
 import difflib
 import inspect
 import os
 import re
 import sys
+import tempfile
+import textwrap
 import unittest
+from subprocess import PIPE, Popen
 from StringIO import StringIO
 
 # IAdminCommandProvider implementations
@@ -49,8 +51,12 @@ import trac.wiki.web_ui
 from trac.admin.api import AdminCommandManager, IAdminCommandProvider, \
                            console_date_format, get_console_locale
 from trac.admin.console import TracAdmin, TracAdminHelpMacro, _run
-from trac.core import Component, implements
-from trac.test import EnvironmentStub
+from trac.config import ConfigSection, Option
+from trac.core import Component, ComponentMeta, implements
+from trac.env import Environment
+from trac.test import EnvironmentStub, rmtree
+from trac.util import create_file
+from trac.util.compat import close_fds
 from trac.util.datefmt import format_date, get_date_format_hint, \
                               get_datetime_format_hint
 from trac.util.translation import get_available_locales, has_babel
@@ -122,11 +128,7 @@ def execute_run(args):
     return _execute(func)
 
 
-class TracadminTestCase(unittest.TestCase):
-    """
-    Tests the output of trac-admin and is meant to be used with
-    .../trac/tests.py.
-    """
+class TracAdminTestCaseBase(unittest.TestCase):
 
     expected_results_file = os.path.join(os.path.dirname(__file__),
                                          'console-tests.txt')
@@ -134,32 +136,11 @@ class TracadminTestCase(unittest.TestCase):
     expected_results = load_expected_results(expected_results_file,
                                              '===== (test_[^ ]+) =====')
 
-    def setUp(self):
-        self.env = EnvironmentStub(default_data=True, enable=('trac.*',),
-                                   disable=('trac.tests.*',))
-        self._admin = TracAdmin()
-        self._admin.env_set('', self.env)
-
-        # Set test date to 11th Jan 2004
-        self._test_date = '2004-01-11'
-
-    def tearDown(self):
-        self.env = None
-
     def _execute(self, cmd, strip_trailing_space=True, input=None):
         return execute_cmd(self._admin, cmd,
                            strip_trailing_space=strip_trailing_space,
                            input=input)
 
-    @property
-    def _datetime_format_hint(self):
-        return get_datetime_format_hint(get_console_locale(self.env))
-
-    def _get_command_help(self, *args):
-        docs = AdminCommandManager(self.env).get_command_help(list(args))
-        self.assertEqual(1, len(docs))
-        return docs[0][2]
-
     def assertExpectedResult(self, output, args=None):
         test_name = inspect.stack()[1][3]
         expected_result = self.expected_results[test_name]
@@ -179,9 +160,15 @@ class TracadminTestCase(unittest.TestCase):
             expected_lines = ['%s\n' % x for x in expected_results.split('\n')]
             return ''.join(difflib.unified_diff(expected_lines, output_lines,
                                                 'expected', 'actual'))
+
+        # psycopg2 2.8.x raises UniqueViolation, which inherits from
+        # IntegrityError.
+        output = re.sub('^(UniqueViolation:)', 'IntegrityError:', output)
         if '[...]' in expected_results:
-            m = re.match(expected_results.replace('[...]', '.*'), output,
-                         re.MULTILINE)
+            m = re.match('.*'.join(map(re.escape,
+                                       expected_results.split('[...]'))) +
+                         '\Z',
+                         output, re.DOTALL)
             unittest.TestCase.assertTrue(self, m,
                                          "%r != %r\n%s" % (expected_results,
                                                            output, diff()))
@@ -189,6 +176,58 @@ class TracadminTestCase(unittest.TestCase):
             unittest.TestCase.assertEqual(self, expected_results, output,
                                           "%r != %r\n%s" % (expected_results,
                                                             output, diff()))
+
+
+class TracadminTestCase(TracAdminTestCaseBase):
+    """
+    Tests the output of trac-admin and is meant to be used with
+    .../trac/tests.py.
+    """
+
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True, enable=('trac.*',),
+                                   disable=('trac.tests.*',))
+        self._admin = TracAdmin()
+        self._admin.env_set('', self.env)
+        self.environ = os.environ.copy()
+
+        # Set test date to 11th Jan 2004
+        self._test_date = '2004-01-11'
+
+    def tearDown(self):
+        self.env = None
+        for name in set(os.environ) - set(self.environ):
+            del os.environ[name]
+        os.environ.update(self.environ)
+
+    @property
+    def _datetime_format_hint(self):
+        return get_datetime_format_hint(get_console_locale(self.env))
+
+    def _get_command_help(self, *args):
+        docs = AdminCommandManager(self.env).get_command_help(list(args))
+        self.assertEqual(1, len(docs))
+        return docs[0][2]
+
+    def _complete_command(self, *args):
+        return AdminCommandManager(self.env).complete_command(list(args))
+
+    def test_python_with_optimizations_returns_error(self):
+        """Error is returned when a command is executed in interpreter
+        with optimizations enabled.
+        """
+        env = os.environ.copy()
+        env['LC_ALL'] = env['LANG'] = 'C'  # use en_US locale messages
+        proc = Popen((sys.executable, '-O', '-m', 'trac.admin.console',
+                      'help'), stdin=PIPE, stdout=PIPE, stderr=PIPE,
+                     close_fds=close_fds, env=env)
+        stdout, stderr = proc.communicate(input='')
+        for f in (proc.stdin, proc.stdout, proc.stderr):
+            f.close()
+        self.assertEqual(2, proc.returncode)
+        self.assertEqual("Python with optimizations is not supported.",
+                         stderr.strip())
+
     # Help test
 
     def test_help_ok(self):
@@ -197,12 +236,10 @@ class TracadminTestCase(unittest.TestCase):
         has no command arguments, it is hard to call it incorrectly.  As
         a result, there is only this one test.
         """
-        from trac import __version__
-
         rv, output = self._execute('help')
         self.assertEqual(0, rv, output)
         self.assertExpectedResult(output, {
-            'version': __version__,
+            'version': self.env.trac_version,
             'date_format_hint': get_date_format_hint()
         })
         self.assertTrue(all(len(line) < 80 for line in output.split('\n')),
@@ -215,48 +252,72 @@ class TracadminTestCase(unittest.TestCase):
         locales = get_available_locales()
         en_US = Locale.parse('en_US')
         de = Locale.parse('de')
-        de_DE = Locale.parse('de_DE')
-        try:
-            default = Locale.default()
-        except UnknownLocaleError:
-            default = None
 
-        language = self.env.config.get('trac', 'default_language')
-        try:
-            self.assertEqual(default, get_console_locale(None, None))
-            self.env.config.set('trac', 'default_language', '')
-            if 'de' in locales:
-                self.assertEqual(de, get_console_locale(None, 'de_DE.UTF8'))
-                self.env.config.set('trac', 'default_language', 'de')
-                self.assertEqual(de, get_console_locale(self.env, None))
-                self.assertEqual(de, get_console_locale(self.env, 'C'))
-                self.env.config.set('trac', 'default_language', 'en_US')
-                self.assertEqual(en_US, get_console_locale(self.env, None))
-                self.assertEqual(en_US, get_console_locale(self.env, 'C'))
-                self.assertEqual(de, get_console_locale(self.env,
-                                                        'de_DE.UTF8'))
-            if not locales:  # compiled catalog is missing
-                self.assertEqual(default, get_console_locale(None,
-                                                             'de_DE.UTF8'))
-                self.env.config.set('trac', 'default_language', 'de')
-                self.assertEqual(default, get_console_locale(self.env, None))
-                self.assertEqual(default, get_console_locale(self.env, 'C'))
-                self.env.config.set('trac', 'default_language', 'en_US')
-                self.assertEqual(en_US, get_console_locale(self.env, None))
-                self.assertEqual(en_US, get_console_locale(self.env, 'C'))
-                self.assertEqual(en_US, get_console_locale(self.env,
-                                                           'de_DE.UTF8'))
-        finally:
-            self.env.config.set('trac', 'default_language', language)
+        def unset_locale_envs():
+            for name in ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG'):
+                if name in os.environ:
+                    del os.environ[name]
+
+        if 'de' in locales:
+            unset_locale_envs()
+            self.assertEqual(None, get_console_locale(None, None))
+            self.assertEqual(de, get_console_locale(None, 'de_DE.UTF8'))
+            self.env.config.set('trac', 'default_language', 'de')
+            self.assertEqual(de, get_console_locale(self.env, None))
+            self.assertEqual(de, get_console_locale(self.env, 'C'))
+            self.env.config.set('trac', 'default_language', 'en_US')
+            self.assertEqual(en_US, get_console_locale(self.env, None))
+            self.assertEqual(en_US, get_console_locale(self.env, 'C'))
+            self.assertEqual(de, get_console_locale(self.env,
+                                                    'de_DE.UTF8'))
+
+            self.env.config.set('trac', 'default_language', 'de')
+            os.environ['LANG'] = 'POSIX'  # unavailable locale in Trac
+            self.assertEqual(None, get_console_locale())
+            self.assertEqual(de, get_console_locale(self.env))
+            os.environ['LANG'] = '****'  # invalid locale
+            self.assertEqual(None, get_console_locale())
+            self.assertEqual(de, get_console_locale(self.env))
+            os.environ['LANG'] = 'en_US.utf-8'
+            self.assertEqual(en_US, get_console_locale())
+            self.assertEqual(en_US, get_console_locale(self.env))
+            os.environ['LC_MESSAGES'] = 'de_DE.utf-8'
+            self.assertEqual(de, get_console_locale())
+            self.assertEqual(de, get_console_locale(self.env))
+            os.environ['LC_ALL'] = 'en_US.utf-8'
+            self.assertEqual(en_US, get_console_locale())
+            self.assertEqual(en_US, get_console_locale(self.env))
+            os.environ['LANGUAGE'] = 'de_DE:en_US:en'
+            self.assertEqual(de, get_console_locale())
+            self.assertEqual(de, get_console_locale(self.env))
+
+        if not locales:  # compiled catalog is missing
+            unset_locale_envs()
+            self.assertEqual(None, get_console_locale(None, 'de_DE.UTF8'))
+            self.env.config.set('trac', 'default_language', 'de')
+            self.assertEqual(None, get_console_locale(self.env, None))
+            self.assertEqual(None, get_console_locale(self.env, 'C'))
+            self.assertEqual(None, get_console_locale(self.env,
+                                                      'de_DE.UTF8'))
+            os.environ['LANG'] = 'en_US.utf-8'
+            os.environ['LC_MESSAGES'] = 'de_DE.utf-8'
+            os.environ['LC_ALL'] = 'en_US.utf-8'
+            os.environ['LANGUAGE'] = 'de_DE:en_US'
+            self.assertEqual(en_US, get_console_locale())
+            self.assertEqual(en_US, get_console_locale(self.env))
 
     def _test_get_console_locale_without_babel(self):
+        os.environ['LANG'] = 'en_US.utf-8'
+        os.environ['LC_MESSAGES'] = 'de_DE.utf-8'
+        os.environ['LC_ALL'] = 'en_US.utf-8'
+        os.environ['LANGUAGE'] = 'de_DE:en_US'
         self.assertEqual(None, get_console_locale(None, 'en_US.UTF8'))
-        language = self.env.config.get('trac', 'default_language')
-        try:
-            self.env.config.set('trac', 'default_language', 'en_US')
-            self.assertEqual(None, get_console_locale(self.env, 'en_US.UTF8'))
-        finally:
-            self.env.config.set('trac', 'default_language', language)
+        self.env.config.set('trac', 'default_language', '')
+        self.assertEqual(None, get_console_locale(self.env, 'en_US.UTF8'))
+        self.assertEqual(None, get_console_locale(self.env))
+        self.env.config.set('trac', 'default_language', 'en_US')
+        self.assertEqual(None, get_console_locale(self.env, 'en_US.UTF8'))
+        self.assertEqual(None, get_console_locale(self.env))
 
     if has_babel:
         test_get_console_locale = _test_get_console_locale_with_babel
@@ -321,6 +382,30 @@ class TracadminTestCase(unittest.TestCase):
         self.assertExpectedResult(output)
         self.assertEqual('My Project', self.env.config.get('project', 'name'))
 
+    def test_config_set_complete_section(self):
+        """Tab complete on a configuration section."""
+        # Empty sections are included.
+        output = self._complete_command('config', 'set', '')
+        self.assertIn('components', output)
+        self.assertIn('project', output)
+        self.assertNotIn('foo-section', output)
+
+        # Sections not defined in registry are included.
+        self.env.config.set('foo-section', 'bar-option', '1')
+        output = self._complete_command('config', 'set', '')
+        self.assertIn('foo-section', output)
+
+    def test_config_set_complete_option(self):
+        """Tab complete on a configuration option."""
+        output = self._complete_command('config', 'set', 'project', '')
+        self.assertEqual(['admin', 'admin_trac_url', 'descr', 'footer', 'icon',
+                          'name', 'url'], sorted(output))
+
+        # Options not defined in registry are included.
+        self.env.config.set('project', 'bar-option', '1')
+        output = self._complete_command('config', 'set', 'project', '')
+        self.assertIn('bar-option', output)
+
     # Permission tests
 
     def test_permission_list_ok(self):
@@ -368,6 +453,25 @@ class TracadminTestCase(unittest.TestCase):
         self.assertEqual(0, rv, output2)
         self.assertExpectedResult(output + output2)
 
+    def test_permission_add_differs_from_action_by_casing(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test passes a permission that differs from an action by casing and
+        checks for the message.
+        """
+        rv, output = self._execute('permission add anonymous Trac_Admin')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_add_unknown_action(self):
+        """
+        Tests the 'permission add' command in trac-admin.  This particular
+        test tries granting NOT_A_PERM to a user. NOT_A_PERM does not exist
+        in the system."""
+        rv, output = self._execute('permission add joe NOT_A_PERM')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
     def test_permission_remove_one_action_ok(self):
         """
         Tests the 'permission remove' command in trac-admin.  This particular
@@ -440,6 +544,29 @@ class TracadminTestCase(unittest.TestCase):
         self.assertEqual(2, rv, output)
         self.assertExpectedResult(output)
 
+    def test_permission_remove_unknown_action(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing NOT_A_PERM from a user. NOT_A_PERM does not exist
+        in the system."""
+        rv, output = self._execute('permission remove joe NOT_A_PERM')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_permission_remove_unknown_action_granted(self):
+        """
+        Tests the 'permission remove' command in trac-admin.  This particular
+        test tries removing NOT_A_PERM from a user. NOT_A_PERM does not exist
+        in the system, but the user possesses the permission."""
+        self.env.db_transaction("""
+            INSERT INTO permission VALUES (%s, %s)
+        """, ('joe', 'NOT_A_PERM'))
+        rv, output = self._execute('permission remove joe NOT_A_PERM')
+        self.assertEqual(0, rv, output)
+        rv, output = self._execute('permission list')
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+
     def test_permission_export_ok(self):
         """
         Tests the 'permission export' command in trac-admin.  This particular
@@ -499,6 +626,34 @@ class TracadminTestCase(unittest.TestCase):
         self.assertEqual(0, rv, output)
         self.assertExpectedResult(output)
 
+    def test_component_add_complete_optional_owner_restrict_owner_false(self):
+        """Tests completion of the 'component add <component>' command with
+        [ticket] restrict_owner = false.
+        """
+        self._execute('config set ticket restrict_owner false')
+        self._execute('session add user1')
+        self._execute('session add user3')
+        self._execute('permission add user1 TICKET_MODIFY')
+        self._execute('permission add user2 TICKET_VIEW')
+        self._execute('permission add user3 TICKET_MODIFY')
+        output = self._complete_command('component', 'add',
+                                        'new_component', '')
+        self.assertEqual([], output)
+
+    def test_component_add_complete_optional_owner_restrict_owner_true(self):
+        """Tests completion of the 'component add <component>' command with
+        [ticket] restrict_owner = true.
+        """
+        self._execute('config set ticket restrict_owner true')
+        self._execute('session add user1')
+        self._execute('session add user3')
+        self._execute('permission add user1 TICKET_MODIFY')
+        self._execute('permission add user2 TICKET_VIEW')
+        self._execute('permission add user3 TICKET_MODIFY')
+        output = self._complete_command('component', 'add',
+                                        'new_component', '')
+        self.assertEqual(['user1', 'user3'], output)
+
     def test_component_add_error_already_exists(self):
         """
         Tests the 'component add' command in trac-admin.  This particular
@@ -547,6 +702,38 @@ class TracadminTestCase(unittest.TestCase):
         self.assertEqual(0, rv, output)
         self.assertExpectedResult(output)
 
+    def test_component_chown_complete_component(self):
+        """Tests completion of the 'component chown' command.
+        """
+        output = self._complete_command('component', 'chown', '')
+        self.assertEqual(['component1', 'component2'], output)
+
+    def test_component_chown_complete_owner_restrict_owner_false(self):
+        """Tests completion of the 'component chown <component>' command with
+        [ticket] restrict_owner = false.
+        """
+        self._execute('config set ticket restrict_owner false')
+        self._execute('session add user1')
+        self._execute('session add user3')
+        self._execute('permission add user1 TICKET_MODIFY')
+        self._execute('permission add user2 TICKET_VIEW')
+        self._execute('permission add user3 TICKET_MODIFY')
+        output = self._complete_command('component', 'chown', 'component1', '')
+        self.assertEqual([], output)
+
+    def test_component_chown_complete_owner_restrict_owner_true(self):
+        """Tests completion of the 'component chown <component>' command with
+        [ticket] restrict_owner = true.
+        """
+        self._execute('config set ticket restrict_owner true')
+        self._execute('session add user1')
+        self._execute('session add user3')
+        self._execute('permission add user1 TICKET_MODIFY')
+        self._execute('permission add user2 TICKET_VIEW')
+        self._execute('permission add user3 TICKET_MODIFY')
+        output = self._complete_command('component', 'chown', 'component1', '')
+        self.assertEqual(['user1', 'user3'], output)
+
     def test_component_chown_error_bad_component(self):
         """
         Tests the 'component chown' command in trac-admin.  This particular
@@ -1280,6 +1467,21 @@ class TracadminTestCase(unittest.TestCase):
         rv, output = self._execute('session list name00')
         self.assertExpectedResult(output)
 
+    def test_session_set_attr_default_handler(self):
+        _prep_session_table(self.env)
+        rv, output = \
+            self._execute('session set default_handler name00 SearchModule')
+        self.assertEqual(0, rv, output)
+        rv, output = self._execute('session list name00')
+        self.assertExpectedResult(output)
+
+    def test_session_set_attr_default_handler_invalid(self):
+        _prep_session_table(self.env)
+        rv, output = \
+            self._execute('session set default_handler name00 InvalidModule')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
     def test_session_set_attr_missing_attr(self):
         rv, output = self._execute('session set')
         self.assertEqual(2, rv, output)
@@ -1361,6 +1563,36 @@ class TracadminTestCase(unittest.TestCase):
         doc = self._get_command_help('session', 'purge')
         self.assertIn(u'"YYYY-MM-DDThh:mm:ss±hh:mm"', doc)
 
+    def test_changeset_add_no_repository_revision(self):
+        rv, output = self._execute('changeset added')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_changeset_add_no_revision(self):
+        rv, output = self._execute('changeset added repos')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_changeset_modify_no_repository_revision(self):
+        rv, output = self._execute('changeset modified')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_changeset_modify_no_revision(self):
+        rv, output = self._execute('changeset modified repos')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_changeset_add_invalid_repository(self):
+        rv, output = self._execute('changeset added repos 123')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+    def test_changeset_modify_invalid_repository(self):
+        rv, output = self._execute('changeset modified repos 123')
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
 
 class TracadminNoEnvTestCase(unittest.TestCase):
 
@@ -1435,18 +1667,189 @@ class TracAdminHelpMacroTestCase(unittest.TestCase):
         try:
             macro.expand_macro(None, None, 'copystatic')
             self.fail("MacroError not raised")
-        except MacroError, e:
+        except MacroError as e:
             self.assertEqual('Unknown trac-admin command "copystatic"',
                              unicode(e))
 
 
-def suite():
+class TracAdminComponentTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True, enable=('trac.*',),
+                                   disable=('trac.tests.*',))
+        self._admin = TracAdmin()
+        self._admin.env_set('', self.env)
+        self._orig = {
+            'ComponentMeta._components': ComponentMeta._components,
+            'ComponentMeta._registry': ComponentMeta._registry,
+            'ConfigSection.registry': ConfigSection.registry,
+            'Option.registry': Option.registry,
+        }
+        ComponentMeta._components = list(ComponentMeta._components)
+        ComponentMeta._registry = dict((interface, list(classes))
+                                       for interface, classes
+                                       in ComponentMeta._registry.iteritems())
+        ConfigSection.registry = {}
+        Option.registry = {}
+
+        class CompA(Component):
+            from trac.config import Option
+            opt1 = Option('compa', 'opt1', 1)
+            opt2 = Option('compa', 'opt2', 2)
+
+    def tearDown(self):
+        self.env = None
+        self._admin = None
+        ComponentMeta._components = self._orig['ComponentMeta._components']
+        ComponentMeta._registry = self._orig['ComponentMeta._registry']
+        ConfigSection.registry = self._orig['ConfigSection.registry']
+        Option.registry = self._orig['Option.registry']
+
+    def _execute(self, cmd, strip_trailing_space=True, input=None):
+        return execute_cmd(self._admin, cmd,
+                           strip_trailing_space=strip_trailing_space,
+                           input=input)
+
+    def test_config_component_enable(self):
+        self.env.config.save()
+        initial_file = copy.deepcopy(self.env.config.parser)
+
+        rv, output = self._execute('config set components '
+                                   'trac.admin.tests.console.* enabled')
+
+        self.assertEqual(0, rv, output)
+        self.assertFalse(initial_file.has_section('compa'))
+        self.assertIn('compa', self.env.config)
+        self.assertIn('1', self.env.config.parser.get('compa', 'opt1'))
+        self.assertIn('2', self.env.config.parser.get('compa', 'opt2'))
+
+
+class TracAdminInitenvTestCase(TracAdminTestCaseBase):
+
+    def setUp(self):
+        self.parent_dir = tempfile.mkdtemp()
+        self.env_path = os.path.join(self.parent_dir, 'trac')
+        self._admin = TracAdmin(self.env_path)
+
+    def tearDown(self):
+        if os.path.isfile(os.path.join(self.env_path, 'VERSION')):
+            self._admin.env.shutdown()
+        rmtree(self.parent_dir)
+
+    def test_config_argument(self):
+        """Options contained in file specified by the --config argument
+        are written to trac.ini.
+        """
+        config_file = os.path.join(self.parent_dir, 'config.ini')
+        create_file(config_file, textwrap.dedent("""\
+            [the-plugin]
+            option_a = 1
+            option_b = 2
+            [components]
+            the_plugin.* = enabled
+            [project]
+            name = project2
+            """))
+        rv, output = self._execute('initenv project1 sqlite:db/sqlite.db '
+                                   '--config=%s' % config_file)
+        env = Environment(self.env_path)
+        cfile = env.config.parser
+
+        self.assertEqual(0, rv, output)
+        self.assertEqual('1', cfile.get('the-plugin', 'option_a'))
+        self.assertEqual('2', cfile.get('the-plugin', 'option_b'))
+        self.assertEqual('enabled', cfile.get('components', 'the_plugin.*'))
+        self.assertEqual('project1', cfile.get('project', 'name'))
+        self.assertEqual('sqlite:db/sqlite.db', cfile.get('trac', 'database'))
+        for (section, name), option in \
+                Option.get_registry(env.compmgr).iteritems():
+            if (section, name) not in \
+                    (('trac', 'database'), ('project', 'name')):
+                self.assertEqual(option.default, cfile.get(section, name))
+
+    def test_config_argument_has_invalid_path(self):
+        """Exception is raised when --config argument is an invalid path."""
+        config_file = os.path.join(self.parent_dir, 'config.ini')
+        rv, output = self._execute('initenv project1 sqlite:db/sqlite.db '
+                                   '--config=%s' % config_file)
+
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output, {
+            'env_path': self.env_path,
+            'config_file': config_file,
+        })
+
+    def test_config_argument_has_invalid_value(self):
+        """Exception is raised when --config argument specifies a malformed
+        configuration file.
+        """
+        config_file = os.path.join(self.parent_dir, 'config.ini')
+        create_file(config_file, textwrap.dedent("""\
+            [the-plugin]
+            option_a = 1
+            [components
+            the_plugin.* = enabled
+            """))
+        rv, output = self._execute('initenv project1 sqlite:db/sqlite.db '
+                                   '--config=%s' % config_file)
+
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output, {
+            'env_path': self.env_path,
+            'config_file': config_file,
+        })
+
+
+class TracAdminDeployTestCase(TracAdminTestCaseBase):
+    """Tests for the trac-admin deploy command."""
+
+    def setUp(self):
+        self.env = Environment(path=tempfile.mkdtemp(), create=True)
+        self._admin = TracAdmin(self.env.path)
+        self._admin.env_set('', self.env)
+
+    def tearDown(self):
+        self.env.shutdown()  # really closes the db connections
+        rmtree(self.env.path)
+
+    def test_deploy(self):
+        """Deploy into valid target directory."""
+        target = os.path.join(self.env.path, 'www')
+        htdocs_dir = os.path.join(target, 'htdocs')
+
+        rv, output = self._execute('deploy %s' % target)
+
+        self.assertEqual(0, rv, output)
+        self.assertExpectedResult(output)
+        self.assertTrue(os.path.exists(os.path.join(target, 'cgi-bin')))
+        self.assertTrue(os.path.exists(htdocs_dir))
+        self.assertTrue(os.path.exists(os.path.join(htdocs_dir, 'common')))
+        self.assertTrue(os.path.exists(os.path.join(htdocs_dir, 'site')))
+
+    def test_deploy_to_invalid_target_raises_error(self):
+        """Running deploy with target directory equal to or below the source
+        directory raises AdminCommandError.
+        """
+        rv, output = self._execute('deploy %s' % self.env.htdocs_dir)
+
+        self.assertEqual(2, rv, output)
+        self.assertExpectedResult(output)
+
+
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(TracadminTestCase))
     suite.addTest(unittest.makeSuite(TracadminNoEnvTestCase))
     suite.addTest(unittest.makeSuite(TracAdminHelpMacroTestCase))
+    if __name__ == 'trac.admin.tests.console':
+        suite.addTest(unittest.makeSuite(TracAdminComponentTestCase))
+    else:
+        print("SKIP: trac.admin.tests.console.TracAdminComponentTestCase "
+              "(__name__ is not trac.admin.tests.console)")
+    suite.addTest(unittest.makeSuite(TracAdminInitenvTestCase))
+    suite.addTest(unittest.makeSuite(TracAdminDeployTestCase))
     return suite
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
