@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2014 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/.
+# history and logs, available at https://trac.edgewall.org/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
@@ -25,7 +25,7 @@ from genshi.builder import tag
 from trac.admin.api import IAdminPanelProvider
 from trac.core import *
 from trac.loader import get_plugin_info
-from trac.log import LOG_LEVELS
+from trac.log import LOG_LEVELS, LOG_LEVEL_ALIASES, LOG_LEVEL_ALIASES_MAP
 from trac.perm import PermissionSystem, IPermissionRequestor
 from trac.util.datefmt import all_timezones, pytz
 from trac.util.text import exception_to_unicode, \
@@ -40,6 +40,11 @@ from trac.web.chrome import add_notice, add_stylesheet, \
 from trac.wiki.formatter import format_to_html
 
 
+_valid_log_levels = set()
+_valid_log_levels.update(LOG_LEVELS)
+_valid_log_levels.update(LOG_LEVEL_ALIASES)
+
+
 class AdminModule(Component):
     """Web administration interface provider and panel manager."""
 
@@ -281,12 +286,12 @@ class LoggingAdminPanel(Component):
                     _("Unknown log type %(type)s", type=new_type),
                     _("Invalid log type")
                 )
-            new_file = req.args.get('log_file', 'trac.log')
+            new_file = req.args.get('log_file', log_file)
             if not new_file:
                 raise TracError(_("You must specify a log file"),
                                 _("Missing field"))
-            new_level = req.args.get('log_level')
-            if new_level not in LOG_LEVELS:
+            new_level = req.args.get('log_level', log_level)
+            if new_level not in _valid_log_levels:
                 raise TracError(
                     _("Unknown log level %(level)s", level=new_level),
                     _("Invalid log level"))
@@ -314,28 +319,22 @@ class LoggingAdminPanel(Component):
                     changed = True
                     log_type = new_type
 
-                if log_type == 'none':
-                    self.config.remove('logging', 'log_level')
+                if new_level != log_level:
+                    self.config.set('logging', 'log_level', new_level)
                     changed = True
-                else:
-                    if new_level != log_level:
-                        self.config.set('logging', 'log_level', new_level)
-                        changed = True
-                        log_level = new_level
+                    log_level = new_level
 
-                if log_type == 'file':
-                    if new_file != log_file:
-                        self.config.set('logging', 'log_file', new_file)
-                        changed = True
-                        log_file = new_file
-                else:
-                    self.config.remove('logging', 'log_file')
+                if new_file != log_file:
+                    self.config.set('logging', 'log_file', new_file)
                     changed = True
+                    log_file = new_file
 
             if changed:
                 _save_config(self.config, req, self.log),
             req.redirect(req.href.admin(cat, page))
 
+        log_level = LOG_LEVEL_ALIASES_MAP.get(log_level, log_level)
+
         data = {
             'type': log_type, 'types': log_types,
             'level': log_level, 'levels': LOG_LEVELS,
@@ -387,7 +386,6 @@ class PermissionAdminPanel(Component):
                     add_notice(req, _("The subject %(subject)s has been "
                                       "granted the permission %(action)s.",
                                       subject=subject, action=action))
-                    req.redirect(req.href.admin(cat, page))
                 else:
                     add_warning(req, _("The permission %(action)s was already "
                                        "granted to %(subject)s.",
@@ -414,7 +412,6 @@ class PermissionAdminPanel(Component):
                     add_notice(req, _("The subject %(subject)s has been added "
                                       "to the group %(group)s.",
                                       subject=subject, group=group))
-                    req.redirect(req.href.admin(cat, page))
                 else:
                     add_warning(req, _("The subject %(subject)s was already "
                                        "added to the group %(group)s.",
@@ -422,7 +419,7 @@ class PermissionAdminPanel(Component):
 
             # Copy permissions to subject
             elif req.args.get('copy') and subject and target:
-                req.perm.require('PERMISSION_GRANT')
+                req.perm('admin', 'general/perm').require('PERMISSION_GRANT')
 
                 subject_permissions = [i[1] for i in all_permissions
                                             if i[0] == subject and
@@ -468,7 +465,8 @@ class PermissionAdminPanel(Component):
                         perm.revoke_permission(subject, action)
                 add_notice(req, _("The selected permissions have been "
                                   "revoked."))
-                req.redirect(req.href.admin(cat, page))
+
+            req.redirect(req.href.admin(cat, page))
 
         return 'admin_perms.html', {
             'actions': all_actions,
