@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2005-2014 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
@@ -405,7 +405,7 @@ class DatabaseManager(Component):
                                         else data_or_callable
             for table, cols, vals in data:
                 db.executemany("INSERT INTO %s (%s) VALUES (%s)"
-                               % (table, ','.join(cols),
+                               % (db.quote(table), ','.join(cols),
                                   ','.join(['%s'] * len(cols))), vals)
 
     def reset_tables(self):
@@ -475,9 +475,10 @@ class DatabaseManager(Component):
                      in the SYSTEM table. Defaults to `database_version`,
                      which contains the database version for Trac.
         """
-        rows = self.env.db_query("""
-                SELECT value FROM system WHERE name=%s
-                """, (name,))
+        with self.env.db_query as db:
+            rows = db("""
+                SELECT value FROM {0} WHERE name=%s
+                """.format(db.quote('system')), (name,))
         return int(rows[0][0]) if rows else False
 
     def get_exceptions(self):
@@ -512,13 +513,15 @@ class DatabaseManager(Component):
         """
         current_database_version = self.get_database_version(name)
         if current_database_version is False:
-            self.env.db_transaction("""
-                    INSERT INTO system (name, value) VALUES (%s, %s)
-                    """, (name, version))
+            with self.env.db_transaction as db:
+                db("""
+                    INSERT INTO {0} (name, value) VALUES (%s, %s)
+                    """.format(db.quote('system')), (name, version))
         else:
-            self.env.db_transaction("""
-                    UPDATE system SET value=%s WHERE name=%s
-                    """, (version, name))
+            with self.env.db_transaction as db:
+                db("""
+                    UPDATE {0} SET value=%s WHERE name=%s
+                    """.format(db.quote('system')), (version, name))
             self.log.info("Upgraded %s from %d to %d",
                           name, current_database_version, version)
 
@@ -544,7 +547,7 @@ class DatabaseManager(Component):
                       name, dbver, version)
         return True
 
-    def upgrade(self, version, name='database_version', pkg=None):
+    def upgrade(self, version, name='database_version', pkg='trac.upgrades'):
         """Invokes `do_upgrade(env, version, cursor)` in module
         `"%s/db%i.py" % (pkg, version)`, for each required version upgrade.
 
@@ -655,7 +658,7 @@ def parse_connection_uri(db_str):
         section = tag.a("[trac]",
                         title=_("TracIni documentation"),
                         class_='trac-target-new',
-                        href='http://trac.edgewall.org/wiki/TracIni'
+                        href='https://trac.edgewall.org/wiki/TracIni'
                              '#trac-section')
         raise ConfigurationError(
             tag_("Database connection string is empty. Set the %(option)s "
@@ -753,5 +756,5 @@ def _doc_db_str():
     return tag.a(_("documentation"),
                  title=_("Database Connection Strings documentation"),
                  class_='trac-target-new',
-                 href='http://trac.edgewall.org/wiki/'
+                 href='https://trac.edgewall.org/wiki/'
                       'TracIni#DatabaseConnectionStrings')
