@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Jeff Weiss <trac@jeffweiss.org>
 # Copyright (C) 2006 Andres Salomon <dilinger@athenacr.com>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import re
@@ -38,10 +38,9 @@ try:
     import MySQLdb
     import MySQLdb.cursors
 except ImportError:
-    has_mysqldb = False
+    MySQLdb = None
     mysqldb_version = None
 else:
-    has_mysqldb = True
     mysqldb_version = get_pkginfo(MySQLdb).get('version', MySQLdb.__version__)
 
     class MySQLUnicodeCursor(MySQLdb.cursors.Cursor):
@@ -70,6 +69,10 @@ _type_map = {
 }
 
 
+def _quote(identifier):
+    return "`%s`" % identifier.replace('`', '``')
+
+
 class MySQLConnector(Component):
     """Database connector for MySQL version 4.1 and greater.
 
@@ -97,7 +100,6 @@ class MySQLConnector(Component):
 
     def __init__(self):
         self._mysql_version = None
-        self.error = None
 
     # ISystemInfoProvider methods
 
@@ -109,9 +111,7 @@ class MySQLConnector(Component):
     # IDatabaseConnector methods
 
     def get_supported_schemes(self):
-        if not has_mysqldb:
-            self.error = _("Cannot load Python bindings for MySQL")
-        yield ('mysql', -1 if self.error else 1)
+        yield 'mysql', 1
 
     def get_connection(self, path, log=None, user=None, password=None,
                        host=None, port=None, params={}):
@@ -132,12 +132,12 @@ class MySQLConnector(Component):
         cnx = self.get_connection(path, log, user, password, host, port,
                                   params)
         self._verify_variables(cnx)
-        utf8_size = self._utf8_size(cnx)
+        max_bytes = self._max_bytes(cnx)
         cursor = cnx.cursor()
         if schema is None:
             from trac.db_default import schema
         for table in schema:
-            for stmt in self.to_sql(table, utf8_size=utf8_size):
+            for stmt in self.to_sql(table, max_bytes=max_bytes):
                 self.log.debug(stmt)
                 cursor.execute(stmt)
         self._verify_table_status(cnx)
@@ -157,7 +157,7 @@ class MySQLConnector(Component):
                                   params)
         return bool(cnx.get_table_names())
 
-    def _utf8_size(self, cnx):
+    def _max_bytes(self, cnx):
         if cnx is None:
             connector, args = DatabaseManager(self.env).get_connector()
             cnx = connector.get_connection(**args)
@@ -167,18 +167,21 @@ class MySQLConnector(Component):
             charset = cnx.charset
         return 4 if charset == 'utf8mb4' else 3
 
-    def _collist(self, table, columns, utf8_size):
+    _max_key_length = 3072
+
+    def _collist(self, table, columns, max_bytes):
         """Take a list of columns and impose limits on each so that indexing
         works properly.
 
-        Some Versions of MySQL limit each index prefix to 1000 bytes total,
+        Some Versions of MySQL limit each index prefix to 3072 bytes total,
         with a max of 767 bytes per column.
         """
         cols = []
-        limit_col = 767 / utf8_size
-        limit = min(1000 / (utf8_size * len(columns)), limit_col)
+        limit_col = 767 / max_bytes
+        limit = min(self._max_key_length / (max_bytes * len(columns)),
+                    limit_col)
         for c in columns:
-            name = '`%s`' % c
+            name = _quote(c)
             table_col = filter((lambda x: x.name == c), table.columns)
             if len(table_col) == 1 and table_col[0].type.lower() == 'text':
                 if table_col[0].key_size is not None:
@@ -190,10 +193,10 @@ class MySQLConnector(Component):
             cols.append(name)
         return ','.join(cols)
 
-    def to_sql(self, table, utf8_size=None):
-        if utf8_size is None:
-            utf8_size = self._utf8_size(None)
-        sql = ['CREATE TABLE %s (' % table.name]
+    def to_sql(self, table, max_bytes=None):
+        if max_bytes is None:
+            max_bytes = self._max_bytes(None)
+        sql = ['CREATE TABLE %s (' % _quote(table.name)]
         coldefs = []
         for column in table.columns:
             ctype = column.type
@@ -203,19 +206,20 @@ class MySQLConnector(Component):
                 # Override the column type, as a text field cannot
                 # use auto_increment.
                 column.type = 'int'
-            coldefs.append('    `%s` %s' % (column.name, ctype))
+            coldefs.append('    %s %s' % (_quote(column.name), ctype))
         if len(table.key) > 0:
             coldefs.append('    PRIMARY KEY (%s)' %
                            self._collist(table, table.key,
-                                         utf8_size=utf8_size))
+                                         max_bytes=max_bytes))
         sql.append(',\n'.join(coldefs) + '\n)')
         yield '\n'.join(sql)
 
         for index in table.indices:
             unique = 'UNIQUE' if index.unique else ''
-            yield 'CREATE %s INDEX %s_%s_idx ON %s (%s);' % (unique, table.name,
-                  '_'.join(index.columns), table.name,
-                  self._collist(table, index.columns, utf8_size=utf8_size))
+            idxname = '%s_%s_idx' % (table.name, '_'.join(index.columns))
+            yield 'CREATE %s INDEX %s ON %s (%s)' % \
+                  (unique, _quote(idxname), _quote(table.name),
+                   self._collist(table, index.columns, max_bytes=max_bytes))
 
     def alter_column_types(self, table, columns):
         """Yield SQL statements altering the type of one or more columns of
@@ -241,7 +245,7 @@ class MySQLConnector(Component):
         db_params = db_prop.setdefault('params', {})
         db_name = os.path.basename(db_prop['path'])
 
-        args = [self.mysqldump_path]
+        args = [self.mysqldump_path, '--no-defaults']
         if 'host' in db_prop:
             args.extend(['-h', db_prop['host']])
         if 'port' in db_prop:
@@ -486,19 +490,25 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
             return table_names
         cursor = self.cursor()
         cursor.execute("""
-            SELECT table_name, auto_increment
-            FROM information_schema.tables
-            WHERE table_schema=%s""", (self.schema,))
-        for table, auto_increment in cursor.fetchall():
+            SELECT t.table_name,
+                   EXISTS (SELECT * FROM information_schema.columns AS c
+                           WHERE c.table_schema=t.table_schema
+                           AND c.table_name=t.table_name
+                           AND extra='auto_increment')
+            FROM information_schema.tables AS t
+            WHERE t.table_schema=%s
+            """, (self.schema,))
+        for table, has_autoinc in cursor.fetchall():
             table_names.append(table)
-            if auto_increment is None or auto_increment == 1:
+            quoted = self.quote(table)
+            if not has_autoinc:
                 # DELETE FROM is preferred to TRUNCATE TABLE, as the
-                # auto_increment is not used or it is 1.
-                cursor.execute("DELETE FROM %s" % table)
+                # auto_increment is not used.
+                cursor.execute("DELETE FROM %s" % quoted)
             else:
                 # TRUNCATE TABLE is preferred to DELETE FROM, as we
                 # need to reset the auto_increment in MySQL.
-                cursor.execute("TRUNCATE TABLE %s" % table)
+                cursor.execute("TRUNCATE TABLE %s" % quoted)
         return table_names
 
     def prefix_match(self):
@@ -508,7 +518,8 @@ class MySQLConnection(ConnectionBase, ConnectionWrapper):
         return self.like_escape(prefix) + '%'
 
     def quote(self, identifier):
-        return "`%s`" % identifier.replace('`', '``')
+        """Return the quoted identifier."""
+        return _quote(identifier)
 
     def update_sequence(self, cursor, table, column='id'):
         # MySQL handles sequence updates automagically
