@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011 Edgewall Software
+# Copyright (C) 2011-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 """Extra commands for setup.py.
 
@@ -19,8 +19,6 @@ we also modify the standard `distutils.command.build` and
 for compiling catalogs are issued upon install.
 """
 
-from __future__ import with_statement
-
 from StringIO import StringIO
 from itertools import izip
 import os
@@ -73,7 +71,7 @@ try:
         `cleandoc_keywords` option: a list of keywords to clean up the
         extracted messages with `cleandoc`.
         """
-        from trac.util.compat import cleandoc
+        from trac.util.text import cleandoc
 
         funcname = lineno = message_lineno = None
         kwargs_maps = func_kwargs_map = None
@@ -373,7 +371,7 @@ try:
             errors = [e for e in message.check(catalog)]
             try:
                 check_genshi_markup(catalog, message)
-            except TranslationError, e:
+            except TranslationError as e:
                 errors.append(e)
             return errors
 
@@ -411,7 +409,7 @@ try:
         try:
             return set([idx for idx, text in parse_msg(message)
                             if idx > 0])
-        except Exception, e:
+        except Exception as e:
             raise TranslationError('cannot parse message (%s: %s)' % \
                                    (e.__class__.__name__, unicode(e)))
 
