@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2014 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2007 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
@@ -27,7 +27,7 @@ from trac import db_default, log
 from trac.admin import AdminCommandError, IAdminCommandProvider
 from trac.cache import CacheManager, cached
 from trac.config import BoolOption, ChoiceOption, ConfigSection, \
-                        Configuration, Option, PathOption
+                        Configuration, IntOption, Option, PathOption
 from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracBaseError, TracError
 from trac.db.api import (DatabaseManager, QueryContextManager,
@@ -35,7 +35,7 @@ from trac.db.api import (DatabaseManager, QueryContextManager,
 from trac.loader import load_components
 from trac.log import logger_handler_factory
 from trac.util import arity, as_bool, copytree, create_file, get_pkginfo, \
-                      is_path_below, lazy, makedirs, read_file
+                      is_path_below, lazy, makedirs
 from trac.util.concurrency import threading
 from trac.util.text import exception_to_unicode, path_to_unicode, printerr, \
                            printout
@@ -170,6 +170,9 @@ class Environment(Component, ComponentManager):
         Plugins in that directory are loaded in addition to those in
         the directory of the environment `plugins`, with this one
         taking precedence.
+
+        Non-absolute paths are relative to the Environment `conf`
+        directory.
         """)
 
     base_url = Option('trac', 'base_url', '',
@@ -201,6 +204,13 @@ class Environment(Component, ComponentManager):
         your Trac instance is only accessible through HTTPS.
         """)
 
+    anonymous_session_lifetime = IntOption(
+        'trac', 'anonymous_session_lifetime', '90',
+        """Lifetime of the anonymous session, in days.
+
+        Set the option to 0 to disable purging old anonymous sessions.
+        (''since 1.0.17'')""")
+
     project_name = Option('project', 'name', 'My Project',
         """Name of the project.""")
 
@@ -226,8 +236,8 @@ class Environment(Component, ComponentManager):
 
     project_footer = Option('project', 'footer',
                             N_('Visit the Trac open source project at<br />'
-                               '<a href="http://trac.edgewall.org/">'
-                               'http://trac.edgewall.org/</a>'),
+                               '<a href="https://trac.edgewall.org/">'
+                               'https://trac.edgewall.org/</a>'),
         """Page footer text (right-aligned).""")
 
     project_icon = Option('project', 'icon', 'common/trac.ico',
@@ -408,13 +418,15 @@ class Environment(Component, ComponentManager):
         """Verify that the provided path points to a valid Trac environment
         directory."""
         try:
-            tag = read_file(os.path.join(self.path, 'VERSION')).splitlines()[0]
-            if tag != _VERSION:
-                raise Exception(_("Unknown Trac environment type '%(type)s'",
-                                  type=tag))
+            with open(os.path.join(self.path, 'VERSION')) as f:
+                tag = f.readline().rstrip('\n')
         except Exception as e:
             raise TracError(_("No Trac environment found at %(path)s\n"
-                              "%(e)s", path=self.path, e=e))
+                              "%(e)s",
+                              path=self.path, e=exception_to_unicode(e)))
+        if tag != _VERSION:
+            raise TracError(_("Unknown Trac environment type '%(type)s'",
+                              type=tag))
 
     @lazy
     def db_exc(self):
@@ -573,7 +585,7 @@ class Environment(Component, ComponentManager):
         create_file(os.path.join(self.path, 'VERSION'), _VERSION + '\n')
         create_file(os.path.join(self.path, 'README'),
                     'This directory contains a Trac environment.\n'
-                    'Visit http://trac.edgewall.org/ for more information.\n')
+                    'Visit https://trac.edgewall.org/ for more information.\n')
 
         # Setup the default configuration
         os.mkdir(self.conf_dir)
@@ -850,7 +862,7 @@ class Environment(Component, ComponentManager):
     def abs_href(self):
         """The application URL"""
         if not self.base_url:
-            self.log.warn("base_url option not set in configuration, "
+            self.log.warn("[trac] base_url option not set in configuration, "
                           "generated links may be incorrect")
         return Href(self.base_url)
 
@@ -873,8 +885,7 @@ class EnvironmentSetup(Component):
         return DatabaseManager(self.env).needs_upgrade(db_default.db_version)
 
     def upgrade_environment(self):
-        DatabaseManager(self.env).upgrade(db_default.db_version,
-                                          pkg='trac.upgrades')
+        DatabaseManager(self.env).upgrade(db_default.db_version)
         self._update_sample_config()
 
     # Internal methods
@@ -884,14 +895,13 @@ class EnvironmentSetup(Component):
         if not os.path.isfile(filename):
             return
         config = Configuration(filename)
-        for (section, name), option in Option.get_registry().iteritems():
-            config.set(section, name, option.dumps(option.default))
+        config.set_defaults()
         try:
             config.save()
             self.log.info("Wrote sample configuration file with the new "
                           "settings and their default values: %s",
                           filename)
-        except IOError as e:
+        except EnvironmentError as e:
             self.log.warn("Couldn't write sample configuration file (%s)", e,
                           exc_info=True)
 
@@ -944,8 +954,8 @@ def open_environment(env_path=None, use_cache=False):
             env.log.error("Exception caught while checking for upgrade: %s",
                           exception_to_unicode(e, traceback=True))
         if needs_upgrade:
-            raise TracError(_('The Trac Environment needs to be upgraded.\n\n'
-                              'Run \'trac-admin "%(path)s" upgrade\'',
+            raise TracError(_('The Trac Environment needs to be upgraded. '
+                              'Run:\n\n  trac-admin "%(path)s" upgrade',
                               path=env_path))
 
     return env
@@ -1047,7 +1057,8 @@ class EnvironmentAdmin(Component):
 
         # Bogus statement to lock the database while copying files
         with self.env.db_transaction as db:
-            db("UPDATE system SET name=NULL WHERE name IS NULL")
+            db("UPDATE " + db.quote('system') +
+               " SET name=NULL WHERE name IS NULL")
 
             printout(_("Hotcopying %(src)s to %(dst)s ...",
                        src=path_to_unicode(self.env.path),
