@@ -1,30 +1,32 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2013 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import doctest
 import unittest
 from StringIO import StringIO
 
-import trac.tests.compat
-from trac.core import *
-from trac.test import EnvironmentStub
-from trac.mimeview import api
-from trac.mimeview.api import get_mimetype, IContentConverter, Mimeview, \
-                              _group_lines
 from genshi import Stream, Namespace
 from genshi.core import Attrs, TEXT, START, END
 from genshi.input import HTMLParser
 
+import trac.tests.compat
+from trac.core import Component, implements
+from trac.test import EnvironmentStub, MockRequest
+from trac.mimeview import api
+from trac.mimeview.api import get_mimetype, IContentConverter, Mimeview, \
+                              _group_lines
+from trac.web.api import RequestDone
+
 
 class GetMimeTypeTestCase(unittest.TestCase):
 
@@ -98,9 +100,9 @@ class MimeviewTestCase(unittest.TestCase):
 
         mimeview = Mimeview(self.env)
         conversions = mimeview.get_supported_conversions('text/x-sample')
-        self.assertEqual(Converter0(self.env), conversions[0][-1])
-        self.assertEqual(Converter1(self.env), conversions[1][-1])
-        self.assertEqual(Converter2(self.env), conversions[2][-1])
+        self.assertEqual(Converter0(self.env), conversions[0].converter)
+        self.assertEqual(Converter1(self.env), conversions[1].converter)
+        self.assertEqual(Converter2(self.env), conversions[2].converter)
 
 class GroupLinesTestCase(unittest.TestCase):
 
@@ -133,7 +135,7 @@ class GroupLinesTestCase(unittest.TestCase):
 
     def test_empty_text_stream(self):
         """
-        http://trac.edgewall.org/ticket/4336
+        https://trac.edgewall.org/ticket/4336
         """
         input = [(TEXT, "", (None, -1, -1))]
         lines = list(_group_lines(input))
@@ -151,7 +153,7 @@ class GroupLinesTestCase(unittest.TestCase):
 
     def test_empty_text_in_span(self):
         """
-        http://trac.edgewall.org/ticket/4336
+        https://trac.edgewall.org/ticket/4336
         """
         ns = Namespace('http://www.w3.org/1999/xhtml')
         input = [(START, (ns.span, Attrs([])), (None, -1, -1)),
@@ -208,13 +210,132 @@ class GroupLinesTestCase(unittest.TestCase):
             self.assertEqual(a.render('html'), b)
 
 
-def suite():
+class TestMimeviewConverter(Component):
+
+    implements(IContentConverter)
+
+    in_mimetype = __module__ + '.' + __name__
+
+    def get_supported_conversions(self):
+        yield ('text', self.__module__, 'txt', self.in_mimetype, 'text/plain',
+               8)
+
+    def convert_content(self, req, mimetype, content, key):
+        if content == 'iterable-bytes':
+            def fn_bytes():
+                for idx in xrange(256):
+                    yield 'b' * 256
+            return fn_bytes(), 'text/plain'
+        if content == 'iterable-unicode':
+            def fn_unicode():
+                for idx in xrange(0x10000):
+                    yield u'ü'
+            return fn_unicode(), 'text/plain'
+        if content == 'bytes':
+            return 'a' * 0x10000, 'text/plain'
+        if content == 'unicode':
+            return u'Ü' * 0x10000, 'text/plain'
+
+
+class MimeviewConverterTestCase(unittest.TestCase):
+
+    in_mimetype = TestMimeviewConverter.in_mimetype
+
+    def setUp(self):
+        self.env = EnvironmentStub(enable=['trac.*', TestMimeviewConverter])
+
+    def tearDown(self):
+        pass
+
+    def _test_convert_content(self, expected, content, iterable):
+        mimeview = Mimeview(self.env)
+        output = mimeview.convert_content(MockRequest(self.env),
+                                          self.in_mimetype,
+                                          content, 'text', iterable=iterable)
+        if iterable:
+            self.assertNotIn(type(output[0]), (str, unicode))
+            self.assertEqual(expected, ''.join(output[0]))
+        else:
+            self.assertEqual(type(expected), type(output[0]))
+            self.assertEqual(expected, output[0])
+        self.assertEqual('text/plain', output[1])
+        self.assertEqual('txt', output[2])
+
+    def test_convert_content_iterable_bytes(self):
+        self._test_convert_content('b' * 0x10000, 'iterable-bytes', False)
+
+    def test_convert_content_iterable_unicode(self):
+        self._test_convert_content(u'ü' * 0x10000, 'iterable-unicode', False)
+
+    def test_convert_content_bytes(self):
+        self._test_convert_content('a' * 0x10000, 'bytes', False)
+
+    def test_convert_content_unicode(self):
+        self._test_convert_content(u'Ü' * 0x10000, 'unicode', False)
+
+    def test_convert_content_iterable_bytes_iterable(self):
+        self._test_convert_content('b' * 0x10000, 'iterable-bytes', True)
+
+    def test_convert_content_iterable_unicode_iterable(self):
+        self._test_convert_content(u'ü' * 0x10000, 'iterable-unicode', True)
+
+    def test_convert_content_bytes_iterable(self):
+        self._test_convert_content('a' * 0x10000, 'bytes', True)
+
+    def test_convert_content_unicode_iterable(self):
+        self._test_convert_content(u'Ü' * 0x10000, 'unicode', True)
+
+    def _test_send_converted(self, expected, content, use_chunked_encoding):
+        self.env.config.set('trac', 'use_chunked_encoding',
+                            'true' if use_chunked_encoding else 'false')
+        mimeview = Mimeview(self.env)
+        req = MockRequest(self.env)
+        self.assertRaises(RequestDone, mimeview.send_converted, req,
+                          self.in_mimetype, content, 'text')
+        result = req.response_sent.getvalue()
+        if use_chunked_encoding:
+            self.assertNotIn('Content-Length', req.headers_sent)
+        else:
+            self.assertIn('Content-Length', req.headers_sent)
+            self.assertEqual(str(len(expected)),
+                             req.headers_sent['Content-Length'])
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(set(expected), set(result))
+        self.assertEqual(expected, result)
+
+    def test_send_converted_iterable_bytes(self):
+        self._test_send_converted('b' * 0x10000, 'iterable-bytes', False)
+
+    def test_send_converted_iterable_unicode(self):
+        self._test_send_converted('ü' * 0x10000, 'iterable-unicode', False)
+
+    def test_send_converted_bytes(self):
+        self._test_send_converted('a' * 0x10000, 'bytes', False)
+
+    def test_send_converted_unicode(self):
+        self._test_send_converted('Ü' * 0x10000, 'unicode', False)
+
+    def test_send_converted_iterable_bytes_chunked(self):
+        self._test_send_converted('b' * 0x10000, 'iterable-bytes', True)
+
+    def test_send_converted_iterable_unicode_chunked(self):
+        self._test_send_converted('ü' * 0x10000, 'iterable-unicode', True)
+
+    def test_send_converted_bytes_chunked(self):
+        self._test_send_converted('a' * 0x10000, 'bytes', True)
+
+    def test_send_converted_unicode_chunked(self):
+        self._test_send_converted('Ü' * 0x10000, 'unicode', True)
+
+
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(api))
     suite.addTest(unittest.makeSuite(GetMimeTypeTestCase))
     suite.addTest(unittest.makeSuite(MimeviewTestCase))
     suite.addTest(unittest.makeSuite(GroupLinesTestCase))
+    suite.addTest(unittest.makeSuite(MimeviewConverterTestCase))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
