@@ -1,26 +1,26 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import csv
 import os
-from time import time
+from itertools import groupby
 
 from trac.admin import AdminCommandError, IAdminCommandProvider, get_dir_list
 from trac.cache import cached
@@ -28,6 +28,7 @@ from trac.config import ExtensionOption, OrderedExtensionsOption
 from trac.core import *
 from trac.resource import Resource, get_resource_name
 from trac.util import file_or_std
+from trac.util.datefmt import time_now
 from trac.util.text import path_to_unicode, print_table, printout, \
                            stream_encoding, to_unicode, wrap
 from trac.util.translation import _, N_
@@ -36,17 +37,21 @@ __all__ = ['IPermissionRequestor', 'IPermissionStore', 'IPermissionPolicy',
            'IPermissionGroupProvider', 'PermissionError', 'PermissionSystem']
 
 
-class PermissionError(StandardError):
-    """Insufficient permissions to complete the operation"""
+class PermissionError(StandardError, TracBaseError):
+    """Insufficient permissions to perform the operation.
+
+    :since 1.0.5: the `msg` attribute is deprecated and will be removed in
+                  1.3.1. Use the `message` property instead.
+    """
 
     title = N_("Forbidden")
 
-    def __init__ (self, action=None, resource=None, env=None, msg=None):
+    def __init__(self, action=None, resource=None, env=None, msg=None):
         self.action = action
         self.resource = resource
         self.env = env
         if self.action:
-            if self.resource:
+            if self.resource and self.resource.id:
                 msg = _("%(perm)s privileges are required to perform "
                         "this operation on %(resource)s. You don't have the "
                         "required permissions.",
@@ -59,7 +64,11 @@ class PermissionError(StandardError):
         elif msg is None:
             msg = _("Insufficient privileges to perform this operation.")
         self.msg = msg
-        StandardError.__init__(self, msg)
+        super(PermissionError, self).__init__(msg)
+
+    @property
+    def message(self):
+        return self.args[0]
 
 
 class IPermissionRequestor(Interface):
@@ -168,8 +177,7 @@ class DefaultPermissionStore(Component):
     group_providers = ExtensionPoint(IPermissionGroupProvider)
 
     def get_user_permissions(self, username):
-        """Retrieve the permissions for the given user and return them in a
-        dictionary.
+        """Retrieve a list of permissions for the given user.
 
         The permissions are stored in the database as (username, action)
         records. There's simple support for groups by using lowercase names for
@@ -221,7 +229,6 @@ class DefaultPermissionStore(Component):
         formatted tuples."""
         return self._all_permissions
 
-
     @cached
     def _all_permissions(self):
         return [(username, action) for username, action in
@@ -274,16 +281,16 @@ class DefaultPermissionPolicy(Component):
 
     def __init__(self):
         self.permission_cache = {}
-        self.last_reap = time()
+        self.last_reap = time_now()
 
     # IPermissionPolicy methods
 
     def check_permission(self, action, username, resource, perm):
-        now = time()
+        now = time_now()
 
         if now - self.last_reap > self.CACHE_REAP_TIME:
             self.permission_cache = {}
-            self.last_reap = time()
+            self.last_reap = time_now()
 
         timestamp, permissions = self.permission_cache.get(username, (0, None))
 
@@ -297,7 +304,6 @@ class DefaultPermissionPolicy(Component):
         return action in permissions or None
 
 
-
 class PermissionSystem(Component):
     """Permission management sub-system."""
 
@@ -314,14 +320,11 @@ class PermissionSystem(Component):
 
     policies = OrderedExtensionsOption('trac', 'permission_policies',
         IPermissionPolicy,
-        'DefaultPermissionPolicy, LegacyAttachmentPolicy',
+        'ReadonlyWikiPolicy, DefaultPermissionPolicy, LegacyAttachmentPolicy',
         False,
-        """List of components implementing `IPermissionPolicy`, in the order in
-        which they will be applied. These components manage fine-grained access
-        control to Trac resources.
-        Defaults to the DefaultPermissionPolicy (pre-0.11 behavior) and
-        LegacyAttachmentPolicy (map ATTACHMENT_* permissions to realm specific
-        ones)""")
+        """List of components implementing `IPermissionPolicy`, in the order
+        in which they will be applied. These components manage fine-grained
+        access control to Trac resources.""")
 
     # Number of seconds a cached user permission set is valid for.
     CACHE_EXPIRY = 5
@@ -330,31 +333,40 @@ class PermissionSystem(Component):
 
     def __init__(self):
         self.permission_cache = {}
-        self.last_reap = time()
+        self.last_reap = time_now()
 
     # Public API
 
     def grant_permission(self, username, action):
-        """Grant the user with the given name permission to perform to specified
-        action."""
+        """Grant the user with the given name permission to perform to
+        specified action."""
         if action.isupper() and action not in self.get_actions():
             raise TracError(_('%(name)s is not a valid action.', name=action))
+        elif not action.isupper() and action.upper() in self.get_actions():
+            raise TracError(_("Permission %(name)s differs from a defined "
+                              "action by casing only, which is not allowed.",
+                              name=action))
 
         self.store.grant_permission(username, action)
 
     def revoke_permission(self, username, action):
-        """Revokes the permission of the specified user to perform an action."""
+        """Revokes the permission of the specified user to perform an
+        action."""
         self.store.revoke_permission(username, action)
 
-    def get_actions_dict(self):
+    def get_actions_dict(self, skip=None):
         """Get all actions from permission requestors as a `dict`.
 
         The keys are the action names. The values are the additional actions
         granted by each action. For simple actions, this is an empty list.
         For meta actions, this is the list of actions covered by the action.
+
+        :since 1.0.17: added `skip` argument.
         """
         actions = {}
         for requestor in self.requestors:
+            if requestor is skip:
+                continue
             for action in requestor.get_permission_actions() or []:
                 if isinstance(action, tuple):
                     actions.setdefault(action[0], []).extend(action[1])
@@ -375,6 +387,33 @@ class PermissionSystem(Component):
                     actions.add(action)
         return list(actions)
 
+    def get_groups_dict(self):
+        """Get all groups as a `dict`.
+
+        The keys are the group names. The values are the group members.
+
+        :since: 1.1.3
+        """
+        groups = sorted((p for p in self.get_all_permissions()
+                           if not p[1].isupper()), key=lambda p: p[1])
+
+        return dict((k, sorted(i[0] for i in list(g)))
+                    for k, g in groupby(groups, key=lambda p: p[1]))
+
+    def get_users_dict(self):
+        """Get all users as a `dict`.
+
+        The keys are the user names. The values are the actions possessed
+        by the user.
+
+        :since: 1.1.3
+        """
+        perms = sorted((p for p in self.get_all_permissions()
+                          if p[1].isupper()), key=lambda p: p[0])
+
+        return dict((k, sorted(i[1] for i in list(g)))
+                    for k, g in groupby(perms, key=lambda p: p[0]))
+
     def get_user_permissions(self, username=None):
         """Return the permissions of the specified user.
 
@@ -409,7 +448,7 @@ class PermissionSystem(Component):
 
         Users are returned as a list of user names.
         """
-        now = time()
+        now = time_now()
         if now - self.last_reap > self.CACHE_REAP_TIME:
             self.permission_cache = {}
             self.last_reap = now
@@ -448,7 +487,8 @@ class PermissionSystem(Component):
             expand_action(a)
         return expanded_actions
 
-    def check_permission(self, action, username=None, resource=None, perm=None):
+    def check_permission(self, action, username=None, resource=None,
+                         perm=None):
         """Return True if permission to perform action for the given resource
         is allowed."""
         if username is None:
@@ -459,10 +499,10 @@ class PermissionSystem(Component):
             decision = policy.check_permission(action, username, resource,
                                                perm)
             if decision is not None:
-                if decision is False:
-                    self.log.debug("%s denies %s performing %s on %r",
-                                   policy.__class__.__name__, username,
-                                   action, resource)
+                self.log.debug("%s %s %s performing %s on %r",
+                               policy.__class__.__name__,
+                               'allows' if decision else 'denies',
+                               username, action, resource)
                 return decision
         self.log.debug("No policy allowed %s performing %s on %r",
                        username, action, resource)
@@ -472,14 +512,9 @@ class PermissionSystem(Component):
 
     def get_permission_actions(self):
         """Implement the global `TRAC_ADMIN` meta permission.
-
-        Implements also the `EMAIL_VIEW` permission which allows for
-        showing email addresses even if `[trac] show_email_addresses`
-        is `false`.
         """
         actions = self.get_actions(skip=self)
-        actions.append('EMAIL_VIEW')
-        return [('TRAC_ADMIN', actions), 'EMAIL_VIEW']
+        return [('TRAC_ADMIN', actions)]
 
 
 class PermissionCache(object):
@@ -541,7 +576,8 @@ class PermissionCache(object):
             'WIKI_VIEW' in perm(resource)
 
         """
-        resource = Resource(realm_or_resource, id, version)
+        resource = Resource(realm_or_resource, id, version) \
+                   if realm_or_resource else None
         if resource and self._resource and resource == self._resource:
             return self
         else:
@@ -560,6 +596,8 @@ class PermissionCache(object):
             cache_decision, cache_resource = cached
             if resource == cache_resource:
                 return cache_decision
+        # Avoid recursion in policies that call has_permission.
+        self._cache[key] = (False, resource)
         perm = self
         if resource is not self._resource:
             perm = PermissionCache(self.env, self.username, resource,
@@ -581,14 +619,6 @@ class PermissionCache(object):
                 raise PermissionError(msg=message)
     assert_permission = require
 
-    def permissions(self):
-        """Deprecated (but still used by the HDF compatibility layer)"""
-        self.env.log.warning("perm.permissions() is deprecated and "
-                             "is only present for HDF compatibility")
-        perm = PermissionSystem(self.env)
-        actions = perm.get_user_permissions(self.username)
-        return [action for action in actions if action in self]
-
 
 class PermissionAdmin(Component):
     """trac-admin command provider for permission system administration."""
@@ -656,14 +686,14 @@ class PermissionAdmin(Component):
             rows = permsys.get_all_permissions()
         rows.sort()
         print_table(rows, [_('User'), _('Action')])
-        print
+        print()
         printout(_("Available actions:"))
         actions = permsys.get_actions()
         actions.sort()
         text = ', '.join(actions)
         printout(wrap(text, initial_indent=' ', subsequent_indent=' ',
                       linesep='\n'))
-        print
+        print()
 
     def _do_add(self, user, *actions):
         permsys = PermissionSystem(self.env)
@@ -676,6 +706,8 @@ class PermissionAdmin(Component):
             except self.env.db_exc.IntegrityError:
                 printout(_("The user %(user)s already has permission "
                            "%(action)s.", user=user, action=action))
+            except TracError as e:
+                raise AdminCommandError(e)
 
     def _do_remove(self, user, *actions):
         permsys = PermissionSystem(self.env)
@@ -693,10 +725,12 @@ class PermissionAdmin(Component):
                             "%(user)s. The permission is granted through "
                             "a meta-permission or group.", action=action,
                             user=user)
-                else:
+                elif action in permsys.get_actions():
                     msg = _("Cannot remove permission %(action)s for user "
                             "%(user)s. The user has not been granted the "
                             "permission.", action=action, user=user)
+                else:
+                    msg = _("%(name)s is not a valid action.", name=action)
                 raise AdminCommandError(msg)
 
     def _do_export(self, filename=None):
@@ -710,7 +744,7 @@ class PermissionAdmin(Component):
                     actions = sorted(self.get_user_perms(user))
                     writer.writerow([s.encode(encoding, 'replace')
                                      for s in [user] + actions])
-        except IOError, e:
+        except IOError as e:
             raise AdminCommandError(
                 _("Cannot export to %(filename)s: %(error)s",
                   filename=path_to_unicode(filename or 'stdout'),
@@ -740,12 +774,12 @@ class PermissionAdmin(Component):
                     old_actions = self.get_user_perms(user)
                     for action in set(actions) - set(old_actions):
                         permsys.grant_permission(user, action)
-        except csv.Error, e:
+        except csv.Error as e:
             raise AdminCommandError(
                 _("Cannot import from %(filename)s line %(line)d: %(error)s ",
                   filename=path_to_unicode(filename or 'stdin'),
                   line=reader.line_num, error=e))
-        except IOError, e:
+        except IOError as e:
             raise AdminCommandError(
                 _("Cannot import from %(filename)s: %(error)s",
                   filename=path_to_unicode(filename or 'stdin'),
