@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2013 Edgewall Software
+# Copyright (C) 2004-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 
@@ -169,6 +169,29 @@ class PermissionSystemTestCase(unittest.TestCase):
     def tearDown(self):
         self.env.reset_db()
 
+    def test_get_actions(self):
+        tpr_perms = ['TEST_ADMIN', 'TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY']
+        all_perms = tpr_perms + ['TRAC_ADMIN']
+        self.assertEqual(all_perms, sorted(self.perm.get_actions()))
+        self.assertEqual(tpr_perms,
+                         sorted(self.perm.get_actions(skip=self.perm)))
+
+    def test_get_actions_dict(self):
+        self.assertEqual({
+            'TEST_ADMIN': ['TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY'],
+            'TEST_CREATE': [],
+            'TEST_DELETE': [],
+            'TEST_MODIFY': [],
+            'TRAC_ADMIN': ['TEST_CREATE', 'TEST_DELETE', 'TEST_ADMIN',
+                           'TEST_MODIFY'],
+        }, self.perm.get_actions_dict())
+        self.assertEqual({
+            'TEST_ADMIN': ['TEST_CREATE', 'TEST_DELETE', 'TEST_MODIFY'],
+            'TEST_CREATE': [],
+            'TEST_DELETE': [],
+            'TEST_MODIFY': [],
+        }, self.perm.get_actions_dict(skip=self.perm))
+
     def test_all_permissions(self):
         self.assertEqual({'TRAC_ADMIN': True, 'TEST_CREATE': True,
                           'TEST_DELETE': True, 'TEST_MODIFY': True,
