@@ -5,29 +5,27 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Brian Meeker <meeker.brian@gmail.com>
 
-from __future__ import with_statement
-
 import re
-from datetime import datetime
 
 from genshi.builder import tag
 
 from trac.core import *
+from trac.notification.api import NotificationSystem
 from trac.perm import IPermissionRequestor
 from trac.ticket import TicketSystem, Ticket
-from trac.ticket.notification import BatchTicketNotifyEmail
-from trac.util.datefmt import datetime_now, utc
+from trac.ticket.notification import BatchTicketChangeEvent
+from trac.util.datefmt import datetime_now, parse_date, user_time, utc
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
-from trac.web import IRequestHandler
+from trac.web.api import IRequestFilter, IRequestHandler, HTTPBadRequest
 from trac.web.chrome import add_warning, add_script_data
 
 
@@ -41,9 +39,11 @@ class BatchModifyModule(Component):
     modify.
     """
 
-    implements(IPermissionRequestor, IRequestHandler)
+    implements(IPermissionRequestor, IRequestFilter, IRequestHandler)
 
-    list_separator_re =  re.compile(r'[;\s,]+')
+    is_valid_default_handler = False
+
+    list_separator_re = re.compile(r'[;\s,]+')
     list_connector_string = ', '
 
     # IRequestHandler methods
@@ -52,19 +52,38 @@ class BatchModifyModule(Component):
         return req.path_info == '/batchmodify'
 
     def process_request(self, req):
+        if req.method != 'POST':
+            raise HTTPBadRequest(_("Invalid request arguments."))
         req.perm.assert_permission('TICKET_BATCH_MODIFY')
 
         comment = req.args.get('batchmod_value_comment', '')
         action = req.args.get('action')
 
-        new_values = self._get_new_ticket_values(req)
-        selected_tickets = self._get_selected_tickets(req)
+        try:
+            new_values = self._get_new_ticket_values(req)
+        except TracError as e:
+            new_values = None
+            add_warning(req, tag_("The changes could not be saved: "
+                                  "%(message)s", message=to_unicode(e)))
 
-        self._save_ticket_changes(req, selected_tickets,
-                                  new_values, comment, action)
+        if new_values is not None:
+            selected_tickets = self._get_selected_tickets(req)
+            self._save_ticket_changes(req, selected_tickets,
+                                      new_values, comment, action)
 
-        #Always redirect back to the query page we came from.
-        req.redirect(req.session['query_href'])
+        # Always redirect back to the query page we came from
+        req.redirect(req.args.get('query_href') or req.href.query())
+
+    # IRequestFilter methods
+
+    def pre_process_request(self, req, handler):
+        return handler
+
+    def post_process_request(self, req, template, data, content_type):
+        if req.path_info == '/query' and data is not None and \
+                'TICKET_BATCH_MODIFY' in req.perm('ticket'):
+            self.add_template_data(req, data, data['tickets'])
+        return template, data, content_type
 
     # IPermissionRequestor methods
 
@@ -79,13 +98,20 @@ class BatchModifyModule(Component):
         for field in TicketSystem(self.env).get_ticket_fields():
             name = field['name']
             if name not in ('id', 'resolution', 'status', 'owner', 'time',
-                            'changetime', 'summary', 'reporter',
-                            'description') and field['type'] != 'textarea':
+                            'changetime', 'summary', 'description') + \
+                           (('reporter',) if 'TICKET_ADMIN' not in req.perm
+                                          else ()) \
+                    and field['type'] != 'textarea':
                 value = req.args.get('batchmod_value_' + name)
                 if value is not None:
-                    values[name] = value
+                    values[name] = self._parse_field_value(req, field, value)
         return values
 
+    def _parse_field_value(self, req, field, value):
+        if field['type'] == 'time':
+            return user_time(req, parse_date, value)
+        return value
+
     def _get_selected_tickets(self, req):
         """The selected tickets will be a comma separated list
         in the request arguments."""
@@ -106,28 +132,39 @@ class BatchModifyModule(Component):
             {'name': _("set to"), 'value': "="},
         ]
         add_script_data(req, batch_list_modes=batch_list_modes,
-                             batch_list_properties=self._get_list_fields())
+                        batch_list_properties=self._get_list_fields())
 
     def _get_list_fields(self):
         return [f['name']
                 for f in TicketSystem(self.env).get_ticket_fields()
                 if f['type'] == 'text' and f.get('format') == 'list']
 
-    def _get_action_controls(self, req, tickets):
-        action_controls = []
-        ts = TicketSystem(self.env)
-        tickets_by_action = {}
+    def _get_action_controls(self, req, ticket_data):
+        tickets = [Ticket(self.env, t['id']) for t in ticket_data]
+        action_weights = {}
+        action_tickets = {}
         for t in tickets:
-            ticket = Ticket(self.env, t['id'])
-            actions = ts.get_available_actions(req, ticket)
-            for action in actions:
-                tickets_by_action.setdefault(action, []).append(ticket)
-        sorted_actions = sorted(set(tickets_by_action.keys()))
+            for ctrl in TicketSystem(self.env).action_controllers:
+                for weight, action in ctrl.get_ticket_actions(req, t) or []:
+                    if action in action_weights:
+                        action_weights[action] = max(action_weights[action],
+                                                     weight)
+                        action_tickets[action].append(t)
+                    else:
+                        action_weights[action] = weight
+                        action_tickets[action] = [t]
+
+        sorted_actions = [a for a, w
+                            in sorted(action_weights.iteritems(),
+                                      key=lambda item: (item[1], item[0]),
+                                      reverse=True)]
+
+        action_controls = []
         for action in sorted_actions:
             first_label = None
             hints = []
             widgets = []
-            ticket = tickets_by_action[action][0]
+            ticket = action_tickets[action][0]
             for controller in self._get_action_controllers(req, ticket,
                                                            action):
                 label, widget, hint = controller.render_ticket_action_control(
@@ -157,13 +194,11 @@ class BatchModifyModule(Component):
                 t = Ticket(self.env, int(id))
                 _values = new_values.copy()
                 for field in list_fields:
-                    if field in new_values:
-                        old = t.values[field] if field in t.values else ''
-                        new = new_values[field]
-                        mode = req.args.get('batchmod_value_' + field +
-                                            '_mode')
-                        new2 = req.args.get('batchmod_value_' + field +
-                                            '_secondary', '')
+                    mode = req.args.get('batchmod_mode_' + field)
+                    if mode:
+                        old = t[field] if field in t else ''
+                        new = req.args.get('batchmod_primary_' + field, '')
+                        new2 = req.args.get('batchmod_secondary_' + field, '')
                         _values[field] = self._change_list(old, new, new2,
                                                            mode)
                 controllers = list(self._get_action_controllers(req, t,
@@ -175,13 +210,14 @@ class BatchModifyModule(Component):
                 t.save_changes(req.authname, comment, when=when)
                 for controller in controllers:
                     controller.apply_action_side_effects(req, t, action)
-        tn = BatchTicketNotifyEmail(self.env)
+        event = BatchTicketChangeEvent(selected_tickets, when,
+                                       req.authname, comment, new_values,
+                                       action)
         try:
-            tn.notify(selected_tickets, new_values, comment, action,
-                      req.authname, when)
-        except Exception, e:
+            NotificationSystem(self.env).notify(event)
+        except Exception as e:
             self.log.error("Failure sending notification on ticket batch"
-                    "change: %s", exception_to_unicode(e))
+                           "change: %s", exception_to_unicode(e))
             add_warning(req, tag_("The changes have been saved, but an "
                                   "error occurred while sending "
                                   "notifications: %(message)s",
@@ -200,7 +236,7 @@ class BatchModifyModule(Component):
 
         if mode == '=':
             changed_list = new_list
-        elif mode ==  '+':
+        elif mode == '+':
             for entry in new_list:
                 if entry not in changed_list:
                     changed_list.append(entry)
