@@ -1,27 +1,30 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014 Edgewall Software
+# Copyright (C) 2014-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from __future__ import print_function
 
 import os
 import tempfile
+import textwrap
 import unittest
 
 import trac.tests.compat
+from trac.config import ConfigurationError
 from trac.perm import PermissionSystem
 from trac.test import EnvironmentStub, MockRequest
 from trac.ticket.api import TicketSystem
 from trac.ticket.batch import BatchModifyModule
+from trac.ticket.default_workflow import ConfigurableTicketWorkflow
 from trac.ticket.model import Component, Ticket
 from trac.ticket.web_ui import TicketModule
 from trac.util import create_file
@@ -203,9 +206,60 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
         self.assertEqual('component4', ticket['component'])
         self.assertEqual('cowner3', ticket['owner'])
 
+    def test_status_change_with_operation(self):
+        """Status change with operation."""
+        ticket = Ticket(self.env)
+        ticket['new'] = 'status1'
+        ticket['owner'] = 'user1'
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket', authname='user2',
+                          method='POST')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket, 'accept')
+
+        self.assertEqual('accept', label)
+        self.assertEqual('', unicode(control))
+        self.assertEqual("The owner will be <span class=\"trac-author-user\">"
+                         "user2</span>. The status will be 'accepted'.",
+                         unicode(hints))
+
+    def test_status_change_with_no_operation(self):
+        """Existing ticket status change with no operation."""
+        config = self.env.config
+        config.set('ticket-workflow', 'change_status', 'status1 -> status2')
+        self._reload_workflow()
+        ticket = Ticket(self.env)
+        ticket['status'] = 'status1'
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket', method='POST')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket,
+                                                   'change_status')
+
+        self.assertEqual('change status', label)
+        self.assertEqual('', unicode(control))
+        self.assertEqual("Next status will be 'status2'.", unicode(hints))
+
+    def test_new_ticket_status_change_with_no_operation(self):
+        """New ticket status change with no operation."""
+        config = self.env.config
+        config.set('ticket-workflow', 'change_status', '<none> -> status1')
+        self._reload_workflow()
+        ticket = Ticket(self.env)
+        req = MockRequest(self.env, path_info='/newticket', method='POST')
+
+        label, control, hints = \
+            self.ctlr.render_ticket_action_control(req, ticket,
+                                                   'change_status')
+
+        self.assertEqual('change status', label)
+        self.assertEqual('', unicode(control))
+        self.assertEqual("The status will be 'status1'.", unicode(hints))
+
     def test_transition_to_star(self):
-        """Workflow hint is not be added in a workflow transition to *,
-        for example: <none> -> *
+        """Action is not rendered for transition to *
 
         AdvancedTicketWorkflow uses the behavior for the triage operation
         (see #12823)
@@ -221,7 +275,7 @@ class ConfigurableTicketWorkflowTestCase(unittest.TestCase):
             self.ctlr.render_ticket_action_control(req, ticket,
                                                    'create_and_triage')
 
-        self.assertEqual('create and triage', label)
+        self.assertIsNone(label)
         self.assertEqual('', unicode(control))
         self.assertEqual('', unicode(hints))
 
@@ -289,9 +343,9 @@ class SetOwnerAttributeTestCase(unittest.TestCase):
         self.ticket = Ticket(self.env)
         self.ticket['status'] = 'new'
         self.ticket.insert()
-        with self.env.db_transaction as db:
-            for user in ('user1', 'user2', 'user3', 'user4'):
-                db("INSERT INTO session VALUES (%s, %s, %s)", (user, 1, 0))
+        self.env.insert_users([
+            (user, None, None) for user in ('user1', 'user2', 'user3', 'user4')
+        ])
         permissions = [
             ('user1', 'TICKET_EDIT_CC'),
             ('user2', 'TICKET_EDIT_CC'),
@@ -350,6 +404,148 @@ value="user1">user1</option><option value="user2">user2</option>\
         self.assertEqual(self.expected, str(control))
 
 
+class SetOwnerToSelfAttributeTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True)
+        self.ctlr = TicketSystem(self.env).action_controllers[0]
+        self.req = MockRequest(self.env, authname='user1')
+        ps = PermissionSystem(self.env)
+        for user in ('user1', 'user2'):
+            ps.grant_permission(user, 'TICKET_MODIFY')
+        self.env.insert_users([('user1', 'User 1', None),
+                               ('user2', 'User 2', None)])
+
+    def _get_ticket_actions(self, req, ticket):
+        return [action[1] for action
+                          in self.ctlr.get_ticket_actions(req, ticket)]
+
+    def _reload_workflow(self):
+        self.ctlr.actions = self.ctlr.get_all_actions()
+
+    def _insert_ticket(self, status, owner, resolution=None):
+        ticket = Ticket(self.env)
+        ticket['status'] = status
+        ticket['owner'] = owner
+        if resolution:
+            ticket['resolution'] = resolution
+        ticket.insert()
+        return ticket
+
+    def test_owner_is_other(self):
+        """Ticket owner is not auth'ed user.
+
+        The workflow action is shown when the state will be changed by
+        the action.
+        """
+        ticket = self._insert_ticket('accepted', 'user2')
+        args = self.req, ticket, 'accept'
+
+        label, control, hints = self.ctlr.render_ticket_action_control(*args)
+        ticket_actions = self._get_ticket_actions(*args[0:2])
+
+        self.assertIn('accept', ticket_actions)
+        self.assertEqual(label, 'accept')
+        self.assertEqual('', unicode(control))
+        self.assertEqual('The owner will be changed from '
+                         '<span class="trac-author">User 2</span> to '
+                         '<span class="trac-author-user">User 1</span>. '
+                         'Next status will be \'accepted\'.', unicode(hints))
+
+    def test_owner_is_self_and_state_change(self):
+        """Ticket owner is auth'ed user with state change.
+
+        The workflow action is shown when the state will be changed by the
+        action, even when the ticket owner is the authenticated user.
+        """
+        ticket = self._insert_ticket('new', 'user1')
+        args = self.req, ticket, 'accept'
+
+        label, control, hints = self.ctlr.render_ticket_action_control(*args)
+        ticket_actions = self._get_ticket_actions(*args[0:2])
+
+        self.assertIn('accept', ticket_actions)
+        self.assertEqual(label, 'accept')
+        self.assertEqual('', unicode(control))
+        self.assertEqual('The owner will remain <span class="trac-author-user">'
+                         'User 1</span>. Next status will be \'accepted\'.',
+                         unicode(hints))
+
+    def test_owner_is_self_and_no_state_change(self):
+        """Ticket owner is the auth'ed user and no state change.
+
+        The ticket action is not in the list of available actions
+        when the state will not be changed by the action and the ticket
+        owner is the authenticated user.
+        """
+        ticket = self._insert_ticket('accepted', 'user1')
+        args = self.req, ticket, 'accept'
+
+        ticket_actions = self._get_ticket_actions(*args[0:2])
+
+        self.assertNotIn('accept', ticket_actions)
+
+    def test_owner_is_self_state_change_and_multiple_operations(self):
+        """Ticket owner is auth'ed user, state change and multiple ops.
+
+        The set_owner_to_self workflow hint is shown when the ticket status
+        is changed by the action, even when the ticket owner is the
+        authenticated user.
+        """
+        ticket = self._insert_ticket('new', 'user1')
+        workflow = self.env.config['ticket-workflow']
+        workflow.set('resolve_as_owner', '* -> closed')
+        workflow.set('resolve_as_owner.operations',
+                     'set_owner_to_self, set_resolution')
+        workflow.set('resolve_as_owner.set_resolution', 'fixed')
+        self._reload_workflow()
+        args = self.req, ticket, 'resolve_as_owner'
+
+        label, control, hints = self.ctlr.render_ticket_action_control(*args)
+        ticket_actions = self._get_ticket_actions(*args[0:2])
+
+        self.assertIn('resolve_as_owner', ticket_actions)
+        self.assertEqual(label, 'resolve as owner')
+        self.assertEqual(
+            'as fixed<input type="hidden" '
+            'name="action_resolve_as_owner_resolve_resolution" value="fixed" '
+            'id="action_resolve_as_owner_resolve_resolution"/>',
+            unicode(control))
+        self.assertEqual(
+            "The owner will remain <span class=\"trac-author-user\">User 1"
+            "</span>. The resolution will be set to fixed. Next status will "
+            "be 'closed'.", unicode(hints))
+
+    def test_owner_is_self_no_state_change_and_multiple_operations(self):
+        """Ticket owner is auth'ed user, no state change and multiple ops.
+
+        The set_owner_to_self workflow hint is not shown when the ticket
+        state is not changed by the action and the ticket owner is the
+        authenticated user.
+        """
+        ticket = self._insert_ticket('closed', 'user1', 'fixed')
+        workflow = self.env.config['ticket-workflow']
+        workflow.set('fix_resolution', 'closed -> closed')
+        workflow.set('fix_resolution.operations',
+                     'set_owner_to_self, set_resolution')
+        workflow.set('fix_resolution.set_resolution', 'invalid')
+        self._reload_workflow()
+        args = self.req, ticket, 'fix_resolution'
+
+        label, control, hints = self.ctlr.render_ticket_action_control(*args)
+        ticket_actions = self._get_ticket_actions(*args[0:2])
+
+        self.assertIn('fix_resolution', ticket_actions)
+        self.assertEqual(label, 'fix resolution')
+        self.assertEqual(
+            'as invalid<input type="hidden" '
+            'name="action_fix_resolution_resolve_resolution" value="invalid" '
+            'id="action_fix_resolution_resolve_resolution"/>',
+            unicode(control))
+        self.assertEqual("The resolution will be set to invalid. "
+                         "Next status will be 'closed'.", unicode(hints))
+
+
 class RestrictOwnerTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -403,10 +599,10 @@ class RestrictOwnerTestCase(unittest.TestCase):
         owners list (#10833).
         """
         self.env.config.set('trac', 'show_full_names', False)
-        create_file(self.authz_file, """\
-[ticket:1]
-user4 = !TICKET_MODIFY
-""")
+        create_file(self.authz_file, textwrap.dedent("""\
+            [ticket:1]
+            user4 = !TICKET_MODIFY
+            """))
 
         ctrl = self.ctlr.render_ticket_action_control(self.req1, self.ticket,
                                                       'reassign')
@@ -432,12 +628,60 @@ id="action_reassign_reassign_owner">\
 """, str(ctrl[1]))
 
 
+class SetResolutionAttributeTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True)
+        for ctlr in TicketSystem(self.env).action_controllers:
+            if isinstance(ctlr, ConfigurableTicketWorkflow):
+                self.ctlr = ctlr
+
+    def _reload_workflow(self):
+        self.ctlr.actions = self.ctlr.get_all_actions()
+
+    def test_empty_set_resolution(self):
+        config = self.env.config['ticket-workflow']
+        config.set('resolve.set_resolution', '')
+        self._reload_workflow()
+        ticket = Ticket(self.env)
+        ticket.populate({'summary': '#12882', 'status': 'new'})
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket/%d' % ticket.id)
+        try:
+            self.ctlr.render_ticket_action_control(req, ticket, 'resolve')
+            self.fail('ConfigurationError not raised')
+        except ConfigurationError as e:
+            self.assertIn('but none is defined', unicode(e))
+
+    def test_undefined_resolutions(self):
+        config = self.env.config['ticket-workflow']
+        ticket = Ticket(self.env)
+        ticket.populate({'summary': '#12882', 'status': 'new'})
+        ticket.insert()
+        req = MockRequest(self.env, path_info='/ticket/%d' % ticket.id)
+
+        config.set('resolve.set_resolution',
+                   'fixed,invalid,wontfix,,duplicate,worksforme,,,,,')
+        self._reload_workflow()
+        self.ctlr.render_ticket_action_control(req, ticket, 'resolve')
+
+        config.set('resolve.set_resolution', 'undefined,fixed')
+        self._reload_workflow()
+        try:
+            self.ctlr.render_ticket_action_control(req, ticket, 'resolve')
+            self.fail('ConfigurationError not raised')
+        except ConfigurationError as e:
+            self.assertIn('but uses undefined resolutions', unicode(e))
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ConfigurableTicketWorkflowTestCase))
     suite.addTest(unittest.makeSuite(ResetActionTestCase))
     suite.addTest(unittest.makeSuite(SetOwnerAttributeTestCase))
+    suite.addTest(unittest.makeSuite(SetOwnerToSelfAttributeTestCase))
     suite.addTest(unittest.makeSuite(RestrictOwnerTestCase))
+    suite.addTest(unittest.makeSuite(SetResolutionAttributeTestCase))
     return suite
 
 
