@@ -1,17 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import with_statement
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from datetime import datetime, timedelta
 from StringIO import StringIO
@@ -22,10 +20,10 @@ import trac.tests.compat
 from trac import core
 from trac.attachment import Attachment
 from trac.core import TracError, implements
-from trac.resource import ResourceNotFound
+from trac.resource import Resource, ResourceNotFound
 from trac.test import EnvironmentStub
 from trac.ticket.model import (
-    Ticket, Component, Milestone, Priority, Type, Version
+    Ticket, Component, Milestone, Priority, Report, Type, Version
 )
 from trac.ticket.roadmap import MilestoneModule
 from trac.ticket.api import (
@@ -110,6 +108,15 @@ class TicketTestCase(unittest.TestCase):
         ticket['foo'] = 'This is a custom field'
         return ticket
 
+    def test_resource_id_is_none(self):
+        ticket = Ticket(self.env)
+        self.assertEqual(Resource('ticket'), ticket.resource)
+
+    def test_resource_exists(self):
+        ticket_id = self._insert_ticket('Foo')
+        ticket = Ticket(self.env, ticket_id)
+        self.assertEqual(Resource('ticket', 1), ticket.resource)
+
     def test_invalid_ticket_id(self):
         self.assertEqual(Ticket.id_is_valid(-1), False)
         self.assertEqual(Ticket.id_is_valid(0), False)
@@ -119,6 +126,12 @@ class TicketTestCase(unittest.TestCase):
         self.assertRaises(ResourceNotFound, Ticket, self.env, -1)
         self.assertRaises(ResourceNotFound, Ticket, self.env, 1L << 32)
 
+    def test_repr(self):
+        ticket = self._create_a_ticket()
+        self.assertEqual("<Ticket None>", repr(ticket))
+        ticket.insert()
+        self.assertEqual("<Ticket 1>", repr(ticket))
+
     def test_create_ticket_1(self):
         ticket = self._create_a_ticket()
         self.assertEqual('santa', ticket['reporter'])
@@ -199,6 +212,8 @@ class TicketTestCase(unittest.TestCase):
     def test_create_empty_strings_stored_as_null(self):
         """Ticket fields with empty strings are NULL when creating ticket.
         """
+        self.env.config.set('ticket-custom', 'time1', 'time')
+        self.env.config.save()
         ticket = Ticket(self.env)
         ticket.populate(dict((name, '') for name in ticket.editable_fields))
         ticket.insert()
@@ -208,6 +223,8 @@ class TicketTestCase(unittest.TestCase):
     def test_change_empty_strings_stored_as_null(self):
         """Ticket fields with empty strings are NULL when changing ticket.
         """
+        self.env.config.set('ticket-custom', 'time1', 'time')
+        self.env.config.save()
         ticket = Ticket(self.env)
         ticket.insert()
         ticket.populate(dict((name, '') for name in ticket.editable_fields))
@@ -244,6 +261,18 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual(ticket_id, ticket.id)
         self.assertEqual(ticket.resource.id, ticket_id)
 
+    def test_resource_not_found_for_invalid_ticket_id(self):
+        try:
+            Ticket(self.env, '42')
+            self.fail('ResourceNotFound not raised')
+        except ResourceNotFound as e:
+            self.assertEqual(u'Ticket 42 does not exist.', unicode(e))
+        try:
+            Ticket(self.env, 'blah')
+            self.fail('ResourceNotFound not raised')
+        except ResourceNotFound as e:
+            self.assertEqual(u'Ticket blah does not exist.', unicode(e))
+
     def test_can_save_ticket_without_explicit_comment(self):
         ticket = Ticket(self.env)
         ticket.insert()
@@ -314,6 +343,23 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual('two', ticket['bar'])
         self.assertEqual('three', ticket['baz'])
 
+    def test_ticket_custom_field_default_values(self):
+        """Ticket created before a custom field is added will have the
+        default value for the custom field.
+        """
+        tid = self._insert_ticket("The summary")
+        self.env.config.set('ticket-custom', 'baz', 'text')
+        self.env.config.set('ticket-custom', 'baz.value', 'Something')
+        self.env.config.set('ticket-custom', 'bar', 'select')
+        self.env.config.set('ticket-custom', 'bar.options', 'one|two|three')
+        self.env.config.set('ticket-custom', 'bar.value', 'two')
+        TicketSystem(self.env).reset_ticket_fields()
+        del TicketSystem(self.env).custom_fields
+        ticket = Ticket(self.env, tid)
+
+        self.assertEqual('Something', ticket['baz'])
+        self.assertEqual('two', ticket['bar'])
+
     def test_set_field_stripped(self):
         """
         Verify that whitespace around ticket fields is stripped, except for
@@ -334,77 +380,6 @@ class TicketTestCase(unittest.TestCase):
             ticket['component'] = ['  foo  ',  '  bar  ']
         self.assertRaises(TracError, set_multi_valued)
 
-    def test_owner_from_component(self):
-        """
-        Verify that the owner of a new ticket is set to the owner of the
-        component.
-        """
-        component = Component(self.env)
-        component.name = 'test'
-        component.owner = 'joe'
-        component.insert()
-
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'santa'
-        ticket['summary'] = 'Foo'
-        ticket['component'] = 'test'
-        ticket.insert()
-        self.assertEqual('joe', ticket['owner'])
-
-    def test_owner_from_changed_component(self):
-        """
-        Verify that the owner of a new ticket is updated when the component is
-        changed.
-        """
-        component1 = Component(self.env)
-        component1.name = 'test1'
-        component1.owner = 'joe'
-        component1.insert()
-
-        component2 = Component(self.env)
-        component2.name = 'test2'
-        component2.owner = 'kate'
-        component2.insert()
-
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'santa'
-        ticket['summary'] = 'Foo'
-        ticket['component'] = 'test1'
-        ticket['status'] = 'new'
-        tktid = ticket.insert()
-
-        ticket = Ticket(self.env, tktid)
-        ticket['component'] = 'test2'
-        ticket.save_changes('jane', 'Testing')
-        self.assertEqual('kate', ticket['owner'])
-
-    def test_no_disown_from_changed_component(self):
-        """
-        Verify that a ticket is not disowned when the component is changed to
-        a non-assigned component.
-        """
-        component1 = Component(self.env)
-        component1.name = 'test1'
-        component1.owner = 'joe'
-        component1.insert()
-
-        component2 = Component(self.env)
-        component2.name = 'test2'
-        component2.owner = ''
-        component2.insert()
-
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'santa'
-        ticket['summary'] = 'Foo'
-        ticket['component'] = 'test1'
-        ticket['status'] = 'new'
-        tktid = ticket.insert()
-
-        ticket = Ticket(self.env, tktid)
-        ticket['component'] = 'test2'
-        ticket.save_changes('jane', 'Testing')
-        self.assertEqual('joe', ticket['owner'])
-
     def test_populate_ticket(self):
         data = {'summary': 'Hello world', 'reporter': 'john',
                 'foo': 'bar', 'checkbox_cbon': '', 'cbon': 'on',
@@ -426,6 +401,28 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual('on', ticket['cbon'])
         self.assertEqual('0', ticket['cboff'])
 
+    def test_custom_time(self):
+        # Add a custom field of type 'time'
+        self.env.config.set('ticket-custom', 'due', 'time')
+        ticket = Ticket(self.env)
+        self.assertFalse('due' in ticket.std_fields)
+        self.assertTrue('due' in ticket.time_fields)
+        ticket['reporter'] = 'john'
+        ticket['summary'] = 'Task1'
+        tktid = ticket.insert()
+        ticket = Ticket(self.env, tktid)
+        # Empty string is default value, but not a time stamp
+        self.assertEqual(None, ticket['due'])
+        ts = datetime(2011, 11, 11, 0, 0, 0, 0, utc)
+        ticket['due'] = ts
+        t1 = datetime(2001, 1, 1, 1, 1, 1, 0, utc)
+        ticket.save_changes('joe', when=t1)
+        self.assertEqual(ts, ticket['due'])
+        ticket['due'] = ''
+        t2 = datetime(2001, 1, 1, 1, 1, 2, 0, utc)
+        ticket.save_changes('joe', when=t2)
+        self.assertEqual('', ticket['due'])
+
     def test_changelog(self):
         tkt_id = self._insert_ticket('Test', reporter='joe', component='foo',
                                      milestone='bar')
@@ -434,7 +431,7 @@ class TicketTestCase(unittest.TestCase):
         ticket['milestone'] = 'foo'
         now = datetime(2001, 1, 1, 1, 1, 1, 0, utc)
         ticket.save_changes('jane', 'Testing', now)
-        changelog = sorted(ticket.get_changelog())
+        changelog = ticket.get_changelog()
         self.assertEqual([(now, 'jane', 'comment', '1', 'Testing', True),
                           (now, 'jane', 'component', 'foo', 'bar', True),
                           (now, 'jane', 'milestone', 'bar', 'foo', True)],
@@ -459,7 +456,7 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual((t1, 'jane', 'comment', '1', 'Testing', True), log[0])
         self.assertEqual([(t2, 'mark', 'attachment', '', 'file.txt', False),
                           (t2, 'mark', 'comment', '', 'My file', False)],
-                          sorted(log[1:3]))
+                          log[1:3])
         self.assertEqual((t3, 'jim', 'comment', '2', 'Other', True), log[3])
 
     def test_subsecond_change(self):
@@ -719,6 +716,15 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
         self.assertEqual('New Comment 2', listener.comment)
         self.assertEqual('Comment 2', listener.old_comment)
 
+    def test_get_comment_number(self):
+        ticket = Ticket(self.env, self.id)
+        self.assertEqual(1, ticket.get_comment_number(self.created +
+                                                      timedelta(seconds=1)))
+        self.assertEqual(2, ticket.get_comment_number(self.created +
+                                                      timedelta(seconds=2)))
+        self.assertEqual(3, ticket.get_comment_number(self.created +
+                                                      timedelta(seconds=3)))
+
 
 class TicketCommentDeleteTestCase(TicketCommentTestCase):
 
@@ -757,7 +763,7 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
         self.assertEqual(t, ticket.time_changed)
 
     def test_delete_last_comment_when_custom_field_gone(self):
-        """Regression test for http://trac.edgewall.org/ticket/10858"""
+        """Regression test for https://trac.edgewall.org/ticket/10858"""
         ticket = Ticket(self.env, self.id)
         self.assertEqual('a', ticket['keywords'])
         self.assertEqual('change4', ticket['foo'])
@@ -874,6 +880,7 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
                               foo=('change 1', 'change2')),
                          listener.changes)
 
+
 class EnumTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -882,6 +889,11 @@ class EnumTestCase(unittest.TestCase):
     def tearDown(self):
         self.env.reset_db()
 
+    def test_repr(self):
+        self.assertEqual("<Priority None None>", repr(Priority(self.env)))
+        self.assertEqual("<Priority 'major' u'3'>",
+                         repr(Priority(self.env, 'major')))
+
     def test_priority_fetch(self):
         prio = Priority(self.env, 'major')
         self.assertEqual(prio.name, 'major')
@@ -979,6 +991,7 @@ class MilestoneTestCase(unittest.TestCase):
         self.assertIsNone(milestone.due)
         self.assertIsNone(milestone.completed)
         self.assertEqual('', milestone.description)
+        self.assertEqual("<Milestone None>", repr(milestone))
 
     def test_new_milestone_empty_name(self):
         """
@@ -991,6 +1004,7 @@ class MilestoneTestCase(unittest.TestCase):
         self.assertIsNone(milestone.due)
         self.assertIsNone(milestone.completed)
         self.assertEqual('', milestone.description)
+        self.assertEqual("<Milestone None>", repr(milestone))
 
     def test_existing_milestone(self):
         self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
@@ -1001,6 +1015,7 @@ class MilestoneTestCase(unittest.TestCase):
         self.assertIsNone(milestone.due)
         self.assertIsNone(milestone.completed)
         self.assertEqual('', milestone.description)
+        self.assertEqual("<Milestone u'Test'>", repr(milestone))
 
     def test_create_and_update_milestone(self):
         milestone = Milestone(self.env)
@@ -1259,11 +1274,16 @@ class ComponentTestCase(unittest.TestCase):
 
     def test_exists(self):
         """
-        http://trac.edgewall.org/ticket/4247
+        https://trac.edgewall.org/ticket/4247
         """
         for c in Component.select(self.env):
             self.assertEqual(c.exists, True)
 
+    def test_repr(self):
+        self.assertEqual('<Component None>', repr(Component(self.env)))
+        self.assertEqual("<Component 'component1'>",
+                         repr(Component(self.env, 'component1')))
+
     def test_create_and_update(self):
         component = Component(self.env)
         component.name = 'Test'
@@ -1279,6 +1299,115 @@ class ComponentTestCase(unittest.TestCase):
         self.assertEqual([('Test', 'joe', None)], self.env.db_query(
             "SELECT name, owner, description FROM component WHERE name='Test'"))
 
+
+class ReportTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(default_data=True)
+
+    def tearDown(self):
+        self.env.reset_db()
+
+    def test_repr(self):
+        report = Report(self.env)
+        report.query = "SELECT 1"
+        report.insert()
+        self.assertEqual("<Report 1>", repr(Report(self.env, 1)))
+        self.assertEqual("<Report None>", repr(Report(self.env)))
+
+    def test_create(self):
+        report = Report(self.env, 4)
+        self.assertTrue(report.exists)
+        self.assertEqual(4, report.id)
+        self.assertEqual("Accepted, Active Tickets by Owner", report.title)
+        self.assertEqual("List accepted tickets, group by ticket owner, "
+                         "sorted by priority.\n", report.description)
+        self.assertIn("SELECT p.value AS __color__,", report.query)
+
+    def test_create_exists_false(self):
+        self.assertRaises(ResourceNotFound, Report, self.env, 9)
+
+    def test_insert(self):
+        report = Report(self.env)
+        report.title = "The report"
+        report.description = "The description"
+        report.query = "SELECT 1"
+        report.insert()
+        self.assertEqual(9, report.id)
+
+    def test_insert_query_is_empty(self):
+        """TracError is raised when query attribute is empty."""
+        report = Report(self.env)
+        report.title = "The report"
+        report.description = "The description"
+        report.query = ""
+
+        self.assertRaises(TracError, report.insert)
+        self.assertIsNone(report.id)
+
+    def test_insert_existing_report(self):
+        report = Report(self.env, 1)
+        self.assertRaises(AssertionError, report.insert)
+
+    def test_delete(self):
+        report = Report(self.env, 1)
+        report.delete()
+        self.assertFalse(report.exists)
+        self.assertRaises(ResourceNotFound, Report, self.env, 1)
+
+    def test_delete_not_exists(self):
+        report = Report(self.env)
+        self.assertRaises(AssertionError, report.delete)
+
+    def test_update(self):
+        report = Report(self.env, 1)
+        title, description, query = \
+            report.title, report.description, report.query
+        report.title = "The report"
+        report.description = "The description"
+        report.query = "SELECT 1"
+        report.update()
+
+        report = Report(self.env, 1)
+        self.assertNotEqual(title, report.title)
+        self.assertNotEqual(description, report.description)
+        self.assertNotEqual(query, report.query)
+        self.assertEqual("The report", report.title)
+        self.assertEqual("The description", report.description)
+        self.assertEqual("SELECT 1", report.query)
+
+    def test_update_query_is_empty(self):
+        """TracError is raised when query attribute is empty."""
+        report = Report(self.env, 1)
+        report.query = ""
+
+        self.assertRaises(TracError, report.update)
+
+    def test_select(self):
+        reports = list(Report.select(self.env))
+        self.assertEqual(1, reports[0].id)
+        self.assertEqual('Active Tickets', reports[0].title)
+        self.assertEqual(" * List all active tickets by priority.\n"
+                         " * Color each row based on priority.\n",
+                         reports[0].description)
+        self.assertIn("SELECT p.value AS __color__", reports[0].query)
+        self.assertEqual(8, len(reports))
+        self.assertEqual(1, reports[0].id)
+        self.assertEqual(8, reports[-1].id)
+
+    def test_select_sort_desc(self):
+        reports = list(Report.select(self.env, asc=False))
+        self.assertEqual(8, len(reports))
+        self.assertEqual(8, reports[0].id)
+        self.assertEqual(1, reports[-1].id)
+
+    def test_select_order_by_title(self):
+        reports = list(Report.select(self.env, sort='title'))
+        self.assertEqual(8, len(reports))
+        self.assertEqual(4, reports[0].id)
+        self.assertEqual(7, reports[-1].id)
+
+
 class VersionTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -1294,11 +1423,15 @@ class VersionTestCase(unittest.TestCase):
 
     def test_exists(self):
         """
-        http://trac.edgewall.org/ticket/4247
+        https://trac.edgewall.org/ticket/4247
         """
         for v in Version.select(self.env):
             self.assertEqual(v.exists, True)
 
+    def test_repr(self):
+        self.assertEqual('<Version None>', repr(Version(self.env)))
+        self.assertEqual("<Version '1.0'>", repr(Version(self.env, '1.0')))
+
     def test_create_and_update(self):
         version = Version(self.env)
         version.name = 'Test'
@@ -1314,7 +1447,7 @@ class VersionTestCase(unittest.TestCase):
             "SELECT name, time, description FROM version WHERE name='Test'"))
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(TicketTestCase))
     suite.addTest(unittest.makeSuite(TicketCommentEditTestCase))
@@ -1322,8 +1455,9 @@ def suite():
     suite.addTest(unittest.makeSuite(EnumTestCase))
     suite.addTest(unittest.makeSuite(MilestoneTestCase))
     suite.addTest(unittest.makeSuite(ComponentTestCase))
+    suite.addTest(unittest.makeSuite(ReportTestCase))
     suite.addTest(unittest.makeSuite(VersionTestCase))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
