@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Include a basic SMTP server, based on L. Smithson
 # (lsmithson@open-networks.co.uk) extensible Python SMTP Server
@@ -31,7 +31,7 @@ from trac.test import EnvironmentStub, MockRequest
 from trac.tests.notification import SMTP_TEST_PORT, SMTPThreadedServer, \
                                     parse_smtp_message
 from trac.ticket.model import Ticket
-from trac.ticket.notification import BatchTicketNotifyEmail, \
+from trac.ticket.notification import BatchTicketNotifyEmail, Subscription, \
                                      TicketChangeEvent, TicketNotifyEmail
 from trac.ticket.web_ui import TicketModule
 from trac.util.datefmt import datetime_now, utc
@@ -69,6 +69,7 @@ def config_subscriber(env, updater=False, owner=False, reporter=False):
                        'TicketReporterSubscriber')
     del NotificationSystem(env).subscriber_defaults
 
+
 def config_smtp(env):
     env.config.set('project', 'name', 'TracTest')
     env.config.set('notification', 'smtp_enabled', 'true')
@@ -78,6 +79,11 @@ def config_smtp(env):
     #       for '::1' first, then only '127.0.0.1' after a 1s timeout
 
 
+def extract_recipients(header):
+    rcpts = [rcpt.strip() for rcpt in header.split(',')]
+    return [rcpt for rcpt in rcpts if rcpt]
+
+
 class RecipientTestCase(unittest.TestCase):
     """Notification test cases for email recipients."""
 
@@ -207,7 +213,7 @@ class RecipientTestCase(unittest.TestCase):
             now = datetime_now(utc)
             ticket.save_changes('joe@example.org', 'this is my comment',
                                 when=now)
-            notify_ticket_created(self.env, ticket)
+            notify_ticket_changed(self.env, ticket)
             recipients = notifysuite.smtpd.get_recipients()
             if enabled:
                 self.assertEqual(1, len(recipients))
@@ -229,7 +235,7 @@ class RecipientTestCase(unittest.TestCase):
             now = datetime_now(utc)
             ticket.save_changes('joe@example.org', 'this is my comment',
                                 when=now)
-            notify_ticket_created(self.env, ticket)
+            notify_ticket_changed(self.env, ticket)
             recipients = notifysuite.smtpd.get_recipients()
             if enabled:
                 self.assertEqual(1, len(recipients))
@@ -240,6 +246,49 @@ class RecipientTestCase(unittest.TestCase):
         for enable in False, True:
             _test_reporter(enable)
 
+    def test_notify_new_tickets(self):
+        """Notification to NewTicketSubscribers."""
+        def _test_new_ticket():
+            ticket = Ticket(self.env)
+            ticket['reporter'] = 'user4'
+            ticket['owner'] = 'user5'
+            ticket['summary'] = 'New Ticket Subscribers'
+            ticket.insert()
+            notify_ticket_created(self.env, ticket)
+            return ticket
+
+        def _add_new_ticket_subscriber(sid, authenticated):
+            Subscription.add(self.env, {
+                'sid': sid, 'authenticated': authenticated,
+                'distributor': 'email', 'format': None, 'adverb': 'always',
+                'class': 'NewTicketSubscriber'
+            })
+
+        _test_new_ticket()
+        recipients = notifysuite.smtpd.get_recipients()
+        self.assertEqual(0, len(recipients))
+
+        self.env.insert_users([
+            ('user1', 'User One', 'joe@example.org', 1),
+            ('user2', 'User Two', 'bar@example.org', 1),
+            ('58bd3a', 'User Three', 'jim@example.org', 0),
+        ])
+        _add_new_ticket_subscriber('user1', 1)
+        _add_new_ticket_subscriber('58bd3a', 0)
+
+        ticket = _test_new_ticket()
+        recipients = notifysuite.smtpd.get_recipients()
+        self.assertEqual(2, len(recipients))
+        self.assertIn('joe@example.org', recipients)
+        self.assertIn('jim@example.org', recipients)
+
+        ticket.save_changes('user4', 'this is my comment',
+                            when=datetime_now(utc))
+        notify_ticket_changed(self.env, ticket)
+
+        recipients = notifysuite.smtpd.get_recipients()
+        self.assertEqual(0, len(recipients))
+
     def test_no_duplicates(self):
         """Email addresses should be found only once in the recipient list."""
         self.env.config.set('notification', 'smtp_always_cc',
@@ -374,7 +423,7 @@ class NotificationTestCase(unittest.TestCase):
             # Msg should have a To header
             self.assertEqual('undisclosed-recipients: ;', headers['To'])
             # Extract the list of 'Cc' recipients from the message
-            cc = [rcpt.strip() for rcpt in headers['Cc'].split(',')]
+            cc = extract_recipients(headers['Cc'])
             # Extract the list of the actual SMTP recipients
             rcptlist = notifysuite.smtpd.get_recipients()
             # Build the list of the expected 'Cc' recipients
@@ -413,14 +462,16 @@ class NotificationTestCase(unittest.TestCase):
             recipients = set(notifysuite.smtpd.get_recipients())
             headers, body = parse_smtp_message(message)
             # Msg should always have a 'To' field
-            self.assertEqual('undisclosed-recipients: ;', headers['To'])
+            if use_short_addr:
+                self.assertEqual(address, headers['To'])
+            else:
+                self.assertEqual('undisclosed-recipients: ;', headers['To'])
             # Msg should have a 'Cc' field
             self.assertIn('Cc', headers)
-            cclist = set(addr.strip() for addr in headers['Cc'].split(','))
+            cclist = set(extract_recipients(headers['Cc']))
             if use_short_addr:
                 # Msg should be delivered to the reporter
-                self.assertEqual(set([address, 'joe.bar@example.net',
-                                      'john']), cclist)
+                self.assertEqual(set(['joe.bar@example.net', 'john']), cclist)
                 self.assertEqual(set([address, 'joe.bar@example.net',
                                       'john']), recipients)
             else:
@@ -456,7 +507,7 @@ class NotificationTestCase(unittest.TestCase):
             headers, body = parse_smtp_message(message)
             # Msg should always have a 'Cc' field
             self.assertIn('Cc', headers)
-            cclist = set(addr.strip() for addr in headers['Cc'].split(','))
+            cclist = set(extract_recipients(headers['Cc']))
             if enable:
                 self.assertEqual(set(['joenodom@example.org',
                                       'foo@example.org',
@@ -490,14 +541,12 @@ class NotificationTestCase(unittest.TestCase):
         notify_ticket_created(self.env, ticket)
         message = notifysuite.smtpd.get_message()
         headers, body = parse_smtp_message(message)
-        # Msg should always have a 'To' field
-        self.assertEqual('undisclosed-recipients: ;', headers['To'])
-        # 'Cc' list should have been resolved to the real email address
-        cclist = [addr.strip() for addr in headers['Cc'].split(',')]
-        self.assertIn('user-joe@example.com', cclist)
-        self.assertIn('user-jim@example.com', cclist)
-        self.assertNotIn('joeuser', cclist)
-        self.assertNotIn('jim@domain', cclist)
+        tolist = sorted(extract_recipients(headers['To']))
+        cclist = sorted(extract_recipients(headers['Cc']))
+        # 'To' list should have been resolved to the real email address
+        self.assertEqual(['user-jim@example.com', 'user-joe@example.com'],
+                         tolist)
+        self.assertEqual(['joe@example.com'], cclist)
 
     def test_from_author(self):
         """Using the reporter or change author as the notification sender"""
@@ -601,15 +650,16 @@ class NotificationTestCase(unittest.TestCase):
         notify_ticket_created(self.env, ticket)
         message = notifysuite.smtpd.get_message()
         headers, body = parse_smtp_message(message)
-        # Msg should always have a 'To' field
-        self.assertEqual('undisclosed-recipients: ;', headers['To'])
-        cclist = set(addr.strip() for addr in headers['Cc'].split(','))
-        # 'Cc' list should not contain addresses with non-SMTP domains
-        self.assertNotIn('kerberos@example.com', cclist)
-        self.assertNotIn('kerberos@example.org', cclist)
+        tolist = sorted(extract_recipients(headers['To']))
+        cclist = sorted(extract_recipients(headers['Cc']))
+        # 'To' list should not contain addresses with non-SMTP domains
+        self.assertNotIn('kerberos@example.com', tolist)
+        self.assertNotIn('kerberos@example.org', tolist)
+        # 'To' list should have been resolved to the actual email address
+        self.assertEqual(['kerb@example.net'], tolist)
         # 'Cc' list should have been resolved to the actual email address
-        self.assertEqual(set(['kerb@example.net', 'joe.user@example.net',
-                              'joe.bar@example.net']), cclist)
+        self.assertEqual(['joe.bar@example.net', 'joe.user@example.net'],
+                         cclist)
 
     def test_admit_domains(self):
         """SMTP domain inclusion"""
@@ -626,17 +676,17 @@ class NotificationTestCase(unittest.TestCase):
         message = notifysuite.smtpd.get_message()
         headers, body = parse_smtp_message(message)
         # Msg should always have a 'To' field
-        self.assertEqual('undisclosed-recipients: ;', headers['To'])
+        self.assertEqual('joeuser@example.com', headers['To'])
         self.assertIn('Cc', headers)
-        cclist = [addr.strip() for addr in headers['Cc'].split(',')]
+        cclist = set(extract_recipients(headers['Cc']))
         # 'Cc' list should contain addresses with SMTP included domains
         self.assertIn('joe.user@localdomain', cclist)
         self.assertIn('joe.user@server', cclist)
         # 'Cc' list should not contain non-FQDN domains
         self.assertNotIn('joe.user@unknown', cclist)
-        self.assertEqual(set(['joeuser@example.com', 'joe.user@localdomain',
-                              'joe.user@server', 'joe.user@example.net',
-                              'joe.bar@example.net']), set(cclist))
+        self.assertEqual(set(['joe.user@localdomain', 'joe.user@server',
+                              'joe.user@example.net', 'joe.bar@example.net']),
+                         cclist)
 
     def test_multiline_header(self):
         """Encoded headers split into multiple lines"""
@@ -706,7 +756,8 @@ class NotificationTestCase(unittest.TestCase):
         notify_ticket_created(self.env, ticket)
         message = notifysuite.smtpd.get_message()
         headers, body = parse_smtp_message(message)
-        self.assertEqual('joe.user@example.org', headers['Cc'])
+        self.assertEqual('joe.user@example.org', headers['To'])
+        self.assertNotIn('Cc', headers)
         return headers
 
     def test_md5_digest(self):
