@@ -1,60 +1,198 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2014 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import with_statement
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import doctest
+import unittest
 from datetime import datetime, timedelta
 
-import unittest
-from StringIO import StringIO
-
+import trac
 import trac.tests.compat
-from trac.db.mysql_backend import MySQLConnection
-from trac.perm import PermissionCache, PermissionSystem
+from trac.perm import PermissionSystem
+from trac.resource import ResourceNotFound
 from trac.ticket.model import Ticket
 from trac.ticket.query import QueryModule
-from trac.ticket.report import ReportModule
-from trac.test import EnvironmentStub, Mock, MockPerm, MockRequest
+from trac.ticket.report import Report, ReportModule
+from trac.test import EnvironmentStub, MockRequest
 from trac.util.datefmt import utc
-from trac.web.api import HTTPBadRequest, Request, RequestDone
+from trac.web.api import HTTPBadRequest, RequestDone
 from trac.web.chrome import Chrome
-from trac.web.href import Href
-import trac
 
 
-class MockMySQLConnection(MySQLConnection):
-    def __init__(self):
-        pass
-
-
-class ReportTestCase(unittest.TestCase):
+class ReportModuleTestCase(unittest.TestCase):
 
     def setUp(self):
-        self.env = EnvironmentStub()
+        self.env = EnvironmentStub(default_data=True)
         self.report_module = ReportModule(self.env)
 
     def tearDown(self):
         self.env.reset_db()
 
-    def _make_environ(self, scheme='http', server_name='example.org',
-                      server_port=80, method='GET', script_name='/trac',
-                      **kwargs):
-        environ = {'wsgi.url_scheme': scheme, 'wsgi.input': StringIO(''),
-                   'REQUEST_METHOD': method, 'SERVER_NAME': server_name,
-                   'SERVER_PORT': server_port, 'SCRIPT_NAME': script_name}
-        environ.update(kwargs)
-        return environ
+    def test_create_report(self):
+        req = MockRequest(self.env, method='POST', args={
+            'action': 'new',
+            'title': "New Report",
+            'query': "SELECT 1",
+            'description': "The description"})
+
+        self.assertRaises(RequestDone,
+                          self.report_module.process_request, req)
+        self.assertTrue(Report(self.env, 9).exists)
+        self.assertIn("The report has been created.",
+                      req.chrome['notices'])
+        self.assertEqual('http://example.org/trac.cgi/report/9',
+                         req.headers_sent['Location'])
+
+    def test_create_report_cancel(self):
+        req = MockRequest(self.env, method='POST', args={
+            'action': 'new',
+            'cancel': True,
+            'title': "New Report",
+            'query': "SELECT 1",
+            'description': "The description"})
+
+        self.assertRaises(RequestDone, self.report_module.process_request,
+                          req)
+        self.assertRaises(ResourceNotFound, Report, self.env, 9)
+        self.assertNotIn("The report has been created.",
+                         req.chrome['notices'])
+        self.assertEqual('http://example.org/trac.cgi/report',
+                         req.headers_sent['Location'])
+
+    def test_delete_report(self):
+        req = MockRequest(self.env, method='POST', args={
+            'action': 'delete',
+            'id': '1'})
+
+        self.assertRaises(RequestDone, self.report_module.process_request,
+                          req)
+        self.assertRaises(ResourceNotFound, Report, self.env, 1)
+        self.assertIn("The report {1} has been deleted.",
+                      req.chrome['notices'])
+        self.assertEqual('http://example.org/trac.cgi/report',
+                         req.headers_sent['Location'])
+
+    def test_delete_report_cancel(self):
+        req = MockRequest(self.env, method='POST', args={
+            'action': 'delete',
+            'cancel': True,
+            'id': '1'})
+
+        self.assertRaises(RequestDone, self.report_module.process_request,
+                          req)
+        self.assertTrue(Report(self.env, 1).exists)
+        self.assertNotIn("The report {1} has been deleted.",
+                         req.chrome['notices'])
+        self.assertEqual('http://example.org/trac.cgi/report/1',
+                         req.headers_sent['Location'])
+
+    def test_update_report(self):
+        report = Report(self.env)
+        report.query = "SELECT 1"
+        report.insert()
+        req = MockRequest(self.env, method='POST', args={
+            'action': 'edit',
+            'id': '1',
+            'title': "New Report edited",
+            'query': "SELECT 2",
+            'description': "The description edited"})
+
+        self.assertRaises(RequestDone, self.report_module.process_request,
+                          req)
+        report = Report(self.env, 1)
+        self.assertEqual("New Report edited", report.title)
+        self.assertEqual("SELECT 2", report.query)
+        self.assertEqual("The description edited", report.description)
+        self.assertIn("Your changes have been saved.", req.chrome['notices'])
+
+    def test_update_report_cancel(self):
+        report = Report(self.env)
+        report.query = "SELECT 1"
+        report.insert()
+        req = MockRequest(self.env, method='POST', args={
+            'action': 'edit',
+            'cancel': True,
+            'id': '1',
+            'title': "New Report edited",
+            'query': "SELECT 2",
+            'description': "The description edited"})
+
+        self.assertRaises(RequestDone, self.report_module.process_request,
+                          req)
+        report = Report(self.env, 1)
+        self.assertEqual("Active Tickets", report.title)
+        self.assertEqual(" * List all active tickets by priority.\n"
+                         " * Color each row based on priority.\n",
+                         report.description)
+        self.assertIn("SELECT p.value AS __color__", report.query)
+        self.assertNotIn("Your changes have been saved.",
+                         req.chrome['notices'])
+
+    def test_render_confirm_delete(self):
+        req = MockRequest(self.env, method='GET', args={
+            'action': 'delete',
+            'id': '1'})
+
+        data = self.report_module.process_request(req)[1]
+
+        self.assertEqual("Delete Report {1} Active Tickets", data['title'])
+        self.assertEqual('delete', data['action'])
+        self.assertEqual(1, data['report']['id'])
+        self.assertEqual('Active Tickets', data['report']['title'])
+
+    def test_render_editor(self):
+        req = MockRequest(self.env, method='GET', args={
+            'action': 'edit',
+            'id': '1'})
+
+        data = self.report_module.process_request(req)[1]
+
+        self.assertEqual(1, data['report']['id'])
+        self.assertEqual("Active Tickets", data['report']['title'])
+        self.assertEqual(" * List all active tickets by priority.\n"
+                         " * Color each row based on priority.\n",
+                         data['report']['description'])
+        self.assertIn("SELECT p.value AS __color__", data['report']['sql'])
+
+    def test_render_list(self):
+        req = MockRequest(self.env, method='GET', args={
+            'action': 'view',
+            'id': '-1'})
+
+        data = self.report_module.process_request(req)[1]
+
+        self.assertEqual('report', data['sort'])
+        self.assertEqual(1, data['asc'])
+        self.assertEqual(8, len(data['reports']))
+        self.assertEqual(1, data['reports'][0][0])
+        self.assertEqual('Active Tickets', data['reports'][0][1])
+        self.assertEqual(" * List all active tickets by priority.\n"
+                         " * Color each row based on priority.\n",
+                         data['reports'][0][2])
+        self.assertTrue('Active Reports', data['reports'][0][3])
+        self.assertTrue('Active Reports', data['reports'][0][4])
+
+    def test_render_view(self):
+        req = MockRequest(self.env, method='GET', args={
+            'action': 'view',
+            'id': '1'})
+
+        data = self.report_module.process_request(req)[1]
+
+        self.assertEqual(1, data['report']['id'])
+        self.assertEqual("{1} Active Tickets", data['title'])
+        self.assertEqual(" * List all active tickets by priority.\n"
+                         " * Color each row based on priority.\n",
+                         data['description'])
 
     def test_sub_var_no_quotes(self):
         sql, values, missing_args = self.report_module.sql_sub_vars(
@@ -73,7 +211,9 @@ class ReportTestCase(unittest.TestCase):
     def test_sub_var_quotes(self):
         sql, values, missing_args = self.report_module.sql_sub_vars(
             "'$VAR'", {'VAR': 'value'})
-        self.assertEqual(self.env.get_read_db().concat("''", '%s', "''"), sql)
+        with self.env.db_query as db:
+            concatenated = db.concat("''", '%s', "''")
+        self.assertEqual(concatenated, sql)
         self.assertEqual(['value'], values)
         self.assertEqual([], missing_args)
 
@@ -85,51 +225,38 @@ class ReportTestCase(unittest.TestCase):
         self.assertEqual(['PARAM', 'MISSING'], missing_args)
 
     def test_csv_escape(self):
-        buf = StringIO()
-        def start_response(status, headers):
-            return buf.write
-        environ = self._make_environ()
-        req = Request(environ, start_response)
+        req = MockRequest(self.env)
         cols = ['TEST_COL', 'TEST_ZERO']
         rows = [('value, needs escaped', 0)]
-        try:
-            self.report_module._send_csv(req, cols, rows)
-        except RequestDone:
-            pass
-        self.assertEqual('\xef\xbb\xbfTEST_COL,TEST_ZERO\r\n"value, needs escaped",0\r\n',
-                         buf.getvalue())
+
+        self.assertRaises(RequestDone,
+                          self.report_module._send_csv, req, cols, rows)
+        self.assertEqual('\xef\xbb\xbfTEST_COL,TEST_ZERO\r\n"'
+                         'value, needs escaped",0\r\n',
+                         req.response_sent.getvalue())
 
     def test_saved_custom_query_redirect(self):
         query = u'query:?type=résumé'
-        db = self.env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute("INSERT INTO report (title,query,description) "
-                       "VALUES (%s,%s,%s)", ('redirect', query, ''))
-        id = db.get_last_id(cursor, 'report')
-        db.commit()
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            cursor.execute("INSERT INTO report (title,query,description) "
+                           "VALUES (%s,%s,%s)", ('redirect', query, ''))
+            id = db.get_last_id(cursor, 'report')
+        req = MockRequest(self.env)
 
-        headers_sent = {}
-        def start_response(status, headers):
-            headers_sent.update(dict(headers))
-        environ = self._make_environ()
-        req = Request(environ, start_response)
-        req.authname = 'anonymous'
-        req.session = Mock(save=lambda: None)
         self.assertRaises(RequestDone,
                           self.report_module._render_view, req, id)
-        self.assertEqual('http://example.org/trac/query?' + \
+        self.assertEqual('http://example.org/trac.cgi/query?' +
                          'type=r%C3%A9sum%C3%A9&report=' + str(id),
-                         headers_sent['Location'])
+                         req.headers_sent['Location'])
 
     def test_quoted_id_with_var(self):
-        req = Mock(base_path='', chrome={}, args={}, session={},
-                   abs_href=Href('/'), href=Href('/'), locale='',
-                   perm=MockPerm(), authname=None, tz=None)
-        db = self.env.get_read_db()
+        req = MockRequest(self.env)
         name = """%s"`'%%%?"""
-        sql = 'SELECT 1 AS %s, $USER AS user' % db.quote(name)
-        rv = self.report_module.execute_paginated_report(req, db, 1, sql,
-                                                         {'USER': 'joe'})
+        with self.env.db_query as db:
+            sql = 'SELECT 1 AS %s, $USER AS user' % db.quote(name)
+            rv = self.report_module.execute_paginated_report(req, 1, sql,
+                                                             {'USER': 'joe'})
         self.assertEqual(5, len(rv), repr(rv))
         cols, results, num_items, missing_args, limit_offset = rv
         self.assertEqual([name, 'user'], cols)
@@ -137,6 +264,71 @@ class ReportTestCase(unittest.TestCase):
         self.assertEqual([], missing_args)
         self.assertEqual(None, limit_offset)
 
+    def test_with_no_sql(self):
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            cursor.execute("""INSERT INTO report (title,query,description)
+                              VALUES ('#12709', '', '')""")
+            id_ = db.get_last_id(cursor, 'report')
+        req = MockRequest(self.env, path_info='/report/%d' % id_)
+        self.assertTrue(self.report_module.match_request(req))
+        data = self.report_module.process_request(req)[1]
+        self.assertEqual('Report failed: Report {%d} has no SQL query.' % id_,
+                         data['message'])
+
+    def test_value_error_from_get_var_args(self):
+        req = MockRequest(self.env, path_info='/report/1')
+        def get_var_args(req):
+            raise ValueError('<error>')
+        self.report_module.get_var_args = get_var_args
+        self.assertTrue(self.report_module.match_request(req))
+        data = self.report_module.process_request(req)[1]
+        self.assertEqual('Report failed: <error>', data['message'])
+
+    def test_title_and_description_with_sub_vars(self):
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            cursor.execute(
+                """INSERT INTO report (title,query,description)
+                   VALUES (%s,%s,%s)""",
+                ('Tickets on $M for $USER',
+                 'SELECT * FROM ticket WHERE milestone=$M AND owner=$USER',
+                 'Show tickets on $M for $USER'))
+            id_ = db.get_last_id(cursor, 'report')
+            for milestone in ('milestone1', 'milestone2'):
+                ticket = Ticket(self.env)
+                ticket.populate({'status': 'new', 'summary': 'Test 1',
+                                 'owner': 'joe', 'milestone': milestone})
+                ticket.insert()
+        req = MockRequest(self.env, path_info='/report/%d' % id_,
+                          authname='joe', args={'M': 'milestone2'})
+        self.assertTrue(self.report_module.match_request(req))
+        data = self.report_module.process_request(req)[1]
+        self.assertEqual('{%d} Tickets on milestone2 for joe' % id_,
+                         data['title'])
+        self.assertEqual('Show tickets on milestone2 for joe',
+                         data['description'])
+        self.assertIsNone(data['message'])
+        self.assertEqual(1, data['numrows'])
+
+    def test_clear(self):
+        req = MockRequest(self.env, method='GET', path_info='/report',
+                          args={'action': 'clear'})
+        req.session['query_href'] = req.href.query()
+        req.session['query_tickets'] = '42'
+        self.assertTrue(self.report_module.match_request(req))
+        self.assertEqual('report_list.html',
+                         self.report_module.process_request(req)[0])
+        self.assertIn('query_href', req.session)
+        self.assertIn('query_tickets', req.session)
+
+        req = MockRequest(self.env, method='POST', path_info='/report',
+                          args={'action': 'clear'})
+        self.assertTrue(self.report_module.match_request(req))
+        self.assertRaises(RequestDone, self.report_module.process_request, req)
+        self.assertNotIn('query_href', req.session)
+        self.assertNotIn('query_tickets', req.session)
+
 
 class ExecuteReportTestCase(unittest.TestCase):
 
@@ -166,9 +358,8 @@ class ExecuteReportTestCase(unittest.TestCase):
     def _execute_report(self, id, args=None):
         mod = self.report_module
         req = MockRequest(self.env)
-        with self.env.db_query as db:
-            title, description, sql = mod.get_report(id)
-            return mod.execute_paginated_report(req, db, id, sql, args or {})
+        report = Report(self.env, id)
+        return mod.execute_paginated_report(req, id, report.query, args or {})
 
     def _generate_tickets(self, columns, data, attrs):
         with self.env.db_transaction as db:
@@ -487,8 +678,8 @@ class ExecuteReportTestCase(unittest.TestCase):
     def test_report_8_active_tickets_mine_first(self):
         attrs = dict(component='component1', milestone='milestone1',
                      version='1.0', type='defect')
-        tickets = self._generate_tickets(('status', 'owner', 'priority'),
-                                         self.REPORT_8_DATA, attrs)
+        self._generate_tickets(('status', 'owner', 'priority'),
+                                self.REPORT_8_DATA, attrs)
 
         rv = self._execute_report(8, {'USER': 'john'})
         cols, results, num_items, missing_args, limit_offset = rv
@@ -515,13 +706,29 @@ class ExecuteReportTestCase(unittest.TestCase):
         self.assertEqual(['Active Tickets'],
                          sorted(set(r[idx_group] for r in results)))
 
+    def test_execute_paginated_report_legacy_signature(self):
+        """`execute_paginated_report` returns the same results with and
+        without the deprecated `db` argument."""
+        id = 1
+        attrs = dict(reporter='joe', component='component1', version='1.0',
+                     milestone='milestone1', type='defect', owner='joe')
+        self._generate_tickets(('status', 'priority'), self.REPORT_1_DATA,
+                               attrs)
+        mod = self.report_module
+        req = MockRequest(self.env)
+        sql = Report(self.env, id).query
+
+        rv1 = mod.execute_paginated_report(req, id, sql, {})
+        with self.env.db_query as db:
+            rv2 = mod.execute_paginated_report(req, db, id, sql, {})
+        self.assertEqual(rv2, rv1)
+
     def test_asc_argument_is_invalid(self):
         """Invalid value for `asc` argument is coerced to default."""
         req = MockRequest(self.env, args={'asc': '--'})
 
-        data = ReportModule(self.env).process_request(req)[1]
-
-        self.assertFalse(data['asc'])
+        self.assertRaises(HTTPBadRequest,
+                          ReportModule(self.env).process_request, req)
 
     def test_invalid_post_request_raises_exception(self):
         req = MockRequest(self.env, method='POST', action=None)
@@ -578,8 +785,7 @@ class NavContribReportModuleEnabledTestCase(NavigationContributorTestCase):
         """No navigation item when user has neither REPORT_VIEW or
         TICKET_VIEW.
         """
-        req = Mock(href=Href('/'),
-                   perm=PermissionCache(self.env, 'anonymous'))
+        req = MockRequest(self.env, authname='anonymous')
 
         navigation_items = self.get_navigation_items(req, self.report_module)
         self.assertEqual(0, len(navigation_items))
@@ -592,11 +798,10 @@ class NavContribReportModuleEnabledTestCase(NavigationContributorTestCase):
         enabled
         and the user has REPORT_VIEW.
         """
-        req = Mock(href=Href('/'),
-                   perm=PermissionCache(self.env, 'has_report_view'))
+        req = MockRequest(self.env, authname='has_report_view')
 
         navigation_items = self.get_navigation_items(req, self.report_module)
-        self.assertNavItem('/report', navigation_items)
+        self.assertNavItem('/trac.cgi/report', navigation_items)
 
         navigation_items = self.get_navigation_items(req, self.query_module)
         self.assertEqual(0, len(navigation_items))
@@ -605,24 +810,22 @@ class NavContribReportModuleEnabledTestCase(NavigationContributorTestCase):
         """Navigation item directs to QueryModule when ReportModule is
         enabled and the user has TICKET_VIEW but not REPORT_VIEW.
         """
-        req = Mock(href=Href('/'),
-                   perm=PermissionCache(self.env, 'has_ticket_view'))
+        req = MockRequest(self.env, authname='has_ticket_view')
 
         navigation_items = self.get_navigation_items(req, self.report_module)
         self.assertEqual(0, len(navigation_items))
 
         navigation_items = self.get_navigation_items(req, self.query_module)
-        self.assertNavItem('/query', navigation_items)
+        self.assertNavItem('/trac.cgi/query', navigation_items)
 
     def test_user_has_report_view_and_ticket_view(self):
         """Navigation item directs to ReportModule when ReportModule is
          enabled and the user has REPORT_VIEW and TICKET_VIEW.
         """
-        req = Mock(href=Href('/'),
-                   perm=PermissionCache(self.env, 'has_both'))
+        req = MockRequest(self.env, authname='has_both')
 
         navigation_items = self.get_navigation_items(req, self.report_module)
-        self.assertNavItem('/report', navigation_items)
+        self.assertNavItem('/trac.cgi/report', navigation_items)
 
         navigation_items = self.get_navigation_items(req, self.query_module)
         self.assertEqual(0, len(navigation_items))
@@ -638,21 +841,19 @@ class NavContribReportModuleDisabledTestCase(NavigationContributorTestCase):
         """Navigation item directs to QueryModule when ReportModule is
         disabled and the user has TICKET_VIEW.
         """
-        req = Mock(href=Href('/'),
-                   perm=PermissionCache(self.env, 'has_ticket_view'))
+        req = MockRequest(self.env, authname='has_ticket_view')
 
         navigation_items = self.get_navigation_items(req, self.report_module)
         self.assertEqual(0, len(navigation_items))
 
         navigation_items = self.get_navigation_items(req, self.query_module)
-        self.assertNavItem('/query', navigation_items)
+        self.assertNavItem('/trac.cgi/query', navigation_items)
 
     def test_user_no_ticket_view(self):
         """No Navigation item when ReportModule is disabled and the user
         has only REPORT_VIEW.
         """
-        req = Mock(href=Href('/'),
-                   perm=PermissionCache(self.env, 'has_report_view'))
+        req = MockRequest(self.env, authname='has_report_view')
 
         navigation_items = self.get_navigation_items(req, self.report_module)
         self.assertEqual(0, len(navigation_items))
@@ -661,14 +862,14 @@ class NavContribReportModuleDisabledTestCase(NavigationContributorTestCase):
         self.assertEqual(0, len(navigation_items))
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(trac.ticket.report))
-    suite.addTest(unittest.makeSuite(ReportTestCase))
+    suite.addTest(unittest.makeSuite(ReportModuleTestCase))
     suite.addTest(unittest.makeSuite(ExecuteReportTestCase))
     suite.addTest(unittest.makeSuite(NavContribReportModuleEnabledTestCase))
     suite.addTest(unittest.makeSuite(NavContribReportModuleDisabledTestCase))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
