@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2013 Edgewall Software
+# Copyright (C) 2006-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import unittest
 from datetime import timedelta
@@ -26,30 +26,42 @@ from trac.wiki.tests import formatter
 
 TICKET_TEST_CASES = u"""
 ============================== ticket: link resolver
+bug:1
+issue:1
 ticket:1
 ticket:12
 ticket:abc
 ------------------------------
 <p>
-<a class="new ticket" href="/ticket/1" title="This is the summary (new)">ticket:1</a>
+<a class="new ticket" href="/ticket/1" title="#1: This is the summary (new)">bug:1</a>
+<a class="new ticket" href="/ticket/1" title="#1: This is the summary (new)">issue:1</a>
+<a class="new ticket" href="/ticket/1" title="#1: This is the summary (new)">ticket:1</a>
 <a class="missing ticket">ticket:12</a>
 <a class="missing ticket">ticket:abc</a>
 </p>
 ------------------------------
 ============================== ticket: link resolver + arguments
-ticket:1?format=csv
+bug:1#comment:3
+issue:1#comment:3
 ticket:1#comment:3
+ticket:1?format=csv
 ------------------------------
 <p>
-<a class="new ticket" href="/ticket/1?format=csv" title="This is the summary (new)">ticket:1?format=csv</a>
-<a class="new ticket" href="/ticket/1#comment:3" title="This is the summary (new)">ticket:1#comment:3</a>
+<a class="new ticket" href="/ticket/1#comment:3" title="#1: This is the summary (new)">bug:1#comment:3</a>
+<a class="new ticket" href="/ticket/1#comment:3" title="#1: This is the summary (new)">issue:1#comment:3</a>
+<a class="new ticket" href="/ticket/1#comment:3" title="#1: This is the summary (new)">ticket:1#comment:3</a>
+<a class="new ticket" href="/ticket/1?format=csv" title="#1: This is the summary (new)">ticket:1?format=csv</a>
 </p>
 ------------------------------
 ============================== ticket: link resolver with ranges
+bug:12-14,33
+issue:12-14,33
 ticket:12-14,33
 ticket:12,33?order=created
 ------------------------------
 <p>
+<a href="/query?id=12-14%2C33" title="Tickets 12-14, 33">bug:12-14,33</a>
+<a href="/query?id=12-14%2C33" title="Tickets 12-14, 33">issue:12-14,33</a>
 <a href="/query?id=12-14%2C33" title="Tickets 12-14, 33">ticket:12-14,33</a>
 <a href="/query?id=12%2C33&amp;order=created" title="Tickets 12, 33">ticket:12,33?order=created</a>
 </p>
@@ -59,7 +71,7 @@ ticket:12,33?order=created
 #12, #abc
 ------------------------------
 <p>
-<a class="new ticket" href="/ticket/1" title="This is the summary (new)">#1</a>, <a class="missing ticket">#2</a>
+<a class="new ticket" href="/ticket/1" title="#1: This is the summary (new)">#1</a>, <a class="missing ticket">#2</a>
 <a class="missing ticket">#12</a>, #abc
 </p>
 ------------------------------
@@ -94,10 +106,10 @@ trac:ticket:2041
 #trac2041
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>trac:ticket:2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>Trac #2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>#T2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in Trac's Trac"><span class="icon"></span>#trac2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>trac:ticket:2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>Trac #2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>#T2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/ticket%3A2041" title="ticket:2041 in The Trac Project"><span class="icon"></span>#trac2041</a>
 </p>
 ------------------------------
 ============================== Ticket InterTrac shorthands
@@ -105,8 +117,8 @@ T:#2041
 trac:#2041
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/%232041" title="#2041 in Trac's Trac"><span class="icon"></span>T:#2041</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/%232041" title="#2041 in Trac's Trac"><span class="icon"></span>trac:#2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/%232041" title="#2041 in The Trac Project"><span class="icon"></span>T:#2041</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/%232041" title="#2041 in The Trac Project"><span class="icon"></span>trac:#2041</a>
 </p>
 ------------------------------
 ============================== ticket syntax with unicode digits
@@ -118,22 +130,24 @@ trac:#2041
 ------------------------------
 <p>
 #⁴²
-<a class="new ticket" href="/ticket/1" title="This is the summary (new)">#1</a>-⁵,42
-<a class="new ticket" href="/ticket/1" title="This is the summary (new)">#1</a>,³,5,7
+<a class="new ticket" href="/ticket/1" title="#1: This is the summary (new)">#1</a>-⁵,42
+<a class="new ticket" href="/ticket/1" title="#1: This is the summary (new)">#1</a>,³,5,7
 #T²⁰⁴¹
 #trac²⁰⁴¹
 </p>
 ------------------------------
 """ # "
 
+
 def ticket_setup(tc):
     config = tc.env.config
     config.set('ticket-custom', 'custom1', 'text')
     config.save()
     ticket = Ticket(tc.env)
-    ticket.values.update({'reporter': 'santa',
-                          'summary': 'This is the summary',
-                          'status': 'new'})
+
+    ticket.populate({'reporter': 'santa',
+                     'summary': 'This is the summary',
+                     'status': 'new'})
     ticket.insert()
 
 def ticket_teardown(tc):
@@ -186,11 +200,11 @@ trac:report:1
 {trac 1}
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>trac:report:1</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>Trac r1</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>{T1}</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>{trac1}</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/report%3A1" title="report:1 in Trac's Trac"><span class="icon"></span>{trac 1}</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>trac:report:1</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>Trac r1</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>{T1}</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>{trac1}</a>
+<a class="ext-link" href="https://trac.edgewall.org/intertrac/report%3A1" title="report:1 in The Trac Project"><span class="icon"></span>{trac 1}</a>
 </p>
 ------------------------------
 ============================== report syntax with unicode digits
@@ -331,11 +345,18 @@ Reopened tickets: [[TicketQuery(status=reopened)]]
 Reopened tickets: <span class="query_no_results">No results</span>
 </p>
 ------------------------------
+============================== TicketQuery macro: no results, count 0 (raw)
+Reopened tickets: [[TicketQuery(status=reopened, format=rawcount)]]
+------------------------------
+<p>
+Reopened tickets: <span class="query_count" title="0 tickets matching status=reopened, max=0, order=id">0</span>
+</p>
+------------------------------
 ============================== TicketQuery macro: no results, count 0
 Reopened tickets: [[TicketQuery(status=reopened, format=count)]]
 ------------------------------
 <p>
-Reopened tickets: <span class="query_count" title="0 tickets for which status=reopened&amp;max=0&amp;order=id">0</span>
+Reopened tickets: <a href="/query?status=reopened&amp;max=0&amp;order=id" title="0 tickets matching status=reopened, max=0, order=id">0</a>
 </p>
 ------------------------------
 ============================== TicketQuery macro: no results, compact form
@@ -352,11 +373,18 @@ New tickets: [[TicketQuery(status=new)]]
 New tickets: </p><div><dl class="wiki compact"><dt><a class="new" href="/ticket/1" title="This is the summary">#1</a></dt><dd>This is the summary</dd></dl></div><p>
 </p>
 ------------------------------
+============================== TicketQuery macro: one result, count 1 (raw)
+New tickets: [[TicketQuery(status=new, format=rawcount)]]
+------------------------------
+<p>
+New tickets: <span class="query_count" title="1 ticket matching status=new, max=0, order=id">1</span>
+</p>
+------------------------------
 ============================== TicketQuery macro: one result, count 1
 New tickets: [[TicketQuery(status=new, format=count)]]
 ------------------------------
 <p>
-New tickets: <span class="query_count" title="1 ticket for which status=new&amp;max=0&amp;order=id">1</span>
+New tickets: <a href="/query?status=new&amp;max=0&amp;order=id" title="1 ticket matching status=new, max=0, order=id">1</a>
 </p>
 ------------------------------
 ============================== TicketQuery macro: one result, compact form
@@ -390,11 +418,18 @@ New tickets: [[TicketQuery(status=new, order=reporter)]]
 New tickets: </p><div><dl class="wiki compact"><dt><a class="new" href="/ticket/2" title="This is another summary">#2</a></dt><dd>This is another summary</dd><dt><a class="new" href="/ticket/1" title="This is the summary">#1</a></dt><dd>This is the summary</dd></dl></div><p>
 </p>
 ------------------------------
+============================== TicketQuery macro: two results, count 2 (raw)
+New tickets: [[TicketQuery(status=new, order=reporter, format=rawcount)]]
+------------------------------
+<p>
+New tickets: <span class="query_count" title="2 tickets matching status=new, max=0, order=reporter">2</span>
+</p>
+------------------------------
 ============================== TicketQuery macro: two results, count 2
 New tickets: [[TicketQuery(status=new, order=reporter, format=count)]]
 ------------------------------
 <p>
-New tickets: <span class="query_count" title="2 tickets for which status=new&amp;max=0&amp;order=reporter">2</span>
+New tickets: <a href="/query?status=new&amp;max=0&amp;order=reporter" title="2 tickets matching status=new, max=0, order=reporter">2</a>
 </p>
 ------------------------------
 ============================== TicketQuery macro: two results, compact form
@@ -408,14 +443,14 @@ New tickets: <span><a class="new" href="/ticket/2" title="This is another summar
 
 def query2_setup(tc):
     ticket = Ticket(tc.env)
-    ticket.values.update({'reporter': 'santa',
-                          'summary': 'This is the summary',
-                          'status': 'new'})
+    ticket.populate({'reporter': 'santa',
+                     'summary': 'This is the summary',
+                     'status': 'new'})
     ticket.insert()
     ticket = Ticket(tc.env)
-    ticket.values.update({'reporter': 'claus',
-                          'summary': 'This is another summary',
-                          'status': 'new'})
+    ticket.populate({'reporter': 'claus',
+                     'summary': 'This is another summary',
+                     'status': 'new'})
     ticket.insert()
 
 def query2_teardown(tc):
@@ -440,10 +475,10 @@ comment:ticket::2 (deprecated)
 comment:ticket:: (deprecated)
 ------------------------------
 <p>
-<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for Ticket #1">comment:ticket:1:1</a> (deprecated)
-<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for Ticket #1">see above</a> (deprecated)
-<a class="new ticket" href="/ticket/1#comment:description" title="Description for Ticket #1">comment:ticket:1:description</a> (deprecated)
-<a class="new ticket" href="/ticket/1#comment:description" title="Description for Ticket #1">see descr</a> (deprecated)
+<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for #1: This is the summary for ticket 1 (new)">comment:ticket:1:1</a> (deprecated)
+<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for #1: This is the summary for ticket 1 (new)">see above</a> (deprecated)
+<a class="new ticket" href="/ticket/1#comment:description" title="Description for #1: This is the summary for ticket 1 (new)">comment:ticket:1:description</a> (deprecated)
+<a class="new ticket" href="/ticket/1#comment:description" title="Description for #1: This is the summary for ticket 1 (new)">see descr</a> (deprecated)
 <a class="ticket" href="/ticket/2#comment:1" title="Comment 1">comment:ticket:2:1</a> (deprecated)
 <a class="missing ticket" title="ticket comment does not exist">comment:ticket:2:3</a> (deprecated)
 <a class="missing ticket" title="ticket does not exist">comment:ticket:3:1</a> (deprecated)
@@ -458,6 +493,7 @@ comment:ticket:: (deprecated)
 ------------------------------
 ============================== comment: link resolver
 comment:1
+comment:₁
 [comment:1 see above]
 comment:description
 [comment:description see descr]
@@ -467,6 +503,7 @@ comment:1a
 ------------------------------
 <p>
 <a class="ticket" href="/ticket/2#comment:1" title="Comment 1">comment:1</a>
+<a class="missing ticket" title="ticket comment does not exist">comment:₁</a>
 <a class="ticket" href="/ticket/2#comment:1" title="Comment 1">see above</a>
 <a class="ticket" href="/ticket/2#comment:description" title="Description">comment:description</a>
 <a class="ticket" href="/ticket/2#comment:description" title="Description">see descr</a>
@@ -476,7 +513,10 @@ comment:1a
 </p>
 ------------------------------
 ============================== comment: link resolver with ticket number
+comment:1:bug:1
+comment:1:issue:1
 comment:1:ticket:1
+comment:₁:ticket:1
 [comment:1:ticket:1 see above]
 comment:description:ticket:1
 [comment:description:ticket:1 see descr]
@@ -492,10 +532,13 @@ comment:2:ticket:
 comment::ticket:
 ------------------------------
 <p>
-<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for Ticket #1">comment:1:ticket:1</a>
-<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for Ticket #1">see above</a>
-<a class="new ticket" href="/ticket/1#comment:description" title="Description for Ticket #1">comment:description:ticket:1</a>
-<a class="new ticket" href="/ticket/1#comment:description" title="Description for Ticket #1">see descr</a>
+<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for #1: This is the summary for ticket 1 (new)">comment:1:bug:1</a>
+<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for #1: This is the summary for ticket 1 (new)">comment:1:issue:1</a>
+<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for #1: This is the summary for ticket 1 (new)">comment:1:ticket:1</a>
+<a class="missing ticket" title="ticket comment does not exist">comment:₁:ticket:1</a>
+<a class="new ticket" href="/ticket/1#comment:1" title="Comment 1 for #1: This is the summary for ticket 1 (new)">see above</a>
+<a class="new ticket" href="/ticket/1#comment:description" title="Description for #1: This is the summary for ticket 1 (new)">comment:description:ticket:1</a>
+<a class="new ticket" href="/ticket/1#comment:description" title="Description for #1: This is the summary for ticket 1 (new)">see descr</a>
 <a class="ticket" href="/ticket/2#comment:1" title="Comment 1">comment:1:ticket:2</a>
 <a class="missing ticket" title="ticket comment does not exist">comment:3:ticket:2</a>
 <a class="missing ticket" title="ticket does not exist">comment:1:ticket:3</a>
@@ -521,15 +564,15 @@ comment::ticket:
 
 def comment_setup(tc):
     ticket1 = Ticket(tc.env)
-    ticket1.values.update({'reporter': 'santa',
-                            'summary': 'This is the summary for ticket 1',
-                            'status': 'new'})
+    ticket1.populate({'reporter': 'santa',
+                      'summary': 'This is the summary for ticket 1',
+                      'status': 'new'})
     ticket1.insert()
     ticket1.save_changes(comment='This is the comment for ticket 1')
     ticket2 = Ticket(tc.env)
-    ticket2.values.update({'reporter': 'claws',
-                           'summary': 'This is the summary for ticket 2',
-                           'status': 'closed'})
+    ticket2.populate({'reporter': 'claws',
+                      'summary': 'This is the summary for ticket 2',
+                      'status': 'closed'})
     ticket2.insert()
     ticket2.save_changes(comment='This is the comment for ticket 2')
 
@@ -537,20 +580,22 @@ def comment_teardown(tc):
     tc.env.reset_db()
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(formatter.suite(TICKET_TEST_CASES, ticket_setup, __file__,
-                                  ticket_teardown))
-    suite.addTest(formatter.suite(REPORT_TEST_CASES, report_setup, __file__))
-    suite.addTest(formatter.suite(MILESTONE_TEST_CASES, milestone_setup,
-                                  __file__, milestone_teardown))
-    suite.addTest(formatter.suite(QUERY_TEST_CASES, ticket_setup, __file__,
-                                  ticket_teardown))
-    suite.addTest(formatter.suite(QUERY2_TEST_CASES, query2_setup, __file__,
-                                  query2_teardown))
-    suite.addTest(formatter.suite(COMMENT_TEST_CASES, comment_setup, __file__,
-                                  comment_teardown, ('ticket', 2)))
+    suite.addTest(formatter.test_suite(TICKET_TEST_CASES, ticket_setup,
+                                       __file__, ticket_teardown))
+    suite.addTest(formatter.test_suite(REPORT_TEST_CASES, report_setup,
+                                       __file__))
+    suite.addTest(formatter.test_suite(MILESTONE_TEST_CASES, milestone_setup,
+                                       __file__, milestone_teardown))
+    suite.addTest(formatter.test_suite(QUERY_TEST_CASES, ticket_setup,
+                                       __file__, ticket_teardown))
+    suite.addTest(formatter.test_suite(QUERY2_TEST_CASES, query2_setup,
+                                       __file__, query2_teardown))
+    suite.addTest(formatter.test_suite(COMMENT_TEST_CASES, comment_setup,
+                                       __file__, comment_teardown,
+                                       ('ticket', 2)))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
