@@ -1,8 +1,21 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007-2019 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at https://trac.edgewall.org/.
+
 
 def do_upgrade(env, ver, cursor):
     """Upgrade the reports to better handle the new workflow capabilities"""
-    db = env.get_db_cnx()
-    owner = db.concat('owner', "' *'")
+    with env.db_query as db:
+        owner = db.concat('owner', "' *'")
     cursor.execute('SELECT id, query, description FROM report')
     reports = cursor.fetchall()
     for report, query, description in reports:
@@ -18,5 +31,6 @@ def do_upgrade(env, ver, cursor):
             d = d.replace(" * If a ticket has been accepted, a '*' is"
                           " appended after the owner's name\n", '')
         if q != query or d != description:
-            cursor.execute("UPDATE report SET query=%s, description=%s "
-                           "WHERE id=%s", (q, d, report))
+            cursor.execute("""
+                UPDATE report SET query=%s, description=%s WHERE id=%s
+                """, (q, d, report))
