@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Matthew Good <trac@matt-good.net>
@@ -27,9 +27,6 @@ import textwrap
 from urllib import quote, quote_plus, unquote
 from unicodedata import east_asian_width
 
-from trac.util.translation import _
-
-
 CRLF = '\r\n'
 
 class Empty(unicode):
@@ -52,8 +49,8 @@ def to_unicode(text, charset=None):
     the original byte sequence by mapping each byte to the corresponding
     unicode code point in the range U+0000 to U+00FF.
 
-    Otherwise, a simple `unicode()` conversion is attempted, with some special
-    care taken for `Exception` objects.
+    For anything else, a simple `unicode()` conversion is attempted,
+    with special care taken with `Exception` objects.
     """
     if isinstance(text, str):
         try:
@@ -61,6 +58,13 @@ def to_unicode(text, charset=None):
         except UnicodeDecodeError:
             return unicode(text, 'latin1')
     elif isinstance(text, Exception):
+        if os.name == 'nt' and isinstance(text, (OSError, IOError)):
+            # the exception might have a localized error string encoded with
+            # ANSI codepage if OSError and IOError on Windows
+            try:
+                return unicode(str(text), 'mbcs')
+            except UnicodeError:
+                pass
         # two possibilities for storing unicode strings in exception data:
         try:
             # custom __str__ method on the exception (e.g. PermissionError)
@@ -72,6 +76,11 @@ def to_unicode(text, charset=None):
 
 
 def exception_to_unicode(e, traceback=False):
+    """Convert an `Exception` to an `unicode` object.
+
+    In addition to `to_unicode`, this representation of the exception
+    also contains the class name and optionally the traceback.
+    """
     message = '%s: %s' % (e.__class__.__name__, to_unicode(e))
     if traceback:
         from trac.util import get_last_traceback
@@ -90,12 +99,46 @@ def path_to_unicode(path):
     return unicode(path)
 
 
+_ws_leading_re = re.compile(ur'\A[\s\u200b]+', re.UNICODE)
+_ws_trailing_re = re.compile(ur'[\s\u200b]+\Z', re.UNICODE)
+
+def stripws(text, leading=True, trailing=True):
+    """Strips unicode white-spaces and ZWSPs from ``text``.
+
+    :param leading: strips leading spaces from ``text`` unless ``leading`` is
+                    `False`.
+    :param trailing: strips trailing spaces from ``text`` unless ``trailing``
+                     is `False`.
+    """
+    if leading:
+        text = _ws_leading_re.sub('', text)
+    if trailing:
+        text = _ws_trailing_re.sub('', text)
+    return text
+
+
+def strip_line_ws(text, leading=True, trailing=True):
+    """Strips unicode white-spaces and ZWSPs from each line of ``text``.
+
+    :param leading: strips leading spaces from ``text`` unless ``leading`` is
+                    `False`.
+    :param trailing: strips trailing spaces from ``text`` unless ``trailing``
+                     is `False`.
+    """
+    lines = re.compile(r'(\n|\r\n|\r)').split(text)
+    if leading:
+        lines[::2] = (_ws_leading_re.sub('', line) for line in lines[::2])
+    if trailing:
+        lines[::2] = (_ws_trailing_re.sub('', line) for line in lines[::2])
+    return ''.join(lines)
+
+
 _js_quote = {'\\': '\\\\', '"': '\\"', '\b': '\\b', '\f': '\\f',
              '\n': '\\n', '\r': '\\r', '\t': '\\t', "'": "\\'"}
-for i in range(0x20) + [ord(c) for c in '&<>']:
-    _js_quote.setdefault(chr(i), '\\u%04x' % i)
-_js_quote_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t\'&<>]')
-_js_string_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t&<>]')
+for i in range(0x20) + [ord(c) for c in u'&<>\u2028\u2029']:
+    _js_quote.setdefault(unichr(i), '\\u%04x' % i)
+_js_quote_re = re.compile(ur'[\x00-\x1f\\"\b\f\n\r\t\'&<>\u2028\u2029]')
+_js_string_re = re.compile(ur'[\x00-\x1f\\"\b\f\n\r\t&<>\u2028\u2029]')
 
 
 def javascript_quote(text):
@@ -114,7 +157,7 @@ def to_js_string(text):
     (conform to the JSON spec)
     """
     if not text:
-        return ''
+        return '""'
     def replace(match):
         return _js_quote[match.group(0)]
     return '"%s"' % _js_string_re.sub(replace, text)
@@ -128,8 +171,8 @@ def unicode_quote(value, safe='/'):
     :param safe: as in `quote`, the characters that would otherwise be
                  quoted but shouldn't here (defaults to '/')
     """
-    return quote(isinstance(value, unicode) and value.encode('utf-8') or
-                 str(value), safe)
+    return quote(value.encode('utf-8') if isinstance(value, unicode)
+                 else str(value), safe)
 
 
 def unicode_quote_plus(value, safe=''):
@@ -141,13 +184,13 @@ def unicode_quote_plus(value, safe=''):
                  otherwise be quoted but shouldn't here (defaults to
                  '/')
     """
-    return quote_plus(isinstance(value, unicode) and value.encode('utf-8') or
-                      str(value), safe)
+    return quote_plus(value.encode('utf-8') if isinstance(value, unicode)
+                      else str(value), safe)
 
 
 def unicode_unquote(value):
     """A unicode aware version of `urllib.unquote`.
-    
+
     :param str: UTF-8 encoded `str` value (for example, as obtained by
                 `unicode_quote`).
     :rtype: `unicode`
@@ -157,7 +200,7 @@ def unicode_unquote(value):
 
 def unicode_urlencode(params, safe=''):
     """A unicode aware version of `urllib.urlencode`.
-    
+
     Values set to `empty` are converted to the key alone, without the
     equal sign.
     """
@@ -168,7 +211,7 @@ def unicode_urlencode(params, safe=''):
         if v is empty:
             l.append(unicode_quote_plus(k, safe))
         else:
-            l.append(unicode_quote_plus(k, safe) + '=' + 
+            l.append(unicode_quote_plus(k, safe) + '=' +
                      unicode_quote_plus(v, safe))
     return '&'.join(l)
 
@@ -180,24 +223,30 @@ def quote_query_string(text):
     """
     return unicode_quote_plus(text, _qs_quote_safe)
 
-def to_utf8(text, charset='iso-8859-15'):
-    """Convert a string to UTF-8, assuming the encoding is either UTF-8, ISO
-    Latin-1, or as specified by the optional `charset` parameter.
 
-    ''Deprecated in 0.10. You should use `unicode` strings only.''
+def to_utf8(text, charset='latin1'):
+    """Convert input to a UTF-8 `str` object.
+
+    If the input is not an `unicode` object, we assume the encoding is
+    already UTF-8, ISO Latin-1, or as specified by the optional
+    *charset* parameter.
     """
-    try:
-        # Do nothing if it's already utf-8
-        u = unicode(text, 'utf-8')
-        return text
-    except UnicodeError:
+    if isinstance(text, str):
         try:
-            # Use the user supplied charset if possible
-            u = unicode(text, charset)
+            u = unicode(text, 'utf-8')
         except UnicodeError:
-            # This should always work
-            u = unicode(text, 'iso-8859-15')
-        return u.encode('utf-8')
+            try:
+                # Use the user supplied charset if possible
+                u = unicode(text, charset)
+            except UnicodeError:
+                # This should always work
+                u = unicode(text, 'latin1')
+        else:
+            # Do nothing if it's already utf-8
+            return text
+    else:
+        u = to_unicode(text)
+    return u.encode('utf-8')
 
 
 class unicode_passwd(unicode):
@@ -206,29 +255,56 @@ class unicode_passwd(unicode):
         return '*******'
 
 
-def console_print(out, *args, **kwargs):
-    cons_charset = getattr(out, 'encoding', None)
+def stream_encoding(stream):
+    """Return the appropriate encoding for the given stream."""
+    encoding = getattr(stream, 'encoding', None)
     # Windows returns 'cp0' to indicate no encoding
-    if cons_charset in (None, 'cp0'):
-        cons_charset = 'utf-8'
-    out.write(' '.join([to_unicode(a).encode(cons_charset, 'replace') 
+    return encoding if encoding not in (None, 'cp0') else 'utf-8'
+
+
+def console_print(out, *args, **kwargs):
+    """Output the given arguments to the console, encoding the output
+    as appropriate.
+
+    :param kwargs: ``newline`` controls whether a newline will be appended
+                   (defaults to `True`)
+    """
+    cons_charset = stream_encoding(out)
+    out.write(' '.join([to_unicode(a).encode(cons_charset, 'replace')
                         for a in args]))
     if kwargs.get('newline', True):
         out.write('\n')
 
 
 def printout(*args, **kwargs):
+    """Do a `console_print` on `sys.stdout`."""
     console_print(sys.stdout, *args, **kwargs)
 
 
 def printerr(*args, **kwargs):
+    """Do a `console_print` on `sys.stderr`."""
     console_print(sys.stderr, *args, **kwargs)
 
 
 def raw_input(prompt):
+    """Input one line from the console and converts it to unicode as
+    appropriate.
+    """
     printout(prompt, newline=False)
     return to_unicode(__builtin__.raw_input(), sys.stdin.encoding)
 
+
+_preferredencoding = locale.getpreferredencoding()
+
+def getpreferredencoding():
+    """Return the encoding, which is retrieved on ahead, according to user
+    preference.
+
+    We should use this instead of `locale.getpreferredencoding()` which
+    is not thread-safe."""
+    return _preferredencoding
+
+
 # -- Plain text formatting
 
 def text_width(text, ambiwidth=1):
@@ -245,38 +321,49 @@ def text_width(text, ambiwidth=1):
 
     cf. http://www.unicode.org/reports/tr11/.
     """
-    twice = ('FW', 'FWA')[ambiwidth == 2]
-    return sum([(1, 2)[east_asian_width(chr) in twice]
+    twice = 'FWA' if ambiwidth == 2 else 'FW'
+    return sum([2 if east_asian_width(chr) in twice else 1
                 for chr in to_unicode(text)])
 
-_default_ambiwidth = 1  # Default width of East Asian Ambiguous (A)
-if os.name == 'nt':
-    try:
-        # `ctypes` is available since Python 2.5
+
+def _get_default_ambiwidth():
+    """Return width of East Asian Ambiguous based on locale environment
+    variables or Windows codepage.
+    """
+
+    if os.name == 'nt':
         import ctypes
         codepage = ctypes.windll.kernel32.GetConsoleOutputCP()
-    except ImportError:
-        # Try to retrieve the codepage from stderr and stdout
-        codepage = (sys.stderr.encoding or sys.stdout.encoding or '')[2:]
-        codepage = codepage.isdigit() and int(codepage) or 0
+        if codepage in (932,   # Japanese (Shift-JIS)
+                        936,   # Chinese Simplified (GB2312)
+                        949,   # Korean (Unified Hangul Code)
+                        950):  # Chinese Traditional (Big5)
+            return 2
+    else:
+        for name in ('LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG'):
+            value = os.environ.get(name) or ''
+            if value:
+                if name == 'LANGUAGE' and ':' in value:
+                    value = value.split(':')[0]
+                return 2 if value.lower().startswith(('zh', 'ja', 'ko')) else 1
 
-    if codepage in (932,  # Japanese (Shift-JIS)
-                    936,  # Chinese Simplified (GB2312)
-                    949,  # Korean (Unified Hangul Code)
-                    950): # Chinese Traditional (Big5)
-        _default_ambiwidth = 2
-    del codepage
-else:
-    if re.match(r'zh|ja|kr', os.environ.get('LANG') or '', re.IGNORECASE):
-        _default_ambiwidth = 2
+    return 1
+
+
+_default_ambiwidth = _get_default_ambiwidth()
 
 
 def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
-    """Print `data` as a table in the terminal.
+    """Print data according to a tabular layout.
 
-    That `ambiwidth` parameter is used for the column width of the East
-    Asian Ambiguous (A). If None, detect ambiwidth with the locale settings.
-    If others, pass to the `ambiwidth` parameter of `text_width`.
+    :param data: a sequence of rows; assume all rows are of equal length.
+    :param headers: an optional row containing column headers; must be of
+                    the same length as each row in `data`.
+    :param sep: column separator
+    :param out: output file descriptor (`None` means use `sys.stdout`)
+    :param ambiwidth: column width of the East Asian Ambiguous (A). If None,
+                      detect ambiwidth with the locale settings. If others,
+                      pass to the `ambiwidth` parameter of `text_width`.
     """
     if out is None:
         out = sys.stdout
@@ -299,25 +386,40 @@ def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
     def tw(text):
         return text_width(text, ambiwidth=ambiwidth)
 
-    # Convert each cell to an unicode object
-    data = [[to_text(cell) for cell in row] for row in data]
+    def to_lines(data):
+        lines = []
+        for row in data:
+            row = [to_text(cell) for cell in row]
+            if any('\n' in cell for cell in row):
+                row = [cell.splitlines() for cell in row]
+                max_lines = max(len(cell) for cell in row)
+                for cell in row:
+                    if len(cell) < max_lines:
+                        cell += [''] * (max_lines - len(cell))
+                lines.extend([cell[idx] for cell in row]
+                             for idx in xrange(max_lines))
+            else:
+                lines.append(row)
+        return lines
 
-    num_cols = len(data[0]) # assumes all rows are of equal length
+    data = to_lines(data)
+
+    num_cols = len(data[0])
     col_width = [max(tw(row[idx]) for row in data)
                  for idx in xrange(num_cols)]
 
     out.write('\n')
     for ridx, row in enumerate(data):
         for cidx, cell in enumerate(row):
-            if headers and ridx == 0:
-                sp = '%*s' % (tw(sep), ' ') # No separator in header
-            else:
-                sp = sep
             if cidx + 1 == num_cols:
-                sp = '' # No separator after last column
-
-            line = u'%-*s%s' % (col_width[cidx] - tw(cell) + len(cell),
-                                cell, sp)
+                line = cell  # No separator after last column
+            else:
+                if headers and ridx == 0:
+                    sp = ' ' * tw(sep)  # No separator in header
+                else:
+                    sp = sep
+                line = u'%-*s%s' % (col_width[cidx] - tw(cell) + len(cell),
+                                    cell, sp)
             line = line.encode(charset, 'replace')
             out.write(line)
 
@@ -325,17 +427,23 @@ def print_table(data, headers=None, sep='  ', out=None, ambiwidth=None):
         if ridx == 0 and headers:
             out.write('-' * (tw(sep) * cidx + sum(col_width)))
             out.write('\n')
-
     out.write('\n')
 
 
 def shorten_line(text, maxlen=75):
-    if len(text or '') < maxlen:
+    """Truncates `text` to length less than or equal to `maxlen` characters.
+
+    This tries to be (a bit) clever and attempts to find a proper word
+    boundary for doing so.
+    """
+    if len(text or '') <= maxlen:
         return text
-    cut = max(text.rfind(' ', 0, maxlen), text.rfind('\n', 0, maxlen))
+    suffix = ' ...'
+    maxtextlen = maxlen - len(suffix)
+    cut = max(text.rfind(' ', 0, maxtextlen), text.rfind('\n', 0, maxtextlen))
     if cut < 0:
-        cut = maxlen
-    return text[:cut] + ' ...'
+        cut = maxtextlen
+    return text[:cut] + suffix
 
 
 class UnicodeTextWrapper(textwrap.TextWrapper):
@@ -484,15 +592,29 @@ def wrap(t, cols=75, initial_indent='', subsequent_indent='',
     return linesep.join(wrappedLines)
 
 
+_obfuscation_char = u'@\u2026'
+
 def obfuscate_email_address(address):
+    """Replace anything looking like an e-mail address (``'@something'``)
+    with a trailing ellipsis (``'@…'``)
+    """
     if address:
         at = address.find('@')
         if at != -1:
-            return address[:at] + u'@\u2026' + \
-                   (address[-1] == '>' and '>' or '')
+            return address[:at] + _obfuscation_char + \
+                   ('>' if address[-1] == '>' else '')
     return address
 
 
+def is_obfuscated(word):
+    """Returns `True` if the `word` looks like an obfuscated e-mail
+    address.
+
+    :since: 1.2
+    """
+    return _obfuscation_char in word
+
+
 def breakable_path(path):
     """Make a path breakable after path separators, and conversely, avoid
     breaking at spaces.
@@ -518,14 +640,35 @@ def normalize_whitespace(text, to_space=u'\u00a0', remove=u'\u200b'):
         text = text.replace(each, '')
     return text
 
+
+def unquote_label(txt):
+    """Remove (one level of) enclosing single or double quotes.
+
+    .. versionadded :: 1.0
+    """
+    return txt[1:-1] if txt and txt[0] in "'\"" and txt[0] == txt[-1] else txt
+
+
+def cleandoc(message):
+    """Removes uniform indentation and leading/trailing whitespace."""
+    from inspect import cleandoc
+    return cleandoc(message).strip()
+
+
 # -- Conversion
 
 def pretty_size(size, format='%.1f'):
+    """Pretty print content size information with appropriate unit.
+
+    :param size: number of bytes
+    :param format: can be used to adjust the precision shown
+    """
     if size is None:
         return ''
 
     jump = 1024
     if size < jump:
+        from trac.util.translation import _
         return _('%(size)s bytes', size=size)
 
     units = ['KB', 'MB', 'GB', 'TB']
@@ -538,6 +681,16 @@ def pretty_size(size, format='%.1f'):
 
 
 def expandtabs(s, tabstop=8, ignoring=None):
+    """Expand tab characters `'\\\\t'` into spaces.
+
+    :param tabstop: number of space characters per tab
+                    (defaults to the canonical 8)
+
+    :param ignoring: if not `None`, the expansion will be "smart" and
+                     go from one tabstop to the next. In addition,
+                     this parameter lists characters which can be
+                     ignored when computing the indent.
+    """
     if '\t' not in s:
         return s
     if ignoring is None:
@@ -572,7 +725,7 @@ def fix_eol(text, eol):
 
 def unicode_to_base64(text, strip_newlines=True):
     """Safe conversion of ``text`` to base64 representation using
-    utf-8 bytes.  
+    utf-8 bytes.
 
     Strips newlines from output unless ``strip_newlines`` is `False`.
     """
@@ -603,3 +756,17 @@ def levenshtein_distance(lhs, rhs):
                             prev[ridx] + cost)) # substitution
         prev = curr
     return prev[-1]
+
+
+sub_vars_re = re.compile("[$]([A-Z_][A-Z0-9_]*)")
+
+def sub_vars(text, args):
+    """Substitute $XYZ-style variables in a string with provided values.
+
+    :param text: string containing variables to substitute.
+    :param args: dictionary with keys matching the variables to be substituted.
+                 The keys should not be prefixed with the $ character."""
+    def repl(match):
+        key = match.group(1)
+        return args[key] if key in args else '$' + key
+    return sub_vars_re.sub(repl, text)
