@@ -1,46 +1,45 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2010 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
+import re
 from datetime import datetime, timedelta
 from fnmatch import fnmatchcase
-import re
 
 from genshi.builder import tag
 
-from trac.config import ListOption, BoolOption, Option
+from trac.config import BoolOption, ListOption, Option
 from trac.core import *
 from trac.mimeview.api import IHTMLPreviewAnnotator, Mimeview, is_binary
 from trac.perm import IPermissionRequestor, PermissionError
 from trac.resource import Resource, ResourceNotFound
 from trac.util import as_bool, embedded_numbers
-from trac.util.datefmt import http_date, to_datetime, utc
-from trac.util.html import escape, Markup
+from trac.util.datefmt import datetime_now, http_date, to_datetime, utc
+from trac.util.html import Markup, escape
 from trac.util.text import exception_to_unicode, shorten_line
 from trac.util.translation import _, cleandoc_
+from trac.versioncontrol.api import NoSuchChangeset, RepositoryManager
+from trac.versioncontrol.web_ui.util import *
 from trac.web.api import IRequestHandler, RequestDone
 from trac.web.chrome import (INavigationContributor, add_ctxtnav, add_link,
                              add_script, add_stylesheet, prevnext_nav,
                              web_context)
-from trac.wiki.api import IWikiSyntaxProvider, IWikiMacroProvider, parse_args
+from trac.wiki.api import IWikiMacroProvider, IWikiSyntaxProvider, parse_args
 from trac.wiki.formatter import format_to_html, format_to_oneliner
 
-from ..api import NoSuchChangeset, RepositoryManager
-from trac.versioncontrol.web_ui.util import * # `from .util import *` FIXME 2.6
-
 
 CHUNK_SIZE = 4096
 
@@ -55,6 +54,7 @@ class IPropertyRenderer(Interface):
          - 'browser' rendered in the browser view
          - 'changeset' rendered in the changeset view as a node property
          - 'revprop' rendered in the changeset view as a revision property
+
         Other identifiers might be used by plugins, so it's advised to simply
         ignore unknown modes.
 
@@ -73,13 +73,13 @@ class IPropertyRenderer(Interface):
         (i.e. the `node.get_properties()`).
 
         The rendered result can be one of the following:
-        - `None`: the property will be skipped
-        - an `unicode` value: the property will be displayed as text
-        - a `RenderedProperty` instance: the property will only be displayed
-          using the instance's `content` attribute, and the other attributes
-          will also be used in some display contexts (like `revprop`)
-        - `Markup` or other Genshi content: the property will be displayed
-          normally, using that content as a block-level markup
+         - `None`: the property will be skipped
+         - an `unicode` value: the property will be displayed as text
+         - a `RenderedProperty` instance: the property will only be displayed
+           using the instance's `content` attribute, and the other attributes
+           will also be used in some display contexts (like `revprop`)
+         - `Markup` or other Genshi content: the property will be displayed
+           normally, using that content as a block-level markup
         """
 
 
@@ -118,15 +118,13 @@ class WikiPropertyRenderer(Component):
                                  'trac:description',
         doc="""Comma-separated list of version control properties to render
         as wiki content in the repository browser.
-
-        (''since 0.11'')""")
+        """)
 
     oneliner_properties = ListOption('browser', 'oneliner_properties',
                                  'trac:summary',
         doc="""Comma-separated list of version control properties to render
         as oneliner wiki content in the repository browser.
-
-        (''since 0.11'')""")
+        """)
 
     def match_property(self, name, mode):
         return 4 if name in self.wiki_properties \
@@ -180,6 +178,8 @@ class BrowserModule(Component):
 
     property_renderers = ExtensionPoint(IPropertyRenderer)
 
+    realm = RepositoryManager.source_realm
+
     downloadable_paths = ListOption('browser', 'downloadable_paths',
                                     '/trunk, /branches/*, /tags/*',
         doc="""List of repository paths that can be downloaded.
@@ -191,14 +191,14 @@ class BrowserModule(Component):
         repository name if the path does not point to the default repository
         (e.g. /reponame/trunk). Note that a simple prefix matching is
         performed on the paths, so aliases won't get automatically resolved.
-        (''since 0.10'')""")
+        """)
 
     color_scale = BoolOption('browser', 'color_scale', True,
         doc="""Enable colorization of the ''age'' column.
 
         This uses the same color scale as the source code annotation:
         blue is older, red is newer.
-        (''since 0.11'')""")
+        """)
 
     NEWEST_COLOR = (255, 136, 136)
 
@@ -206,7 +206,7 @@ class BrowserModule(Component):
         doc="""(r,g,b) color triple to use for the color corresponding
         to the newest color, for the color scale used in ''blame'' or
         the browser ''age'' column if `color_scale` is enabled.
-        (''since 0.11'')""")
+        """)
 
     OLDEST_COLOR = (136, 136, 255)
 
@@ -214,13 +214,13 @@ class BrowserModule(Component):
         doc="""(r,g,b) color triple to use for the color corresponding
         to the oldest color, for the color scale used in ''blame'' or
         the browser ''age'' column if `color_scale` is enabled.
-        (''since 0.11'')""")
+        """)
 
     intermediate_point = Option('browser', 'intermediate_point', '',
         doc="""If set to a value between 0 and 1 (exclusive), this will be the
         point chosen to set the `intermediate_color` for interpolating
         the color value.
-        (''since 0.11'')""")
+        """)
 
     intermediate_color = Option('browser', 'intermediate_color', '',
         doc="""(r,g,b) color triple to use for the color corresponding
@@ -228,7 +228,7 @@ class BrowserModule(Component):
         for the color scale (see `intermediate_point`).
         If not set, the intermediate color between `oldest_color` and
         `newest_color` will be used.
-        (''since 0.11'')""")
+        """)
 
     render_unsafe_content = BoolOption('browser', 'render_unsafe_content',
                                         'false',
@@ -240,12 +240,12 @@ class BrowserModule(Component):
         scripting attacks.
 
         For open repositories where anyone can check-in a file, it is
-        recommended to leave this option disabled (which is the default).""")
+        recommended to leave this option disabled.""")
 
     hidden_properties = ListOption('browser', 'hide_properties', 'svk:merge',
         doc="""Comma-separated list of version control properties to hide from
         the repository browser.
-        (''since 0.9'')""")
+        """)
 
     # public methods
 
@@ -332,13 +332,12 @@ class BrowserModule(Component):
             req.redirect(presel)
 
         path = req.args.get('path', '/')
-        rev = req.args.get('rev', '')
+        rev = req.args.getfirst('rev', '')
         if rev.lower() in ('', 'head'):
             rev = None
         format = req.args.get('format')
         order = req.args.get('order', 'name').lower()
         desc = 'desc' in req.args
-        xhr = req.get_header('X-Requested-With') == 'XMLHttpRequest'
 
         rm = RepositoryManager(self.env)
         all_repositories = rm.get_all_repositories()
@@ -374,9 +373,8 @@ class BrowserModule(Component):
                 # as a special shortcut to the latest revision.
                 rev_or_latest = rev or repos.youngest_rev
                 node = get_existing_node(req, repos, path, rev_or_latest)
-            except NoSuchChangeset, e:
-                raise ResourceNotFound(e.message,
-                                       _('Invalid changeset number'))
+            except NoSuchChangeset as e:
+                raise ResourceNotFound(e, _('Invalid changeset number'))
             if node:
                 try:
                     # use changeset instance to retrieve branches and tags
@@ -384,7 +382,7 @@ class BrowserModule(Component):
                 except NoSuchChangeset:
                     pass
 
-            context = context.child(repos.resource.child('source', path,
+            context = context.child(repos.resource.child(self.realm, path,
                                                    version=rev_or_latest))
             display_rev = repos.display_rev
 
@@ -418,7 +416,7 @@ class BrowserModule(Component):
                 raise ResourceNotFound(_("No node %(path)s", path=path))
 
         quickjump_data = properties_data = None
-        if node and not xhr:
+        if node and not req.is_xhr:
             properties_data = self.render_properties(
                     'browser', context, node.get_properties())
             quickjump_data = list(repos.get_quickjump_entries(rev))
@@ -437,9 +435,9 @@ class BrowserModule(Component):
             'quickjump_entries': quickjump_data,
             'wiki_format_messages': \
                 self.config['changeset'].getbool('wiki_format_messages'),
-            'xhr': xhr,
+            'xhr': req.is_xhr,  # Remove in 1.3.1
         }
-        if xhr: # render and return the content only
+        if req.is_xhr: # render and return the content only
             return 'dir_entries.html', data, None
 
         if dir_data or repo_data:
@@ -518,7 +516,7 @@ class BrowserModule(Component):
                 continue
             try:
                 repos = rm.get_repository(reponame)
-            except TracError, err:
+            except TracError as err:
                 entry = (reponame, repoinfo, None, None,
                          exception_to_unicode(err), None)
             else:
@@ -541,7 +539,7 @@ class BrowserModule(Component):
                 else:
                     entry = (reponame, repoinfo, None, None, u"\u2013", None)
             if entry[4] is not None:  # Check permission in case of error
-                root = Resource('repository', reponame).child('source', '/')
+                root = Resource('repository', reponame).child(self.realm, '/')
                 if 'BROWSER_VIEW' not in context.perm(root):
                     continue
             repositories.append(entry)
@@ -587,7 +585,7 @@ class BrowserModule(Component):
         if rev:
             newest = repos.get_changeset(rev).date
         else:
-            newest = datetime.now(req.tz)
+            newest = datetime_now(req.tz)
 
         # Color scale for the age column
         timerange = custom_colorizer = None
@@ -667,78 +665,77 @@ class BrowserModule(Component):
         mimeview = Mimeview(self.env)
 
         # MIME type detection
-        content = node.get_processed_content()
-        chunk = content.read(CHUNK_SIZE)
-        mime_type = node.content_type
-        if not mime_type or mime_type == 'application/octet-stream':
-            mime_type = mimeview.get_mimetype(node.name, chunk) or \
-                        mime_type or 'text/plain'
+        with content_closing(node.get_processed_content()) as content:
+            chunk = content.read(CHUNK_SIZE)
+            mime_type = node.content_type
+            if not mime_type or mime_type == 'application/octet-stream':
+                mime_type = mimeview.get_mimetype(node.name, chunk) or \
+                            mime_type or 'text/plain'
 
-        # Eventually send the file directly
-        format = req.args.get('format')
-        if format in ('raw', 'txt'):
-            req.send_response(200)
-            req.send_header('Content-Type',
-                            'text/plain' if format == 'txt' else mime_type)
-            req.send_header('Last-Modified', http_date(node.last_modified))
-            if rev is None:
-                req.send_header('Pragma', 'no-cache')
-                req.send_header('Cache-Control', 'no-cache')
-                req.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
-            if not self.render_unsafe_content:
-                # Force browser to download files instead of rendering
-                # them, since they might contain malicious code enabling
-                # XSS attacks
-                req.send_header('Content-Disposition', 'attachment')
-            req.end_headers()
-            # Note: don't pass an iterable instance to RequestDone, instead
-            # call req.write() with each chunk here to avoid SEGVs (#11805)
-            while chunk:
-                req.write(chunk)
-                chunk = content.read(CHUNK_SIZE)
-            raise RequestDone
-        else:
-            # The changeset corresponding to the last change on `node`
-            # is more interesting than the `rev` changeset.
-            changeset = repos.get_changeset(node.created_rev)
+            # Eventually send the file directly
+            format = req.args.get('format')
+            if format in ('raw', 'txt'):
+                req.send_response(200)
+                req.send_header('Content-Type',
+                                'text/plain' if format == 'txt' else mime_type)
+                req.send_header('Last-Modified', http_date(node.last_modified))
+                if rev is None:
+                    req.send_header('Pragma', 'no-cache')
+                    req.send_header('Cache-Control', 'no-cache')
+                    req.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
+                if not self.render_unsafe_content:
+                    # Force browser to download files instead of rendering
+                    # them, since they might contain malicious code enabling
+                    # XSS attacks
+                    req.send_header('Content-Disposition', 'attachment')
+                req.end_headers()
+                # Note: don't pass an iterable instance to RequestDone, instead
+                # call req.write() with each chunk here to avoid SEGVs (#11805)
+                while chunk:
+                    req.write(chunk)
+                    chunk = content.read(CHUNK_SIZE)
+                raise RequestDone
 
-            # add ''Plain Text'' alternate link if needed
-            if not is_binary(chunk) and mime_type != 'text/plain':
-                plain_href = req.href.browser(repos.reponame or None,
-                                              node.path, rev=rev, format='txt')
-                add_link(req, 'alternate', plain_href, _('Plain Text'),
-                         'text/plain')
+        # The changeset corresponding to the last change on `node`
+        # is more interesting than the `rev` changeset.
+        changeset = repos.get_changeset(node.created_rev)
 
-            # add ''Original Format'' alternate link (always)
-            raw_href = req.href.export(rev or repos.youngest_rev,
-                                       repos.reponame or None, node.path)
-            add_link(req, 'alternate', raw_href, _('Original Format'),
-                     mime_type)
+        # add ''Plain Text'' alternate link if needed
+        if not is_binary(chunk) and mime_type != 'text/plain':
+            plain_href = req.href.browser(repos.reponame or None,
+                                          node.path, rev=rev,
+                                          format='txt')
+            add_link(req, 'alternate', plain_href, _('Plain Text'),
+                     'text/plain')
 
-            self.log.debug("Rendering preview of node %s@%s with mime-type %s",
-                           node.name, rev, mime_type)
+        # add ''Original Format'' alternate link (always)
+        raw_href = req.href.export(rev or repos.youngest_rev,
+                                   repos.reponame or None, node.path)
+        add_link(req, 'alternate', raw_href, _('Original Format'),
+                 mime_type)
 
-            content = None # the remainder of that content is not needed
+        self.log.debug("Rendering preview of node %s@%s with "
+                       "mime-type %s", node.name, rev, mime_type)
 
-            add_stylesheet(req, 'common/css/code.css')
+        add_stylesheet(req, 'common/css/code.css')
 
-            annotations = ['lineno']
-            annotate = req.args.get('annotate')
-            if annotate:
-                annotations.insert(0, annotate)
-            preview_data = mimeview.preview_data(context,
-                                                 node.get_processed_content(),
+        annotations = ['lineno']
+        annotate = req.args.get('annotate')
+        if annotate:
+            annotations.insert(0, annotate)
+        with content_closing(node.get_processed_content()) as content:
+            preview_data = mimeview.preview_data(context, content,
                                                  node.get_content_length(),
                                                  mime_type, node.created_path,
                                                  raw_href,
                                                  annotations=annotations,
                                                  force_source=bool(annotate))
-            return {
-                'changeset': changeset,
-                'size': node.content_length,
-                'preview': preview_data,
-                'annotate': annotate,
-                }
+        return {
+            'changeset': changeset,
+            'size': node.content_length,
+            'preview': preview_data,
+            'annotate': annotate,
+            }
 
     def _get_download_href(self, href, repos, node, rev):
         """Return the URL for downloading a file, or a directory as a ZIP."""
@@ -785,7 +782,7 @@ class BrowserModule(Component):
                     rendered = None
                 prop = {'name': name, 'value': value, 'rendered': rendered}
                 return prop
-            except Exception, e:
+            except Exception as e:
                 self.log.warning('Rendering failed for property %s with '
                                  'renderer %s: %s', name,
                                  renderer.__class__.__name__,
@@ -1021,7 +1018,7 @@ class BlameAnnotator(object):
             return
         rev = self.annotations[lineno-1]
         chgset = self.changesets[lineno-1]
-        path = self.paths.get(rev, None)
+        path = self.paths.get(rev)
         # Note: path will be None if copy/rename is not supported
         # by get_history
 
