@@ -1,43 +1,42 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+from abc import ABCMeta
 from BaseHTTPServer import BaseHTTPRequestHandler
 from Cookie import CookieError, BaseCookie, SimpleCookie
 import cgi
 from datetime import datetime
-import errno
 from hashlib import md5
 import new
 import mimetypes
 import os
 import re
-import socket
 from StringIO import StringIO
 import sys
 import urlparse
 
 from genshi.builder import Fragment
-from trac.core import Interface, TracError
-from trac.perm import PermissionError
-from trac.util import get_last_traceback, unquote
+from trac.core import Interface, TracBaseError, TracError
+from trac.util import as_bool, as_int, get_last_traceback, lazy, unquote
 from trac.util.datefmt import http_date, localtz
+from trac.util.html import tag
 from trac.util.text import empty, exception_to_unicode, to_unicode
-from trac.util.translation import _
+from trac.util.translation import _, N_, tag_
 from trac.web.href import Href
-from trac.web.wsgi import _FileWrapper
+from trac.web.wsgi import _FileWrapper, is_client_disconnect_exception
 
 
 class IAuthenticator(Interface):
@@ -50,7 +49,19 @@ class IAuthenticator(Interface):
 
 
 class IRequestHandler(Interface):
-    """Decide which `trac.core.Component` handles which `Request`, and how."""
+    """Decide which `trac.core.Component` handles which `Request`, and how.
+
+    The boolean property `is_valid_default_handler` determines whether the
+    `IRequestFilter` can be used as a `default_handler` and defaults to
+    `True`. To be suitable as a `default_handler`, an `IRequestFilter` must
+    return an HTML document and `data` dictionary for rendering the document,
+    and must not require that `match_request` be called prior to
+    `process_request`.
+
+    The boolean property `jquery_noconflict` determines whether jQuery's
+    `noConflict` mode will be activated by the handler, and defaults to
+    `False`.
+    """
 
     def match_request(req):
         """Return whether the handler wants to process the given request."""
@@ -67,9 +78,21 @@ class IRequestHandler(Interface):
         simply send the response itself and not return anything.
 
         :Since 1.0: Clearsilver templates are no longer supported.
+
+        :Since 1.1.2: the rendering `method` (xml, xhtml or text) may be
+           returned as a fourth parameter in the tuple, but if not specified
+           it will be inferred from the `content_type` when rendering the
+           template.
         """
 
 
+def is_valid_default_handler(handler):
+    """Returns `True` if the `handler` is a valid default handler, as
+    described in the `IRequestHandler` interface documentation.
+    """
+    return handler and getattr(handler, 'is_valid_default_handler', True)
+
+
 class IRequestFilter(Interface):
     """Enable components to interfere with the processing done by the
     main handler, either before and/or after it enters in action.
@@ -82,7 +105,7 @@ class IRequestFilter(Interface):
         Always returns the request handler, even if unchanged.
         """
 
-    def post_process_request(req, template, data, content_type):
+    def post_process_request(req, template, data, content_type, method=None):
         """Do any post-processing the request might need; typically adding
         values to the template `data` dictionary, or changing the Genshi
         template or mime type.
@@ -103,6 +126,11 @@ class IRequestFilter(Interface):
            templates.
 
         :Since 1.0: Clearsilver templates are no longer supported.
+
+        :Since 1.1.2: the rendering `method` will be passed if it is returned
+           by the request handler, otherwise `method` will be `None`. For
+           backward compatibility, the parameter is optional in the
+           implementation's signature.
         """
 
 
@@ -126,15 +154,29 @@ class ITemplateStreamFilter(Interface):
         """
 
 
+class TracNotImplementedError(TracError, NotImplementedError):
+    """Raised when a `NotImplementedError` is trapped.
+
+    This exception is for internal use and should not be raised by
+    plugins. Plugins should raise `NotImplementedError`.
+
+    :since: 1.0.11
+    """
+
+    title = N_("Not Implemented Error")
+
+
 HTTP_STATUS = dict([(code, reason.title()) for code, (reason, description)
                     in BaseHTTPRequestHandler.responses.items()])
 
 
-class HTTPException(Exception):
+class HTTPException(TracBaseError):
+
+    __metaclass__ = ABCMeta
 
     def __init__(self, detail, *args):
         """Factory for HTTPException classes."""
-        if isinstance(detail, (TracError, PermissionError)):
+        if isinstance(detail, TracBaseError):
             self.detail = detail.message
             self.reason = detail.title
         else:
@@ -150,9 +192,9 @@ class HTTPException(Exception):
         # The message is based on the e.detail, which can be an Exception
         # object, but not a TracError one: when creating HTTPException,
         # a TracError.message is directly assigned to e.detail
-        if isinstance(self.detail, Exception): # not a TracError or PermissionError
+        if isinstance(self.detail, Exception): # not a TracBaseError
             message = exception_to_unicode(self.detail)
-        elif isinstance(self.detail, Fragment): # TracError or PermissionError markup
+        elif isinstance(self.detail, Fragment): # TracBaseError markup
             message = self.detail
         else:
             message = to_unicode(self.detail)
@@ -216,6 +258,84 @@ class _RequestArgs(dict):
     """Dictionary subclass that provides convenient access to request
     parameters that may contain multiple values."""
 
+    def as_int(self, name, default=None, min=None, max=None):
+        """Return the value as an integer. Return `default` if
+        if an exception is raised while converting the value to an
+        integer.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified or an exception occurs converting
+                          the value to an integer.
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        return as_int(self.getfirst(name), default, min, max)
+
+    def as_bool(self, name, default=None):
+        """Return the value as a boolean. Return `default` if
+        if an exception is raised while converting the value to a
+        boolean.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified or an exception occurs converting
+                          the value to a boolean.
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        return as_bool(self.getfirst(name), default)
+
+    def getbool(self, name, default=None):
+        """Return the value as a boolean. Raise an `HTTPBadRequest`
+        exception if an exception occurs while converting the value to
+        a boolean.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified.
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        value = self[name]
+        if isinstance(value, list):
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        value = as_bool(value, None)
+        if value is None:
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        return value
+
+    def getint(self, name, default=None, min=None, max=None):
+        """Return the value as an integer. Raise an `HTTPBadRequest`
+        exception if an exception occurs while converting the value
+        to an integer.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        value = as_int(self[name], None, min, max)
+        if value is None:
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        return value
+
     def getfirst(self, name, default=None):
         """Return the first value for the specified parameter, or `default` if
         the parameter was not provided.
@@ -238,12 +358,28 @@ class _RequestArgs(dict):
             val = [val]
         return val
 
+    def require(self, name):
+        """Raise an `HTTPBadRequest` exception if the parameter is
+        not in the request.
+
+        :param name: the name of the request parameter
+
+        :since: 1.2
+        """
+        if name not in self:
+            raise HTTPBadRequest(
+                tag_("Missing request argument. The %(name)s argument "
+                     "must be included in the request.", name=tag.em(name)))
+
 
 def parse_arg_list(query_string):
-    """Parse a query string into a list of `(name, value)` tuples."""
+    """Parse a query string into a list of `(name, value)` tuples.
+
+    :Since 1.1.2: a leading `?` is stripped from `query_string`."""
     args = []
     if not query_string:
         return args
+    query_string = query_string.lstrip('?')
     for arg in query_string.split('&'):
         nv = arg.split('=', 1)
         if len(nv) == 2:
@@ -274,7 +410,7 @@ def arg_list_to_args(arg_list):
     return args
 
 
-class RequestDone(Exception):
+class RequestDone(TracBaseError):
     """Marker exception that indicates whether request processing has completed
     and a response was sent.
     """
@@ -315,6 +451,12 @@ class Request(object):
     This class provides a convenience API over WSGI.
     """
 
+    _disallowed_control_codes_re = re.compile(r'[\x00-\x08\x0a-\x1f\x7f]')
+    _reserved_headers = set(['content-type', 'content-length', 'location',
+                             'etag', 'pragma', 'cache-control', 'expires'])
+    # RFC7230 3.2 Header Fields
+    _valid_header_re = re.compile(r"[-0-9A-Za-z!#$%&'*+.^_`|~]+\Z")
+
     def __init__(self, environ, start_response):
         """Create the request wrapper.
 
@@ -328,6 +470,7 @@ class Request(object):
         self._write = None
         self._status = '200 OK'
         self._response = None
+        self._content_type = None
 
         self._outheaders = []
         self._outcharset = None
@@ -358,12 +501,22 @@ class Request(object):
         raise AttributeError(name)
 
     def __repr__(self):
-        path_info = self.environ.get('PATH_INFO', '')
-        return '<%s "%s %r">' % (self.__class__.__name__, self.method,
-                                 path_info)
+        uri = self.environ.get('PATH_INFO', '')
+        qs = self.query_string
+        if qs:
+            uri += '?' + qs
+        return '<%s "%s %r">' % (self.__class__.__name__, self.method, uri)
 
     # Public API
 
+    @lazy
+    def is_xhr(self):
+        """Returns `True` if the request is an `XMLHttpRequest`.
+
+        :since: 1.1.6
+        """
+        return self.get_header('X-Requested-With') == 'XMLHttpRequest'
+
     @property
     def method(self):
         """The HTTP method of the request"""
@@ -399,6 +552,10 @@ class Request(object):
         if user is not None:
             return to_unicode(user)
 
+    @property
+    def response_started(self):
+        return self._write is not None
+
     @property
     def scheme(self):
         """The scheme of the request URL"""
@@ -448,6 +605,7 @@ class Request(object):
         """
         lower_name = name.lower()
         if lower_name == 'content-type':
+            self._content_type = value.split(';', 1)[0]
             ctpos = value.find('charset=')
             if ctpos >= 0:
                 self._outcharset = value[ctpos + 8:].strip()
@@ -459,6 +617,10 @@ class Request(object):
         """Must be called after all headers have been sent and before the
         actual content is written.
         """
+        if self.method == 'POST' and self._content_type == 'text/html':
+            # Disable XSS protection (#12926)
+            self.send_header('X-XSS-Protection', 0)
+        self._send_configurable_headers()
         self._send_cookie_headers()
         self._write = self._start_response(self._status, self._outheaders)
 
@@ -492,6 +654,8 @@ class Request(object):
             self.end_headers()
             raise RequestDone
 
+    _trident_re = re.compile(r' Trident/([0-9]+)')
+
     def redirect(self, url, permanent=False):
         """Send a redirect to the client, forwarding to the specified URL.
 
@@ -516,9 +680,10 @@ class Request(object):
 
         # Workaround #10382, IE6-IE9 bug when post and redirect with hash
         if status == 303 and '#' in url:
-            match = re.search(' MSIE ([0-9]+)',
-                              self.environ.get('HTTP_USER_AGENT', ''))
-            if match and int(match.group(1)) < 10:
+            user_agent = self.environ.get('HTTP_USER_AGENT', '')
+            match_trident = self._trident_re.search(user_agent)
+            if ' MSIE ' in user_agent and \
+                    (not match_trident or int(match_trident.group(1)) < 6):
                 url = url.replace('#', '#__msie303:')
 
         self.send_header('Location', url)
@@ -548,7 +713,8 @@ class Request(object):
         try:
             if template.endswith('.html'):
                 if env:
-                    from trac.web.chrome import Chrome
+                    from trac.web.chrome import Chrome, add_stylesheet
+                    add_stylesheet(self, 'common/css/code.css')
                     try:
                         data = Chrome(env).render_template(self, template,
                                                            data, 'text/html')
@@ -575,6 +741,7 @@ class Request(object):
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.send_header('Content-Type', content_type + ';charset=utf-8')
         self.send_header('Content-Length', len(data))
+        self._send_configurable_headers()
         self._send_cookie_headers()
 
         self._write = self._start_response(self._status, self._outheaders,
@@ -684,17 +851,22 @@ class Request(object):
                     buf[:] = ()
             if bufsize > 0:
                 self._write(''.join(buf))
-        except (IOError, socket.error), e:
-            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
-                raise RequestDone
-            # Note that mod_wsgi raises an IOError with only a message
-            # if the client disconnects
-            if 'mod_wsgi.version' in self.environ and \
-               e.args[0] in ('failed to write data',
-                             'client connection closed'):
+        except IOError as e:
+            if is_client_disconnect_exception(e):
                 raise RequestDone
             raise
 
+    @classmethod
+    def is_valid_header(cls, name, value=None):
+        """Check whether the field name, and optionally the value, make
+        a valid HTTP header.
+        """
+        valid_name = name and name.lower() not in cls._reserved_headers and \
+                     bool(cls._valid_header_re.match(name))
+        valid_value = not cls._disallowed_control_codes_re.search(value) \
+                      if value else True
+        return valid_name & valid_value
+
     # Internal methods
 
     def _parse_arg_list(self):
@@ -717,17 +889,36 @@ class Request(object):
         # requests. We'll keep the pre 2.6 behaviour for now...
         if self.method == 'POST':
             qs_on_post = self.environ.pop('QUERY_STRING', '')
-        fs = _FieldStorage(fp, environ=self.environ, keep_blank_values=True)
+        try:
+            fs = _FieldStorage(fp, environ=self.environ,
+                               keep_blank_values=True)
+        except IOError as e:
+            if is_client_disconnect_exception(e):
+                raise HTTPBadRequest(
+                    _("Exception caught while reading request: %(msg)s",
+                      msg=exception_to_unicode(e)))
+            raise
         if self.method == 'POST':
             self.environ['QUERY_STRING'] = qs_on_post
 
+        def raise_if_null_bytes(value):
+            if value and '\x00' in value:
+                raise HTTPBadRequest(_("Invalid request arguments."))
+
         args = []
         for value in fs.list or ():
+            name = value.name
+            raise_if_null_bytes(name)
             try:
-                name = unicode(value.name, 'utf-8')
-                if not value.filename:
-                    value = unicode(value.value, 'utf-8')
-            except UnicodeDecodeError, e:
+                if name is not None:
+                    name = unicode(name, 'utf-8')
+                if value.filename:
+                    raise_if_null_bytes(value.filename)
+                else:
+                    value = value.value
+                    raise_if_null_bytes(value)
+                    value = unicode(value, 'utf-8')
+            except UnicodeDecodeError as e:
                 raise HTTPBadRequest(
                     _("Invalid encoding in form data: %(msg)s",
                       msg=exception_to_unicode(e)))
@@ -784,6 +975,12 @@ class Request(object):
         return urlparse.urlunparse((self.scheme, host, self.base_path, None,
                                     None, None))
 
+    def _send_configurable_headers(self):
+        sent_headers = [name.lower() for name, val in self._outheaders]
+        for name, val in getattr(self, 'configurable_headers', []):
+            if name.lower() not in sent_headers:
+                self.send_header(name, val)
+
     def _send_cookie_headers(self):
         for name in self.outcookie.keys():
             path = self.outcookie[name].get('path')
@@ -797,4 +994,5 @@ class Request(object):
         for cookie in cookies.splitlines():
             self._outheaders.append(('Set-Cookie', cookie.strip()))
 
+
 __no_apidoc__ = _HTTPException_subclass_names
