@@ -1,26 +1,23 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import with_statement
+from __future__ import print_function
 
-try:
-    from base64 import b64decode, b64encode
-except ImportError:
-    from base64 import decodestring as b64decode
-    from base64 import encodestring as b64encode
+from abc import ABCMeta, abstractmethod
+from base64 import b64decode, b64encode
 from hashlib import md5, sha1
 import os
 import re
@@ -33,8 +30,9 @@ from genshi.builder import tag
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.web.api import IAuthenticator, IRequestHandler
-from trac.web.chrome import INavigationContributor
+from trac.web.chrome import Chrome, INavigationContributor
 from trac.util import hex_entropy, md5crypt
+from trac.util.compat import crypt
 from trac.util.concurrency import threading
 from trac.util.datefmt import time_now
 from trac.util.translation import _, tag_
@@ -58,20 +56,28 @@ class LoginModule(Component):
 
     implements(IAuthenticator, INavigationContributor, IRequestHandler)
 
+    is_valid_default_handler = False
+
     check_ip = BoolOption('trac', 'check_auth_ip', 'false',
          """Whether the IP address of the user should be checked for
-         authentication (''since 0.9'').""")
+         authentication.""")
 
     ignore_case = BoolOption('trac', 'ignore_auth_case', 'false',
-        """Whether login names should be converted to lower case
-        (''since 0.9'').""")
+        """Whether login names should be converted to lower case.""")
+
+    auth_cookie_domain = Option('trac', 'auth_cookie_domain', '',
+        """Auth cookie domain attribute.
+
+        The auth cookie can be shared among multiple subdomains
+        by setting the value to the domain. (//since 1.2//)
+        """)
 
     auth_cookie_lifetime = IntOption('trac', 'auth_cookie_lifetime', 0,
         """Lifetime of the authentication cookie, in seconds.
 
         This value determines how long the browser will cache
         authentication information, and therefore, after how much
-        inactivity a user will have to log in again. The default value
+        inactivity a user will have to log in again. The value
         of 0 makes the cookie expire at the end of the browsing
         session. (''since 0.12'')""")
 
@@ -105,16 +111,17 @@ class LoginModule(Component):
 
     def get_navigation_items(self, req):
         if req.authname and req.authname != 'anonymous':
-            yield ('metanav', 'login', _('logged in as %(user)s',
-                                         user=req.authname))
+            yield ('metanav', 'login',
+                   tag_("logged in as %(user)s",
+                        user=Chrome(self.env).authorinfo(req, req.authname)))
             yield ('metanav', 'logout',
-                   tag.form(tag.div(tag.button(_('Logout'),
+                   tag.form(tag.div(tag.button(_("Logout"),
                                                name='logout', type='submit')),
                             action=req.href.logout(), method='post',
                             id='logout', class_='trac-logout'))
         else:
             yield ('metanav', 'login',
-                   tag.a(_('Login'), href=req.href.login()))
+                   tag.a(_("Login"), href=req.href.login()))
 
     # IRequestHandler methods
 
@@ -147,10 +154,10 @@ class LoginModule(Component):
         """
         if not req.remote_user:
             # TRANSLATOR: ... refer to the 'installation documentation'. (link)
-            inst_doc = tag.a(_('installation documentation'),
+            inst_doc = tag.a(_("installation documentation"),
                              title=_("Configuring Authentication"),
-                             href=req.href.wiki('TracInstall')
-                                  + "#ConfiguringAuthentication")
+                             href=req.href.wiki('TracInstall') +
+                                                "#ConfiguringAuthentication")
             raise TracError(tag_("Authentication information not available. "
                                  "Please refer to the %(inst_doc)s.",
                                  inst_doc=inst_doc))
@@ -159,7 +166,7 @@ class LoginModule(Component):
             remote_user = remote_user.lower()
 
         if req.authname not in ('anonymous', remote_user):
-            raise TracError(_('Already logged in as %(user)s.',
+            raise TracError(_("Already logged in as %(user)s.",
                               user=req.authname))
 
         with self.env.db_transaction as db:
@@ -181,12 +188,13 @@ class LoginModule(Component):
                          int(time_now())))
         req.authname = remote_user
         req.outcookie['trac_auth'] = cookie
+        if self.auth_cookie_domain:
+            req.outcookie['trac_auth']['domain'] = self.auth_cookie_domain
         req.outcookie['trac_auth']['path'] = self.auth_cookie_path \
                                              or req.base_path or '/'
         if self.env.secure_cookies:
             req.outcookie['trac_auth']['secure'] = True
-        if sys.version_info >= (2, 6):
-            req.outcookie['trac_auth']['httponly'] = True
+        req.outcookie['trac_auth']['httponly'] = True
         if self.auth_cookie_lifetime > 0:
             req.outcookie['trac_auth']['expires'] = self.auth_cookie_lifetime
 
@@ -220,13 +228,14 @@ class LoginModule(Component):
         the "expires" property to a date in the past.
         """
         req.outcookie['trac_auth'] = ''
+        if self.auth_cookie_domain:
+            req.outcookie['trac_auth']['domain'] = self.auth_cookie_domain
         req.outcookie['trac_auth']['path'] = self.auth_cookie_path \
                                              or req.base_path or '/'
         req.outcookie['trac_auth']['expires'] = -10000
         if self.env.secure_cookies:
             req.outcookie['trac_auth']['secure'] = True
-        if sys.version_info >= (2, 6):
-            req.outcookie['trac_auth']['httponly'] = True
+        req.outcookie['trac_auth']['httponly'] = True
 
     def _cookie_to_name(self, req, cookie):
         # This is separated from _get_name_for_cookie(), because the
@@ -278,8 +287,11 @@ class LoginModule(Component):
 
 class HTTPAuthentication(object):
 
+    __metaclass__ = ABCMeta
+
+    @abstractmethod
     def do_auth(self, environ, start_response):
-        raise NotImplementedError
+        pass
 
 
 class PasswordFileAuthentication(HTTPAuthentication):
@@ -302,15 +314,8 @@ class BasicAuthentication(PasswordFileAuthentication):
     def __init__(self, htpasswd, realm):
         # FIXME pass a logger
         self.realm = realm
-        try:
-            import crypt
-            self.crypt = crypt.crypt
-        except ImportError:
-            try:
-                import fcrypt
-                self.crypt = fcrypt.crypt
-            except ImportError:
-                self.crypt = None
+        self.crypt = crypt
+        self.hash = {}
         PasswordFileAuthentication.__init__(self, htpasswd)
 
     def load(self, filename):
@@ -318,23 +323,25 @@ class BasicAuthentication(PasswordFileAuthentication):
         self.hash = {}
         fd = open(filename, 'r')
         for line in fd:
-            line = line.strip()
+            line = line.split('#')[0].strip()
             if not line:
                 continue
             try:
-                u, h = line.split(':')
+                u, h = line.split(':')[:2]
             except ValueError:
-                print>>sys.stderr, 'Warning: invalid password line in %s: ' \
-                    '%s' % (filename, line)
+                print("Warning: invalid password line in %s: %s"
+                      % (filename, line), file=sys.stderr)
                 continue
             if '$' in h or h.startswith('{SHA}') or self.crypt:
                 self.hash[u] = h
             else:
-                print>>sys.stderr, 'Warning: cannot parse password for ' \
-                    'user "%s" without the "crypt" module' % u
+                print('Warning: cannot parse password for user "%s" '
+                      'without the "crypt" module. Install the passlib '
+                      'package from PyPI' % u, file=sys.stderr)
 
         if self.hash == {}:
-            print>>sys.stderr, "Warning: found no users in file:", filename
+            print("Warning: found no users in file:", filename,
+                  file=sys.stderr)
 
     def test(self, user, password):
         self.check_reload()
@@ -345,7 +352,7 @@ class BasicAuthentication(PasswordFileAuthentication):
         if the_hash.startswith('{SHA}'):
             return b64encode(sha1(password).digest()) == the_hash[5:]
 
-        if not '$' in the_hash:
+        if '$' not in the_hash:
             return self.crypt(password, the_hash[:2]) == the_hash
 
         magic, salt = the_hash[1:].split('$')[:2]
@@ -376,6 +383,7 @@ class DigestAuthentication(PasswordFileAuthentication):
         # FIXME pass a logger
         self.active_nonces = []
         self.realm = realm
+        self.hash = {}
         PasswordFileAuthentication.__init__(self, htdigest)
 
     def load(self, filename):
@@ -385,20 +393,21 @@ class DigestAuthentication(PasswordFileAuthentication):
         # FIXME use a logger
         self.hash = {}
         fd = open(filename, 'r')
-        for line in fd.readlines():
-            line = line.strip()
+        for line in fd:
+            line = line.split('#')[0].strip()
             if not line:
                 continue
             try:
-                u, r, a1 = line.split(':')
+                u, r, a1 = line.split(':')[:3]
             except ValueError:
-                print>>sys.stderr, 'Warning: invalid digest line in %s: %s' \
-                    % (filename, line)
+                print("Warning: invalid digest line in %s: %s"
+                      % (filename, line), file=sys.stderr)
                 continue
             if r == self.realm:
                 self.hash[u] = a1
         if self.hash == {}:
-            print>>sys.stderr, "Warning: found no users in realm:", self.realm
+            print("Warning: found no users in realm:", self.realm,
+                  file=sys.stderr)
 
     def parse_auth_header(self, authorization):
         values = {}
