@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Matthew Good <trac@matt-good.net>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Matthew Good <trac@matt-good.net>
@@ -38,8 +38,9 @@ from genshi.output import DocType
 from genshi.template import TemplateLoader
 
 from trac import __version__ as TRAC_VERSION
-from trac.config import BoolOption, ChoiceOption, ConfigurationError, \
-                        ExtensionOption, Option, OrderedExtensionsOption
+from trac.config import BoolOption, ChoiceOption, ConfigSection, \
+                        ConfigurationError, ExtensionOption, Option, \
+                        OrderedExtensionsOption
 from trac.core import *
 from trac.env import open_environment
 from trac.loader import get_plugin_info, match_plugins_to_frames
@@ -164,6 +165,14 @@ class RequestDispatcher(Component):
         """The header to use if `use_xsendfile` is enabled. If Nginx is used,
         set `X-Accel-Redirect`. (''since 1.0.6'')""")
 
+    configurable_headers = ConfigSection('http-headers', """
+        Headers to be added to the HTTP request. (''since 1.2.3'')
+
+        The header name must conform to RFC7230 and the following
+        reserved names are not allowed: content-type, content-length,
+        location, etag, pragma, cache-control, expires.
+        """)
+
     # Public API
 
     def authenticate(self, req):
@@ -317,6 +326,7 @@ class RequestDispatcher(Component):
             'tz': self._get_timezone,
             'use_xsendfile': self._get_use_xsendfile,
             'xsendfile_header': self._get_xsendfile_header,
+            'configurable_headers': self._get_configurable_headers,
         })
 
     @lazy
@@ -411,20 +421,34 @@ class RequestDispatcher(Component):
     def _get_use_xsendfile(self, req):
         return self.use_xsendfile
 
-    # RFC7230 3.2 Header Fields
-    _xsendfile_header_re = re.compile(r"[-0-9A-Za-z!#$%&'*+.^_`|~]+\Z")
-    _warn_xsendfile_header = False
-
-    def _get_xsendfile_header(self, req):
+    @lazy
+    def _xsendfile_header(self):
         header = self.xsendfile_header.strip()
-        if self._xsendfile_header_re.match(header):
+        if Request.is_valid_header(header):
             return to_utf8(header)
         else:
-            if not self._warn_xsendfile_header:
-                self._warn_xsendfile_header = True
-                self.log.warn("[trac] xsendfile_header is invalid: '%s'",
-                              header)
-            return None
+            self.log.warn("[trac] xsendfile_header is invalid: '%s'",
+                          header)
+
+    def _get_xsendfile_header(self, req):
+        return self._xsendfile_header
+
+    @lazy
+    def _configurable_headers(self):
+        headers = []
+        invalids = []
+        for name, val in self.configurable_headers.options():
+            if Request.is_valid_header(name, val):
+                headers.append((name, val))
+            else:
+                invalids.append((name, val))
+        if invalids:
+            self.log.warning('[http-headers] invalid headers are ignored: %s',
+                             ', '.join('%r: %r' % i for i in invalids))
+        return tuple(headers)
+
+    def _get_configurable_headers(self, req):
+        return iter(self._configurable_headers)
 
     def _pre_process_request(self, req, chosen_handler):
         for filter_ in self.filters:
@@ -618,16 +642,19 @@ def _dispatch_request(req, env, env_error):
         if not env and env_error:
             raise HTTPInternalError(env_error)
         dispatcher = RequestDispatcher(env)
+        dispatcher.set_default_callbacks(req)
         try:
-            dispatcher.set_default_callbacks(req)
             dispatcher.dispatch(req)
         except RequestDone as req_done:
             resp = req_done.iterable
-        resp = resp or req._response or []
     except HTTPException as e:
-        _send_user_error(req, env, e)
+        if not req.response_started:
+            _send_user_error(req, env, e)
     except Exception:
-        send_internal_error(env, req, sys.exc_info())
+        if not req.response_started:
+            send_internal_error(env, req, sys.exc_info())
+    else:
+        resp = resp or req._response or []
     return resp
 
 
@@ -716,7 +743,8 @@ def send_internal_error(env, req, exc_info):
         return _("""\
 ==== How to Reproduce ====
 
-While doing a %(method)s operation on `%(path_info)s`, Trac issued an internal error.
+While doing a %(method)s operation on `%(path_info)s`, \
+Trac issued an internal error.
 
 ''(please provide additional details here)''
 
