@@ -1,19 +1,20 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2015 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os.path
 import sys
 import tempfile
+import textwrap
 import unittest
 from StringIO import StringIO
 
@@ -245,7 +246,7 @@ class RequestTestCase(unittest.TestCase):
             status_sent.append(status)
             headers_sent.update(dict(headers))
         environ = _make_environ(method='HEAD')
-        req = Request(environ, start_response,)
+        req = Request(environ, start_response)
         req.session = Mock(save=lambda: None)
         self.assertRaises(RequestDone, req.redirect,
                           'http://example.com/trac/test')
@@ -264,7 +265,7 @@ class RequestTestCase(unittest.TestCase):
                 status_sent.append(status)
                 headers_sent.update(dict(headers))
             environ = _make_environ(method='POST', HTTP_USER_AGENT=ua)
-            req = Request(environ, start_response,)
+            req = Request(environ, start_response)
             req.session = Mock(save=lambda: None)
             self.assertRaises(RequestDone, req.redirect, url)
             self.assertEqual('303 See Other', status_sent[0])
@@ -399,21 +400,21 @@ class RequestTestCase(unittest.TestCase):
 
     def test_post_with_unnamed_value(self):
         boundary = '_BOUNDARY_'
-        form_data = """\
---%(boundary)s\r\n\
-Content-Disposition: form-data; name="foo"\r\n\
-\r\n\
-named value\r\n\
---%(boundary)s\r\n\
-Content-Disposition: form-data; name=""\r\n\
-\r\n\
-name is empty\r\n\
---%(boundary)s\r\n\
-Content-Disposition: form-data\r\n\
-\r\n\
-unnamed value\r\n\
---%(boundary)s--\r\n\
-"""
+        form_data = textwrap.dedent("""\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data; name="foo"\r\n\
+            \r\n\
+            named value\r\n\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data; name=""\r\n\
+            \r\n\
+            name is empty\r\n\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data\r\n\
+            \r\n\
+            unnamed value\r\n\
+            --%(boundary)s--\r\n\
+            """)
         form_data %= {'boundary': boundary}
         content_type = 'multipart/form-data; boundary="%s"' % boundary
         environ = _make_environ(method='POST', **{
@@ -448,39 +449,40 @@ unnamed value\r\n\
             self.fail("HTTPBadRequest not raised.")
 
     def test_post_with_null_bytes_for_filename(self):
-        form_data = """\
---%(boundary)s\r\n\
-Content-Disposition: form-data; name="attachment"; filename="thefi\x00le.txt"\r\n\
-Content-Type: text/plain\r\n\
-\r\n\
-The file content.\r\n\
---%(boundary)s\r\n\
-Content-Disposition: form-data; name="action"\r\n\
-\r\n\
-new\r\n\
---%(boundary)s--\r\n\
-"""
+        form_data = textwrap.dedent("""\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data; name="attachment"; \
+            filename="thefi\x00le.txt"\r\n\
+            Content-Type: text/plain\r\n\
+            \r\n\
+            The file content.\r\n\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data; name="action"\r\n\
+            \r\n\
+            new\r\n\
+            --%(boundary)s--\r\n\
+            """)
         self._test_post_with_null_bytes(form_data)
 
     def test_post_with_null_bytes_for_name(self):
-        form_data = """\
---%(boundary)s\r\n\
-Content-Disposition: form-data; name="acti\x00n"\r\n\
-\r\n\
-new\r\n\
---%(boundary)s--\r\n\
-"""
+        form_data = textwrap.dedent("""\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data; name="acti\x00n"\r\n\
+            \r\n\
+            new\r\n\
+            --%(boundary)s--\r\n\
+            """)
 
         self._test_post_with_null_bytes(form_data)
 
     def test_post_with_null_bytes_for_value(self):
-        form_data = """\
---%(boundary)s\r\n\
-Content-Disposition: form-data; name="action"\r\n\
-\r\n\
-ne\x00w\r\n\
---%(boundary)s--\r\n\
-"""
+        form_data = textwrap.dedent("""\
+            --%(boundary)s\r\n\
+            Content-Disposition: form-data; name="action"\r\n\
+            \r\n\
+            ne\x00w\r\n\
+            --%(boundary)s--\r\n\
+            """)
         self._test_post_with_null_bytes(form_data)
 
     def test_qs_on_post(self):
@@ -509,6 +511,48 @@ ne\x00w\r\n\
         req = Request(environ, None)
         self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
 
+    def test_post_text_html_disables_xss(self):
+        """POST request with content-type text/html disables XSS
+        protection (#12926).
+        """
+        status_sent = []
+        headers_sent = {}
+        buf = StringIO()
+
+        def write(data):
+            buf.write(data)
+
+        def start_response(status, headers):
+            status_sent.append(status)
+            headers_sent.update(dict(headers))
+            return write
+
+        content_type = 'text/html'
+        content = "The content"
+        environ = _make_environ(method='POST',
+                                **{'wsgi.input': StringIO(content),
+                                   'CONTENT_LENGTH': str(len(content)),
+                                   'CONTENT_TYPE': content_type})
+        req = Request(environ, start_response)
+        self.assertRaises(RequestDone, req.send, content, content_type)
+        self.assertIn('0', headers_sent['X-XSS-Protection'])
+
+    def test_is_valid_header(self):
+        # Reserved headers not allowed.
+        for name in ('Content-Type', 'Content-Length', 'Location',
+                     'ETag', 'Pragma', 'Cache-Control', 'Expires'):
+            self.assertFalse(Request.is_valid_header(name))
+            self.assertFalse(Request.is_valid_header(name.lower()))
+        # Control code not allowed in header value.
+        self.assertFalse(Request.is_valid_header('X-Custom-1', '\x00custom1'))
+        self.assertFalse(Request.is_valid_header('X-Custom-1', 'cust\x0aom1'))
+        self.assertFalse(Request.is_valid_header('X-Custom-1', 'custom1\x7f'))
+        # Only a subset of special characters allowed in header name.
+        self.assertFalse(Request.is_valid_header('X-Custom-(2)', 'custom2'))
+        self.assertFalse(Request.is_valid_header('X-Custom-:2:', 'custom2'))
+        self.assertTrue(Request.is_valid_header('Aa0-!#$%&\'*+.^_`|~',
+                                                'custom2'))
+
 
 class RequestSendFileTestCase(unittest.TestCase):
 
