@@ -1,36 +1,40 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import tempfile
+import textwrap
 import unittest
 
-from genshi.builder import tag
-from genshi.core import Markup
 import trac.tests.compat
+from trac.config import ConfigurationError
 from trac.core import Component, TracError, implements
-from trac.test import EnvironmentStub, locale_en
+from trac.perm import PermissionSystem
+from trac.test import EnvironmentStub, MockPerm, MockRequest, locale_en
 from trac.tests.contentgen import random_sentence
+from trac.resource import Resource
 from trac.util import create_file
 from trac.util.datefmt import pytz, timezone, utc
+from trac.util.html import Markup, tag
 from trac.web.chrome import (
     Chrome, INavigationContributor, add_link, add_meta, add_notice, add_script,
-    add_script_data, add_stylesheet, add_warning)
+    add_script_data, add_stylesheet, add_warning, web_context)
 from trac.web.href import Href
 
 
 class Request(object):
     locale = None
+    perm = MockPerm()
     args = {}
     def __init__(self, **kwargs):
         self.chrome = {}
@@ -182,13 +186,13 @@ class ChromeTestCase(unittest.TestCase):
 
     def test_add_warning_escapes_markup(self):
         """Message text is escaped. Regression test for
-        http://trac.edgewall.org/ticket/12285
+        https://trac.edgewall.org/ticket/12285
         """
         self._test_add_message_escapes_markup('warnings', add_warning)
 
     def test_add_notice_escapes_markup(self):
         """Message text is escaped. Regression test for
-        http://trac.edgewall.org/ticket/12285
+        https://trac.edgewall.org/ticket/12285
         """
         self._test_add_message_escapes_markup('notices', add_notice)
 
@@ -273,14 +277,13 @@ class ChromeTestCase(unittest.TestCase):
         links = chrome.prepare_request(req)['links']
         self.assertEqual('/trac.cgi/chrome/common/foo.ico',
                          links['icon'][0]['href'])
-        self.assertEqual('/trac.cgi/chrome/common/foo.ico',
-                         links['shortcut icon'][0]['href'])
+        self.assertNotIn('shortcut icon', links)
 
         # URL relative to the server root for icon config option
         self.env.config.set('project', 'icon', '/favicon.ico')
         links = chrome.prepare_request(req)['links']
         self.assertEqual('/favicon.ico', links['icon'][0]['href'])
-        self.assertEqual('/favicon.ico', links['shortcut icon'][0]['href'])
+        self.assertNotIn('shortcut icon', links)
 
         # Absolute URL for icon config option
         self.env.config.set('project', 'icon',
@@ -288,8 +291,7 @@ class ChromeTestCase(unittest.TestCase):
         links = chrome.prepare_request(req)['links']
         self.assertEqual('http://example.com/favicon.ico',
                          links['icon'][0]['href'])
-        self.assertEqual('http://example.com/favicon.ico',
-                         links['shortcut icon'][0]['href'])
+        self.assertNotIn('shortcut icon', links)
 
     def test_nav_contributor(self):
         class TestNavigationContributor(Component):
@@ -322,49 +324,28 @@ class ChromeTestCase(unittest.TestCase):
         self.assertEqual({'name': 'test', 'label': 'Test', 'active': True},
                          nav['metanav'][0])
 
-    def test_nav_contributor_order(self):
-        class TestNavigationContributor1(Component):
-            implements(INavigationContributor)
-            def get_active_navigation_item(self, req):
-                return None
-            def get_navigation_items(self, req):
-                yield 'metanav', 'test1', 'Test 1'
-        class TestNavigationContributor2(Component):
-            implements(INavigationContributor)
-            def get_active_navigation_item(self, req):
-                return None
-            def get_navigation_items(self, req):
-                yield 'metanav', 'test2', 'Test 2'
-        req = Request(abs_href=Href('http://example.org/trac.cgi'),
-                      href=Href('/trac.cgi'), base_path='/trac.cgi',
-                      path_info='/',
-                      add_redirect_listener=lambda listener: None)
-        chrome = Chrome(self.env)
+    def _get_jquery_ui_script_data(self, lc_time):
+        req = Request(href=Href('/trac.cgi'), tz=utc, lc_time=lc_time)
+        Chrome(self.env).add_jquery_ui(req)
+        return req.chrome['script_data']['jquery_ui']
 
-        # Test with both items set in the order option
-        self.env.config.set('trac', 'metanav', 'test2, test1')
-        items = chrome.prepare_request(req)['nav']['metanav']
-        self.assertEqual('test2', items[0]['name'])
-        self.assertEqual('test1', items[1]['name'])
+    def test_add_jquery_ui_is_iso8601(self):
+        data = self._get_jquery_ui_script_data('iso8601')
+        self.assertIn({'value': -60, 'label': '-01:00'}, data['timezone_list'])
+        self.assertIn({'value': 0, 'label': '+00:00'}, data['timezone_list'])
+        self.assertIn({'value': 60, 'label': '+01:00'}, data['timezone_list'])
 
-        # Test with only test1 in the order options
-        self.env.config.set('trac', 'metanav', 'test1')
-        items = chrome.prepare_request(req)['nav']['metanav']
-        self.assertEqual('test1', items[0]['name'])
-        self.assertEqual('test2', items[1]['name'])
+    def test_add_jquery_ui_default_format(self):
+        data = self._get_jquery_ui_script_data(locale_en)
+        self.assertIsNone(data['timezone_list'])
 
-        # Test with only test2 in the order options
-        self.env.config.set('trac', 'metanav', 'test2')
-        items = chrome.prepare_request(req)['nav']['metanav']
-        self.assertEqual('test2', items[0]['name'])
-        self.assertEqual('test1', items[1]['name'])
+    def test_invalid_default_dateinfo_format_raises_exception(self):
+        self.env.config.set('trac', 'default_dateinfo_format', u'ābšolute')
 
-        # Test with none in the order options (order corresponds to
-        # registration order)
-        self.env.config.set('trac', 'metanav', 'foo, bar')
-        items = chrome.prepare_request(req)['nav']['metanav']
-        self.assertEqual('test1', items[0]['name'])
-        self.assertEqual('test2', items[1]['name'])
+        self.assertEqual(u'ābšolute',
+                         self.env.config.get('trac', 'default_dateinfo_format'))
+        self.assertRaises(ConfigurationError, getattr, Chrome(self.env),
+                          'default_dateinfo_format')
 
     def test_add_jquery_ui_first_week_day(self):
         def first_week_day(locale, lc_time, languages):
@@ -406,32 +387,34 @@ class ChromeTestCase(unittest.TestCase):
         gmt07b = timezone('GMT -7:00')
         gmt04a = timezone('GMT +4:00')
 
-        def timezone_list(lc_time, tz):
+        def verify_tzprops(lc_time, tz, tz_default, tz_label):
             req = Request(href=href, locale=locale_en, lc_time=lc_time, tz=tz)
             chrome.add_jquery_ui(req)
-            return req.chrome['script_data']['jquery_ui']['timezone_list']
+            data = req.chrome['script_data']['jquery_ui']
+            self.assertEqual(tz_default, data['default_timezone'])
+            if tz_default is None:
+                self.assertIsNone(data['timezone_list'])
+            else:
+                self.assertIn({'value': tz_default, 'label': tz_label},
+                              data['timezone_list'])
 
-        self.assertIn({'value': 'Z', 'label': '+00:00'},
-                      timezone_list(lc_time='iso8601', tz=utc))
-        self.assertIn({'value': 'Z', 'label': '+00:00'},
-                      timezone_list(lc_time=locale_en, tz=utc))
-        self.assertIn('-07:00', timezone_list('iso8601', gmt07b))
-        self.assertIn('-07:00', timezone_list(locale_en, gmt07b))
-        self.assertIn('+04:00', timezone_list('iso8601', gmt04a))
-        self.assertIn('+04:00', timezone_list(locale_en, gmt04a))
+        verify_tzprops('iso8601', utc, 0, '+00:00')
+        verify_tzprops(locale_en, utc, None, None)
+        verify_tzprops('iso8601', gmt07b, -420, '-07:00')
+        verify_tzprops(locale_en, gmt07b, None, None)
+        verify_tzprops('iso8601', gmt04a, 240, '+04:00')
+        verify_tzprops(locale_en, gmt04a, None, None)
         if pytz:
             # must use timezones which does not use DST
             guam = timezone('Pacific/Guam')
             monrovia = timezone('Africa/Monrovia')
             panama = timezone('America/Panama')
-            self.assertIn('+10:00', timezone_list('iso8601', guam))
-            self.assertIn('+10:00', timezone_list(locale_en, guam))
-            self.assertIn({'value': 'Z', 'label': '+00:00'},
-                          timezone_list('iso8601', monrovia))
-            self.assertIn({'value': 'Z', 'label': '+00:00'},
-                          timezone_list('iso8601', monrovia))
-            self.assertIn('-05:00', timezone_list('iso8601', panama))
-            self.assertIn('-05:00', timezone_list(locale_en, panama))
+            verify_tzprops('iso8601', guam, 600, '+10:00')
+            verify_tzprops(locale_en, guam, None, None)
+            verify_tzprops('iso8601', monrovia, 0, '+00:00')
+            verify_tzprops(locale_en, monrovia, None, None)
+            verify_tzprops('iso8601', panama, -300, '-05:00')
+            verify_tzprops(locale_en, panama, None, None)
 
     def test_navigation_item_customization(self):
         class TestNavigationContributor1(Component):
@@ -532,6 +515,9 @@ class ChromeTestCase2(unittest.TestCase):
     def tearDown(self):
         self.env.reset_db_and_disk()
 
+    def test_permission_requestor(self):
+        self.assertIn('EMAIL_VIEW', PermissionSystem(self.env).get_actions())
+
     def test_malicious_filename_raises(self):
         req = Request(path_info='/chrome/site/../conf/trac.ini')
         self.assertTrue(self.chrome.match_request(req))
@@ -558,12 +544,345 @@ class ChromeTestCase2(unittest.TestCase):
         self.assertRaises(RequestDone, self.chrome.process_request, req)
 
 
-def suite():
+class NavigationOrderTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.env.clear_component_registry()
+        self.req = Request(abs_href=Href('http://example.org/trac.cgi'),
+                           href=Href('/trac.cgi'), base_path='/trac.cgi',
+                           path_info='/',
+                           add_redirect_listener=lambda listener: None)
+        self.chrome = Chrome(self.env)
+
+        class TestNavigationContributor1(Component):
+            implements(INavigationContributor)
+            def get_active_navigation_item(self, req):
+                return None
+            def get_navigation_items(self, req):
+                yield 'metanav', 'test1', 'Test 1'
+
+        class TestNavigationContributor2(Component):
+            implements(INavigationContributor)
+            def get_active_navigation_item(self, req):
+                return None
+            def get_navigation_items(self, req):
+                yield 'metanav', 'test2', 'Test 2'
+
+    def tearDown(self):
+        self.env.restore_component_registry()
+
+    def test_explicit_ordering(self):
+        """Ordering is explicitly specified."""
+        self.env.config.set('metanav', 'test1.order', 2)
+        self.env.config.set('metanav', 'test2.order', 1)
+        items = self.chrome.prepare_request(self.req)['nav']['metanav']
+        self.assertEqual('test2', items[0]['name'])
+        self.assertEqual('test1', items[1]['name'])
+
+    def test_partial_explicit_ordering_1(self):
+        """Ordering for one item is explicitly specified."""
+        self.env.config.set('metanav', 'test1.order', 1)
+        items = self.chrome.prepare_request(self.req)['nav']['metanav']
+        self.assertEqual('test1', items[0]['name'])
+        self.assertEqual('test2', items[1]['name'])
+
+    def test_partial_explicit_ordering_2(self):
+        """Ordering for one item is explicitly specified."""
+        self.env.config.set('metanav', 'test2.order', 1)
+        items = self.chrome.prepare_request(self.req)['nav']['metanav']
+        self.assertEqual('test2', items[0]['name'])
+        self.assertEqual('test1', items[1]['name'])
+
+    def test_implicit_ordering(self):
+        """When not specified, ordering is alphabetical."""
+        self.env.config.set('metanav', 'foo.order', 1)
+        self.env.config.set('metanav', 'bar.order', 2)
+
+        items = self.chrome.prepare_request(self.req)['nav']['metanav']
+        self.assertEqual('test1', items[0]['name'])
+        self.assertEqual('test2', items[1]['name'])
+
+
+class FormatAuthorTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(enable=['trac.web.chrome.*',
+                                           'trac.perm.*',
+                                           'tracopt.perm.authz_policy'])
+        self.env.config.set('trac', 'permission_policies',
+                            'AuthzPolicy, DefaultPermissionPolicy')
+        fd, self.authz_file = tempfile.mkstemp()
+        with os.fdopen(fd, 'w') as f:
+            f.write(textwrap.dedent("""\
+                [wiki:WikiStart]
+                user2 = EMAIL_VIEW
+                [wiki:TracGuide]
+                user2 =
+                """))
+        PermissionSystem(self.env).grant_permission('user1', 'EMAIL_VIEW')
+        self.env.config.set('authz_policy', 'authz_file', self.authz_file)
+
+    def tearDown(self):
+        os.remove(self.authz_file)
+
+    def test_subject_is_anonymous(self):
+        format_author = Chrome(self.env).format_author
+        self.assertEqual('anonymous', format_author(None, 'anonymous'))
+
+    def test_subject_is_none(self):
+        format_author = Chrome(self.env).format_author
+        self.assertEqual('(none)', format_author(None, None))
+
+    def test_actor_has_email_view(self):
+        req = MockRequest(self.env, authname='user1')
+        author = Chrome(self.env).format_author(req, 'user@domain.com')
+        self.assertEqual('user@domain.com', author)
+
+    def test_actor_no_email_view(self):
+        req = MockRequest(self.env, authname='user2')
+        author = Chrome(self.env).format_author(req, 'user@domain.com')
+        self.assertEqual(u'user@\u2026', author)
+
+    def test_actor_no_email_view_show_email_addresses(self):
+        self.env.config.set('trac', 'show_email_addresses', True)
+        req = MockRequest(self.env, authname='user2')
+        author = Chrome(self.env).format_author(req, 'user@domain.com')
+        self.assertEqual('user@domain.com', author)
+
+    def test_actor_no_email_view_no_req(self):
+        author = Chrome(self.env).format_author(None, 'user@domain.com')
+        self.assertEqual(u'user@\u2026', author)
+
+    def test_actor_has_email_view_for_resource(self):
+        format_author = Chrome(self.env).format_author
+        req = MockRequest(self.env, authname='user2')
+        resource = Resource('wiki', 'WikiStart')
+        author = format_author(req, 'user@domain.com', resource)
+        self.assertEqual('user@domain.com', author)
+
+    def test_actor_has_email_view_for_resource_negative(self):
+        format_author = Chrome(self.env).format_author
+        req = MockRequest(self.env, authname='user2')
+        resource = Resource('wiki', 'TracGuide')
+        author = format_author(req, 'user@domain.com', resource)
+        self.assertEqual(u'user@\u2026', author)
+
+    def test_show_full_names_true(self):
+        format_author = Chrome(self.env).format_author
+        self.env.config.set('trac', 'show_full_names', True)
+        self.env.insert_users([
+            ('user1', 'User One', 'user1@example.org'),
+            ('user2', None, None)
+        ])
+
+        self.assertEqual('User One', format_author(None, 'user1'))
+        self.assertEqual('user2', format_author(None, 'user2'))
+
+    def test_show_full_names_false(self):
+        format_author = Chrome(self.env).format_author
+        self.env.config.set('trac', 'show_full_names', False)
+
+        self.assertEqual('user1', format_author(None, 'user1'))
+        self.assertEqual('user2', format_author(None, 'user2'))
+
+    def test_show_email_true(self):
+        format_author = Chrome(self.env).format_author
+        req = MockRequest(self.env, authname='user2')
+
+        author = format_author(None, 'user@domain.com', show_email=True)
+        self.assertEqual('user@domain.com', author)
+        author = format_author(req, 'user@domain.com', show_email=True)
+        self.assertEqual('user@domain.com', author)
+
+    def test_show_email_false(self):
+        format_author = Chrome(self.env).format_author
+        req = MockRequest(self.env, authname='user1')
+
+        author = format_author(None, 'user@domain.com', show_email=False)
+        self.assertEqual(u'user@\u2026', author)
+        author = format_author(req, 'user@domain.com', show_email=False)
+        self.assertEqual(u'user@\u2026', author)
+
+    def test_show_full_names_true_actor_has_email_view(self):
+        format_author = Chrome(self.env).format_author
+        self.env.config.set('trac', 'show_full_names', True)
+        self.env.insert_users([
+            ('user1', 'User One', 'user1@example.org'),
+            ('user2', None, None)
+        ])
+
+        self.assertEqual('User One', format_author(None, 'user1'))
+        self.assertEqual('user2', format_author(None, 'user2'))
+
+    def test_show_full_names_false_actor_has_email_view(self):
+        req = MockRequest(self.env, authname='user1')
+        format_author = Chrome(self.env).format_author
+        self.env.config.set('trac', 'show_full_names', False)
+
+        self.assertEqual('user1', format_author(req, 'user1'))
+        self.assertEqual('user2', format_author(req, 'user2'))
+
+    def test_show_email_addresses_true(self):
+        format_author = Chrome(self.env).format_author
+        self.env.config.set('trac', 'show_email_addresses', True)
+
+        self.assertEqual('user3@example.org',
+                         format_author(None, 'user3@example.org'))
+        self.assertEqual('user3@example.org',
+                         format_author(Request(), 'user3@example.org'))
+
+    def test_show_email_addresses_false(self):
+        format_author = Chrome(self.env).format_author
+        self.env.config.set('trac', 'show_email_addresses', False)
+
+        self.assertEqual(u'user3@\u2026',
+                         format_author(None, 'user3@example.org'))
+        self.assertEqual('user3@example.org',
+                         format_author(Request(), 'user3@example.org'))
+
+    def test_format_emails(self):
+        format_emails = Chrome(self.env).format_emails
+        to_format = 'user1@example.org, user2; user3@example.org'
+
+        self.assertEqual(u'user1@\u2026, user2, user3@\u2026',
+                         format_emails(None, to_format))
+
+    def test_format_emails_actor_has_email_view(self):
+        req = MockRequest(self.env, authname='user1')
+        context = web_context(req)
+        format_emails = Chrome(self.env).format_emails
+        to_format = 'user1@example.org, user2; user3@example.org'
+
+        self.assertEqual('user1@example.org, user2, user3@example.org',
+                         format_emails(context, to_format))
+
+    def test_format_emails_actor_no_email_view(self):
+        req = MockRequest(self.env, authname='user2')
+        context = web_context(req)
+        format_emails = Chrome(self.env).format_emails
+        to_format = 'user1@example.org, user2; user3@example.org'
+
+        self.assertEqual(u'user1@\u2026, user2, user3@\u2026',
+                         format_emails(context, to_format))
+
+
+class AuthorInfoTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub(enable=['trac.web.chrome.*',
+                                           'trac.perm.*',
+                                           'tracopt.perm.authz_policy'])
+        self.env.config.set('trac', 'permission_policies',
+                            'AuthzPolicy, DefaultPermissionPolicy')
+        fd, self.authz_file = tempfile.mkstemp()
+        with os.fdopen(fd, 'w') as f:
+            f.write(textwrap.dedent("""\
+                [wiki:WikiStart]
+                user2 = EMAIL_VIEW
+                [wiki:TracGuide]
+                user2 =
+                """))
+        PermissionSystem(self.env).grant_permission('user1', 'EMAIL_VIEW')
+        self.env.config.set('authz_policy', 'authz_file', self.authz_file)
+
+    def tearDown(self):
+        os.remove(self.authz_file)
+
+    def test_subject_is_anonymous(self):
+        chrome = Chrome(self.env)
+        req = Request()
+        self.assertEqual('<span class="trac-author-anonymous">anonymous</span>',
+                         str(chrome.authorinfo(req, 'anonymous')))
+        self.assertEqual('<span class="trac-author-anonymous">anonymous</span>',
+                         str(chrome.authorinfo_short('anonymous')))
+
+    def test_subject_is_none(self):
+        chrome = Chrome(self.env)
+        req = Request(authname=None)
+        self.assertEqual('<span class="trac-author">(none)</span>',
+                         str(chrome.authorinfo(req, '(none)')))
+        self.assertEqual('<span class="trac-author-none">(none)</span>',
+                         str(chrome.authorinfo(req, None)))
+        self.assertEqual('<span class="trac-author-none">(none)</span>',
+                         str(chrome.authorinfo(req, '')))
+        self.assertEqual('<span class="trac-author">(none)</span>',
+                         str(chrome.authorinfo_short('(none)')))
+        self.assertEqual('<span class="trac-author-none">(none)</span>',
+                         str(chrome.authorinfo_short(None)))
+        self.assertEqual('<span class="trac-author-none">(none)</span>',
+                         str(chrome.authorinfo_short('')))
+
+    def test_actor_has_email_view(self):
+        chrome = Chrome(self.env)
+        req = MockRequest(self.env, authname='user1')
+        self.assertEqual('<span class="trac-author">user@domain.com</span>',
+                         unicode(chrome.authorinfo(req, 'user@domain.com')))
+        self.assertEqual('<span class="trac-author">User One &lt;user@example.org&gt;</span>',
+                         unicode(chrome.authorinfo(req, 'User One <user@example.org>')))
+        self.assertEqual('<span class="trac-author">user</span>',
+                         str(chrome.authorinfo_short('User One <user@example.org>')))
+        self.assertEqual('<span class="trac-author">user</span>',
+                         str(chrome.authorinfo_short('user@example.org')))
+
+    def test_actor_no_email_view(self):
+        req = MockRequest(self.env, authname='user2')
+        authorinfo = Chrome(self.env).authorinfo
+        self.assertEqual(u'<span class="trac-author">user@\u2026</span>',
+                         unicode(authorinfo(req, 'user@domain.com')))
+        self.assertEqual(u'<span class="trac-author">User One &lt;user@\u2026&gt;</span>',
+                         unicode(authorinfo(req, 'User One <user@domain.com>')))
+
+    def test_actor_no_email_view_show_email_addresses(self):
+        self.env.config.set('trac', 'show_email_addresses', True)
+        req = MockRequest(self.env, authname='user2')
+        authorinfo = Chrome(self.env).authorinfo
+        self.assertEqual('<span class="trac-author">user@domain.com</span>',
+                         unicode(authorinfo(req, 'user@domain.com')))
+        self.assertEqual('<span class="trac-author">User One &lt;user@domain.com&gt;</span>',
+                         unicode(authorinfo(req, 'User One <user@domain.com>')))
+
+    def test_actor_no_email_view_no_req(self):
+        authorinfo = Chrome(self.env).authorinfo
+        self.assertEqual(u'<span class="trac-author">user@\u2026</span>',
+                         unicode(authorinfo(None, 'user@domain.com')))
+        self.assertEqual(u'<span class="trac-author">User One &lt;user@\u2026&gt;</span>',
+                         unicode(authorinfo(None, 'User One <user@domain.com>')))
+
+    def test_actor_has_email_view_for_resource(self):
+        authorinfo = Chrome(self.env).authorinfo
+        authorinfo_short = Chrome(self.env).authorinfo_short
+        req = MockRequest(self.env, authname='user2')
+        resource = Resource('wiki', 'WikiStart')
+        authorinfo = authorinfo(req, 'user@domain.com', resource=resource)
+        author_short = authorinfo_short('user@domain.com')
+        self.assertEqual(u'<span class="trac-author">user@domain.com</span>',
+                         unicode(authorinfo))
+        self.assertEqual(u'<span class="trac-author">user</span>',
+                         unicode(author_short))
+
+    def test_actor_has_email_view_for_resource_negative(self):
+        authorinfo = Chrome(self.env).authorinfo
+        authorinfo_short = Chrome(self.env).authorinfo_short
+        req = MockRequest(self.env, authname='user2')
+        resource = Resource('wiki', 'TracGuide')
+        author = authorinfo(req,  'user@domain.com', resource=resource)
+        author_short = authorinfo_short('user@domain.com')
+        self.assertEqual(u'<span class="trac-author">user@\u2026</span>',
+                         unicode(author))
+        self.assertEqual(u'<span class="trac-author">user</span>',
+                         unicode(author_short))
+
+
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ChromeTestCase))
     suite.addTest(unittest.makeSuite(ChromeTestCase2))
+    suite.addTest(unittest.makeSuite(NavigationOrderTestCase))
+    suite.addTest(unittest.makeSuite(FormatAuthorTestCase))
+    suite.addTest(unittest.makeSuite(AuthorInfoTestCase))
     return suite
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
