@@ -1,20 +1,21 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2013 Edgewall Software
+# Copyright (C) 2010-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os.path
 import re
 import sys
 import tempfile
+import textwrap
 import unittest
 from subprocess import PIPE, Popen
 
@@ -41,17 +42,17 @@ class TestStubRequestHandler(Component):
 
     filename = 'test_stub.html'
 
-    template = """\
-<!DOCTYPE html
-    PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
-    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml"
-      xmlns:py="http://genshi.edgewall.org/">
-  <body>
-    <h1>${greeting}</h1>
-  </body>
-</html>
-"""
+    template = textwrap.dedent("""\
+        <!DOCTYPE html
+            PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
+            "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
+        <html xmlns="http://www.w3.org/1999/xhtml"
+              xmlns:py="http://genshi.edgewall.org/">
+          <body>
+            <h1>${greeting}</h1>
+          </body>
+        </html>
+        """)
 
     def match_request(self, req):
         return req.path_info == '/test-stub'
@@ -522,6 +523,9 @@ class RequestDispatcherTestCase(unittest.TestCase):
         filepath = os.path.join(self.env.templates_dir,
                                 TestStubRequestHandler.filename)
         create_file(filepath, TestStubRequestHandler.template)
+        self.filename = os.path.join(self.env.path, 'test.txt')
+        self.data = 'contents\n'
+        create_file(self.filename, self.data, 'wb')
 
     def tearDown(self):
         self.env.reset_db_and_disk()
@@ -533,6 +537,11 @@ class RequestDispatcherTestCase(unittest.TestCase):
         self.env.insert_users([(sid, name, email, 0)])
         return sid, name, email
 
+    def _content(self):
+        yield 'line1,'
+        yield 'line2,'
+        yield 'line3\n'
+
     def test_invalid_default_date_format_raises_exception(self):
         self.env.config.set('trac', 'default_date_format', u'ĭšo8601')
 
@@ -606,6 +615,104 @@ class RequestDispatcherTestCase(unittest.TestCase):
         self.assertEqual('200 Ok', req.status_sent[0])
         self.assertIn('<h1>Hello World</h1>', req.response_sent.getvalue())
 
+    def test_set_valid_xsendfile_header(self):
+        """Send file using xsendfile header."""
+        self.env.config.set('trac', 'use_xsendfile', True)
+        self.env.config.set('trac', 'xsendfile_header', 'X-Accel-Redirect')
+
+        req = MockRequest(self.env)
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+
+        # File is sent using xsendfile.
+        self.assertRaises(RequestDone, req.send_file, self.filename)
+        self.assertEqual(['200 Ok'], req.status_sent)
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertEqual(self.filename, req.headers_sent['X-Accel-Redirect'])
+        self.assertNotIn('X-Sendfile', req.headers_sent)
+        self.assertEqual(None, req._response)
+        self.assertEqual('', req.response_sent.getvalue())
+
+    def _test_file_not_sent_using_xsendfile_header(self, xsendfile_header):
+        req = MockRequest(self.env)
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+
+        # File is not sent using xsendfile.
+        self.assertRaises(RequestDone, req.send_file, self.filename)
+        self.assertEqual(['200 Ok'], req.status_sent)
+        self.assertEqual('text/plain', req.headers_sent['Content-Type'])
+        self.assertNotIn(xsendfile_header, req.headers_sent)
+        self.assertEqual('_FileWrapper', type(req._response).__name__)
+        self.assertEqual('', req.response_sent.getvalue())
+
+    def test_set_invalid_xsendfile_header(self):
+        """Not sent by xsendfile header because header is invalid."""
+        xsendfile_header = '(X-SendFile)'
+        self.env.config.set('trac', 'use_xsendfile', True)
+        self.env.config.set('trac', 'xsendfile_header', xsendfile_header)
+
+        self._test_file_not_sent_using_xsendfile_header(xsendfile_header)
+
+    def test_xsendfile_is_disabled(self):
+        """Not sent by xsendfile header because xsendfile is disabled."""
+        xsendfile_header = 'X-SendFile'
+        self.env.config.set('trac', 'use_xsendfile', False)
+        self.env.config.set('trac', 'xsendfile_header', xsendfile_header)
+
+        self._test_file_not_sent_using_xsendfile_header(xsendfile_header)
+
+    def _test_configurable_headers(self, method):
+        # Reserved headers not allowed.
+        content_type = 'not-allowed'
+        self.env.config.set('http-headers', 'Content-Type', content_type)
+        # Control code not allowed.
+        custom1 = '\x00custom1'
+        self.env.config.set('http-headers', 'X-Custom-1', custom1)
+        # Many special characters allowed in header name.
+        custom2 = 'Custom2-!#$%&\'*+.^_`|~'
+        self.env.config.set('http-headers', custom2, 'custom2')
+        # Some special characters not allowed in header name.
+        self.env.config.set('http-headers', 'X-Custom-(3)', 'custom3')
+
+        req = MockRequest(self.env, method='POST')
+        request_dispatcher = RequestDispatcher(self.env)
+        request_dispatcher.set_default_callbacks(req)
+        self.assertRaises(RequestDone, getattr(req, method), self._content())
+
+        self.assertNotEqual('not-allowed', req.headers_sent.get('Content-Type'))
+        self.assertNotIn('x-custom-1', req.headers_sent)
+        self.assertIn(custom2.lower(), req.headers_sent)
+        self.assertNotIn('x-custom-(3)', req.headers_sent)
+
+    def test_send_configurable_headers(self):
+        self._test_configurable_headers('send')
+
+    def test_send_error_configurable_headers(self):
+        self._test_configurable_headers('send_error')
+
+    def test_send_configurable_headers_no_override(self):
+        """Headers in request not overridden by configurable headers."""
+        self.env.config.set('http-headers', 'X-XSS-Protection', '1; mode=block')
+        request_dispatcher = RequestDispatcher(self.env)
+        req1 = MockRequest(self.env)
+        request_dispatcher.set_default_callbacks(req1)
+
+        self.assertRaises(RequestDone, req1.send, self._content())
+
+        self.assertNotIn('X-XSS-protection', req1.headers_sent)
+        self.assertIn('x-xss-protection', req1.headers_sent)
+        self.assertEqual('1; mode=block', req1.headers_sent['x-xss-protection'])
+
+        req2 = MockRequest(self.env, method='POST')
+        request_dispatcher.set_default_callbacks(req2)
+
+        self.assertRaises(RequestDone, req2.send, self._content())
+
+        self.assertNotIn('x-xss-protection', req2.headers_sent)
+        self.assertIn('X-XSS-Protection', req2.headers_sent)
+        self.assertEqual('0', req2.headers_sent['X-XSS-Protection'])
+
 
 class HdfdumpTestCase(unittest.TestCase):
 
