@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import time
 from datetime import datetime
@@ -294,19 +294,21 @@ class SessionTestCase(unittest.TestCase):
             WHERE sid='123456' AND name='foo'
             """)[0][0])
 
-    def test_purge_anonymous_session(self):
-        """
-        Verify that old sessions get purged.
-        """
+    def _purge_anonymous_session(self):
+        now = int(time_now())
+        lifetime = 90 * 86400  # default lifetime
         with self.env.db_transaction as db:
-            db("INSERT INTO session VALUES ('123456', 0, %s)", (0,))
-            db("INSERT INTO session VALUES ('987654', 0, %s)",
-               (int(time_now() - PURGE_AGE - 3600),))
-            db("""
+            db.executemany("INSERT INTO session VALUES (%s, 0, %s)",
+                           [('123456', 0),
+                            ('987654', now - lifetime - 3600),
+                            ('876543', now - lifetime + 3600),
+                            ('765432', now - 3600)])
+            db.executemany("""
                 INSERT INTO session_attribute
-                VALUES ('987654', 0, 'foo', 'bar')
-                """)
+                VALUES (%s, 0, 'foo', 'bar')
+                """, [('987654',), ('876543',), ('765432',)])
 
+        with self.env.db_transaction as db:
             # We need to modify a different session to trigger the purging
             req = MockRequest(self.env, authname='anonymous')
             req.incookie['trac_session'] = '123456'
@@ -314,9 +316,26 @@ class SessionTestCase(unittest.TestCase):
             session['foo'] = 'bar'
             session.save()
 
-        self.assertEqual(0, self.env.db_query("""
-            SELECT COUNT(*) FROM session WHERE sid='987654' AND authenticated=0
-            """)[0][0])
+        return [row[0] for row in self.env.db_query("""
+            SELECT sid FROM session WHERE authenticated=0 ORDER BY sid
+            """)]
+
+    def test_purge_anonymous_session(self):
+        """
+        Verify that old sessions get purged.
+        """
+        sids = self._purge_anonymous_session()
+        self.assertEqual(['123456', '765432', '876543'], sids)
+
+    def test_purge_anonymous_session_with_short_lifetime(self):
+        self.env.config.set('trac', 'anonymous_session_lifetime', '1')
+        sids = self._purge_anonymous_session()
+        self.assertEqual(['123456', '765432'], sids)
+
+    def test_purge_anonymous_session_disabled(self):
+        self.env.config.set('trac', 'anonymous_session_lifetime', '0')
+        sids = self._purge_anonymous_session()
+        self.assertEqual(['123456', '765432', '876543', '987654'], sids)
 
     def test_delete_empty_session(self):
         """
@@ -559,31 +578,65 @@ class SessionTestCase(unittest.TestCase):
             WHERE sid='john' AND name='foo'
             """)[0][0])
 
-    def test_session_set_email(self):
-        """Setting session email invalidates known_users cache."""
-        session = DetachedSession(self.env, 'user')
-        session['email'] = 'user@domain.org'
+    def test_create_new_session(self):
+        """Setting attribute on a new session invalidates known_users cache."""
+        sid = 'user'
+        session = DetachedSession(self.env, sid)
+        session.authenticated = True
 
         self.assertEqual([], list(self.env.get_known_users()))
         session.save()
-        email = None
-        for sid, name, email in self.env.get_known_users():
-            if sid == 'user':
-                break
-        self.assertEqual(session['email'], email)
+        known_users = list(self.env.get_known_users())
+        self.assertEqual(1, len(known_users))
+        self.assertEqual(sid, known_users[0][0])
+        self.assertIsNone(known_users[0][1])
+        self.assertIsNone(known_users[0][2])
+
+    def test_create_new_anonymous_session(self):
+        """Setting attribute on a new anonymous session doesn't invalidate
+        known_users cache."""
+        sid = 'anonymous'
+        session = DetachedSession(self.env, sid)
+        session.authenticated = False
+
+        self.assertEqual([], list(self.env.get_known_users()))
+        # insert a session record without invalidating cache
+        self.env.insert_users([('user', 'Name' 'user@example.com', 1)])
+        session.save()
+        self.assertEqual([], list(self.env.get_known_users()))
+
+        self.env.invalidate_known_users_cache()
+        self.assertEqual(1, len(list(self.env.get_known_users())))
+
+    def test_session_set_email(self):
+        """Setting session email invalidates known_users cache."""
+        sid = 'user'
+        email = 'user@domain.org'
+        session = DetachedSession(self.env, sid)
+        session['email'] = email
+
+        self.assertEqual([], list(self.env.get_known_users()))
+        session.save()
+        known_users = list(self.env.get_known_users())
+        self.assertEqual(1, len(known_users))
+        self.assertEqual(sid, known_users[0][0])
+        self.assertIsNone(known_users[0][1])
+        self.assertEqual(email, known_users[0][2])
 
     def test_session_set_name(self):
         """Setting session name invalidates known_users cache."""
-        session = DetachedSession(self.env, 'user')
-        session['name'] = 'The User'
+        sid = 'user'
+        name = 'The User'
+        session = DetachedSession(self.env, sid)
+        session['name'] = name
 
         self.assertEqual([], list(self.env.get_known_users()))
         session.save()
-        name = None
-        for sid, name, email in self.env.get_known_users():
-            if sid == 'user':
-                break
-        self.assertEqual(session['name'], name)
+        known_users = list(self.env.get_known_users())
+        self.assertEqual(1, len(known_users))
+        self.assertEqual(sid, known_users[0][0])
+        self.assertEqual(name, known_users[0][1])
+        self.assertIsNone(known_users[0][2])
 
     def test_session_admin_list(self):
         auth_list, anon_list, all_list = _prep_session_table(self.env)
