@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2019 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
@@ -8,11 +8,11 @@
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
@@ -24,25 +24,26 @@ import os
 from StringIO import StringIO
 
 from genshi.builder import tag, Element
-from genshi.core import Stream, Markup, escape
+from genshi.core import Stream
 from genshi.input import HTMLParser, ParseError
 from genshi.util import plaintext
 
 from trac.core import *
 from trac.mimeview import *
 from trac.resource import get_relative_resource, get_resource_url
-from trac.util import arity
+from trac.util import arity, as_int
+from trac.util.html import Markup, TracHTMLSanitizer, escape, to_fragment
 from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
-                           unicode_quote, unicode_quote_plus, unquote_label
-from trac.util.html import TracHTMLSanitizer
+                           unicode_quote, unquote_label
 from trac.util.translation import _, tag_
 from trac.wiki.api import WikiSystem, parse_args
 from trac.wiki.parser import WikiParser, parse_processor_args
 
 __all__ = ['wiki_to_html', 'wiki_to_oneliner', 'wiki_to_outline',
-           'Formatter', 'format_to', 'format_to_html', 'format_to_oneliner',
-           'extract_link', 'split_url_into_path_query_fragment',
-           'concat_path_query_fragment']
+           'Formatter', 'MacroError', 'ProcessorError', 'format_to',
+           'format_to_html', 'format_to_oneliner', 'extract_link',
+           'split_url_into_path_query_fragment', 'concat_path_query_fragment']
+
 
 
 def system_message(msg, text=None):
@@ -111,10 +112,24 @@ def _markup_to_unicode(markup):
 
 
 class MacroError(TracError):
+    """Exception raised on incorrect macro usage.
+
+    The exception is trapped by the wiki formatter and the message is
+    rendered in a `pre` tag, wrapped in a div with class `system-message`.
+
+    :since: 1.0.11
+    """
     pass
 
 
 class ProcessorError(TracError):
+    """Exception raised on incorrect processor usage.
+
+    The exception is trapped by the wiki formatter and the message is
+    rendered in a `pre` tag, wrapped in a div with class `system-message`.
+
+    :since: 0.12
+    """
     pass
 
 
@@ -131,7 +146,7 @@ class WikiProcessor(object):
         :param name: the name of the processor
         :param args: extra parameters for the processor
 
-        (since 0.11)
+        (''since 0.11'')
         """
         self.formatter = formatter
         self.env = formatter.env
@@ -160,7 +175,9 @@ class WikiProcessor(object):
                              'span': True, 'Span': True,
                              }.get(name)
 
-        self._sanitizer = TracHTMLSanitizer(formatter.wiki.safe_schemes)
+        self._sanitizer = TracHTMLSanitizer(
+            safe_schemes=formatter.wiki.safe_schemes,
+            safe_origins=formatter.wiki.safe_origins)
 
         self.processor = builtin_processors.get(name)
         if not self.processor:
@@ -171,7 +188,10 @@ class WikiProcessor(object):
                         if hasattr(macro_provider, 'expand_macro'):
                             self.processor = self._macro_processor
                         else:
-                            self.processor = self._legacy_macro_processor
+                            raise TracError(
+                                tag_("Pre-0.11 macros with the %(method)s "
+                                     "method are no longer supported.",
+                                     method=tag.code("render_macro")))
                         self.macro_provider = macro_provider
                         self.inline_check = getattr(macro_provider, 'is_inline',
                                                     False)
@@ -213,7 +233,12 @@ class WikiProcessor(object):
         return ''
 
     def _default_processor(self, text):
-        return tag.pre(text, class_="wiki")
+        if self.args and 'lineno' in self.args:
+            self.name = \
+                Mimeview(self.formatter.env).get_mimetype('text/plain')
+            return self._mimeview_processor(text)
+        else:
+            return tag.pre(text, class_="wiki")
 
     def _html_processor(self, text):
         if WikiSystem(self.env).render_unsafe_content:
@@ -221,7 +246,7 @@ class WikiProcessor(object):
         try:
             stream = Stream(HTMLParser(StringIO(text)))
             return (stream | self._sanitizer).render('xhtml', encoding=None)
-        except ParseError, e:
+        except ParseError as e:
             self.env.log.warn(e)
             line = unicode(text).splitlines()[e.lineno - 1].strip()
             return system_message(_('HTML parsing error: %(message)s',
@@ -275,7 +300,7 @@ class WikiProcessor(object):
             elt = self._elt_processor('tr', self._format_row, text)
             self.formatter.open_table()
             return elt
-        except ProcessorError, e:
+        except ProcessorError as e:
             return system_message(e)
 
     def _table_processor(self, text):
@@ -284,7 +309,7 @@ class WikiProcessor(object):
         self.args.setdefault('class', 'wiki')
         try:
             return self._elt_processor('table', self._format_table, text)
-        except ProcessorError, e:
+        except ProcessorError as e:
             return system_message(e)
 
     def _tablecell_processor(self, eltname, text):
@@ -332,12 +357,6 @@ class WikiProcessor(object):
 
     # generic processors
 
-    def _legacy_macro_processor(self, text): # TODO: remove in 0.12
-        self.env.log.warning('Executing pre-0.11 Wiki macro %s by '
-                             'provider %s', self.name, self.macro_provider)
-        return self.macro_provider.render_macro(self.formatter.req, self.name,
-                                                text)
-
     def _macro_processor(self, text):
         self.env.log.debug('Executing Wiki macro %s by provider %s',
                            self.name, self.macro_provider)
@@ -349,8 +368,23 @@ class WikiProcessor(object):
                                                     text)
 
     def _mimeview_processor(self, text):
-        return Mimeview(self.env).render(self.formatter.context,
-                                         self.name, text)
+        annotations = []
+        context = self.formatter.context.child()
+        args = self.args.copy() if self.args else self.args
+        if args and 'lineno' in args:
+            lineno = as_int(args.pop('lineno'), 1, min=1)
+            context.set_hints(lineno=lineno)
+            id = str(args.pop('id', '')) or \
+                 self.formatter._unique_anchor('a')
+            context.set_hints(id=id + '-')
+            if 'marks' in args:
+                context.set_hints(marks=args.pop('marks'))
+            annotations.append('lineno')
+        if args:  # Remaining args are assumed to be lexer options
+            context.set_hints(lexer_options=args)
+        return tag.div(class_='wiki-code')(
+            Mimeview(self.env).render(context, self.name, text,
+                                      annotations=annotations))
     # TODO: use convert('text/html') instead of render
 
     def process(self, text, in_paragraph=False):
@@ -368,7 +402,7 @@ class WikiProcessor(object):
         else:
             return self.inline_check
 
-    def ensure_inline(self, text):
+    def ensure_inline(self, text, in_paragraph=True):
         content_for_span = None
         interrupt_paragraph = False
         if isinstance(text, Element):
@@ -394,7 +428,7 @@ class WikiProcessor(object):
                 interrupt_paragraph = True
         if content_for_span:
             text = tag.span(class_='code-block')(*content_for_span)
-        elif interrupt_paragraph:
+        elif interrupt_paragraph and in_paragraph:
             text = "</p>%s<p>" % _markup_to_unicode(text)
         return text
 
@@ -407,13 +441,7 @@ class Formatter(object):
 
     flavor = 'default'
 
-    # 0.10 compatibility
-    INTERTRAC_SCHEME = WikiParser.INTERTRAC_SCHEME
-    QUOTED_STRING = WikiParser.QUOTED_STRING
-    LINK_SCHEME = WikiParser.LINK_SCHEME
-
     def __init__(self, env, context):
-        """Note: `req` is still temporarily used."""
         self.env = env
         self.context = context.child()
         self.context.set_hints(disable_warnings=True)
@@ -559,10 +587,10 @@ class Formatter(object):
         return self._indirect_tag_handler(match, 'MM_SUPERSCRIPT')
 
     def _inlinecode_formatter(self, match, fullmatch):
-        return tag.tt(fullmatch.group('inline'))
+        return tag.code(fullmatch.group('inline'))
 
     def _inlinecode2_formatter(self, match, fullmatch):
-        return tag.tt(fullmatch.group('inline2'))
+        return tag.code(fullmatch.group('inline2'))
 
     # pre-0.12 public API (no longer used by Trac itself but kept for plugins)
 
@@ -620,7 +648,7 @@ class Formatter(object):
     def _make_lhref_link(self, match, fullmatch, rel, ns, target, label):
         if not label: # e.g. `[http://target]` or `[wiki:target]`
             if target:
-                if target.startswith('//'):     # for `[http://target]`
+                if ns and target.startswith('//'):     # for `[http://target]`
                     label = ns + ':' + target   #  use `http://target`
                 else:                           # for `wiki:target`
                     label = target.lstrip('/')  #  use only `target`
@@ -692,26 +720,18 @@ class Formatter(object):
     def get_intertrac_url(self, ns, target):
         intertrac = self.env.config['intertrac']
         url = intertrac.get(ns + '.url')
-        if not url and ns == 'trac':
-            url = 'http://trac.edgewall.org'
+        name = _("Trac project %(name)s", name=ns)
+        if not url and ns.lower() == 'trac':
+            url = 'https://trac.edgewall.org'
+            name = _("The Trac Project")
         if url:
-            name = intertrac.get(ns + '.title', 'Trac project %s' % ns)
-            compat = intertrac.getbool(ns + '.compat', 'false')
-            # set `compat` default to False now that 0.10 is widely used
-            # TODO: remove compatibility code completely for 1.0 release
-            if compat:
-                sep = target.find(':')
-                if sep != -1:
-                    url = '%s/%s/%s' % (url, target[:sep], target[sep + 1:])
-                else:
-                    url = '%s/search?q=%s' % (url, unicode_quote_plus(target))
-            else:
-                url = '%s/intertrac/%s' % (url, unicode_quote(target))
+            name = intertrac.get(ns + '.title', name)
+            url = '%s/intertrac/%s' % (url, unicode_quote(target))
             if target:
-                title = _('%(target)s in %(name)s', target=target, name=name)
+                title = _("%(target)s in %(name)s", target=target, name=name)
             else:
                 title = name
-            return (url, title)
+            return url, title
 
     def shorthand_intertrac_helper(self, ns, target, label, fullmatch):
         if fullmatch: # short form
@@ -732,8 +752,7 @@ class Formatter(object):
                 return self._make_ext_link(url, label, title)
 
     def _make_ext_link(self, url, text, title=''):
-        local_url = self.env.project_url or \
-                    (self.req or self.env).abs_href.base
+        local_url = self.env.project_url or self.env.abs_href.base
         if not url.startswith(local_url):
             return tag.a(tag.span(u'\u200b', class_="icon"), text,
                          class_="ext-link", href=url, title=title or None)
@@ -753,6 +772,15 @@ class Formatter(object):
             label = format_to_oneliner(self.env, self.context, label)
         return '<span class="wikianchor" id="%s">%s</span>' % (anchor, label)
 
+    def _unique_anchor(self, anchor):
+        i = 1
+        anchor_base = anchor
+        while anchor in self._anchors:
+            anchor = anchor_base + str(i)
+            i += 1
+        self._anchors[anchor] = True
+        return anchor
+
     # WikiMacros or WikiCreole links
 
     def _macrolink_formatter(self, match, fullmatch):
@@ -789,15 +817,20 @@ class Formatter(object):
             args = name[:-1] or '*'
         else:
             args = fullmatch.group('macroargs')
+        in_paragraph = not (getattr(self, 'in_list_item', True) or
+                            getattr(self, 'in_table', True) or
+                            getattr(self, 'in_def_list', True))
         try:
-            return macro.ensure_inline(macro.process(args))
-        except MacroError, e:
-            return system_message(e)
-        except Exception, e:
-            self.env.log.error('Macro %s(%s) failed:%s', name, args,
+            return macro.ensure_inline(macro.process(args), in_paragraph)
+        except MacroError as e:
+            return system_message(_("Macro %(name)s(%(args)s) failed",
+                                    name=name, args=args), to_fragment(e))
+        except Exception as e:
+            self.env.log.error("Macro %s(%s) failed for %s:%s", name,
+                               args, self.resource,
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Macro %(name)s(%(args)s) failed",
-                                    name=name, args=args), to_unicode(e))
+                                    name=name, args=args), to_fragment(e))
 
     # Headings
 
@@ -819,12 +852,7 @@ class Formatter(object):
             if not anchor or anchor[0].isdigit() or anchor[0] in '.-':
                 # an ID must start with a Name-start character in XHTML
                 anchor = 'a' + anchor # keeping 'a' for backward compat
-        i = 1
-        anchor_base = anchor
-        while anchor in self._anchors:
-            anchor = anchor_base + str(i)
-            i += 1
-        self._anchors[anchor] = True
+        anchor = self._unique_anchor(anchor)
         if shorten:
             heading = format_to_oneliner(self.env, self.context, htext, True)
         return (depth, heading, anchor)
@@ -938,7 +966,11 @@ class Formatter(object):
     # Definition Lists
 
     def _definition_formatter(self, match, fullmatch):
-        tmp = '</dd>' if self.in_def_list else '<dl class="wiki">'
+        if self.in_def_list:
+            tmp = '</dd>'
+        else:
+            self.close_paragraph()
+            tmp = '<dl class="wiki">'
         definition = match[:match.find('::')]
         tmp += '<dt>%s</dt><dd>' % format_to_oneliner(self.env, self.context,
                                                       definition)
@@ -985,7 +1017,7 @@ class Formatter(object):
                 self._quote_stack.append(d)
                 self._set_tab(d)
                 class_attr = ' class="citation"' if citation else ''
-                self.out.write('<blockquote%s>' % class_attr + os.linesep)
+                self.out.write('<blockquote%s>\n' % class_attr)
             if citation:
                 for d in range(quote_depth+1, depth+1):
                     open_one_quote(d)
@@ -995,7 +1027,7 @@ class Formatter(object):
             self.close_table()
             self.close_paragraph()
             self._quote_stack.pop()
-            self.out.write('</blockquote>' + os.linesep)
+            self.out.write('</blockquote>\n')
         quote_depth = self._get_quote_depth()
         if depth > quote_depth:
             self._set_tab(depth)
@@ -1089,7 +1121,7 @@ class Formatter(object):
             self.close_list()
             self.close_def_list()
             self.in_table = 1
-            self.out.write('<table class="wiki">' + os.linesep)
+            self.out.write('<table class="wiki">\n')
 
     def open_table_row(self, params=''):
         if not self.in_table_row:
@@ -1109,20 +1141,20 @@ class Formatter(object):
     def close_table(self):
         if self.in_table:
             self.close_table_row(force=True)
-            self.out.write('</table>' + os.linesep)
+            self.out.write('</table>\n')
             self.in_table = 0
 
     # Paragraphs
 
     def open_paragraph(self):
         if not self.paragraph_open:
-            self.out.write('<p>' + os.linesep)
+            self.out.write('<p>\n')
             self.paragraph_open = 1
 
     def close_paragraph(self):
         self.flush_tags()
         if self.paragraph_open:
-            self.out.write('</p>' + os.linesep)
+            self.out.write('</p>\n')
             self.paragraph_open = 0
 
     # Code blocks
@@ -1160,7 +1192,7 @@ class Formatter(object):
                         self.code_buf = [l[code_indent:]
                                          for l in self.code_buf]
                     self.code_buf.append('')
-                code_text = os.linesep.join(self.code_buf)
+                code_text = '\n'.join(self.code_buf)
                 processed = self._exec_processor(self.code_processor,
                                                  code_text)
                 self.out.write(_markup_to_unicode(processed))
@@ -1186,11 +1218,15 @@ class Formatter(object):
     def _exec_processor(self, processor, text):
         try:
             return processor.process(text)
-        except Exception, e:
-            self.env.log.error('Processor %s failed:%s', processor.name,
+        except ProcessorError as e:
+            return system_message(_("Processor %(name)s failed",
+                                    name=processor.name), to_fragment(e))
+        except Exception as e:
+            self.env.log.error("Processor %s failed for %s:%s",
+                               processor.name, self.resource,
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Processor %(name)s failed",
-                                    name=processor.name), to_unicode(e))
+                                    name=processor.name), to_fragment(e))
 
     # > quotes
 
@@ -1268,7 +1304,6 @@ class Formatter(object):
         self.paragraph_open = 0
         return source
 
-
     def format(self, text, out=None, escape_newlines=False):
         text = self.reset(text, out)
         if isinstance(text, basestring):
@@ -1300,7 +1335,7 @@ class Formatter(object):
                 self.close_indentation()
                 self.close_list()
                 self.close_def_list()
-                self.out.write('<hr />' + os.linesep)
+                self.out.write('<hr />\n')
                 continue
             # Handle new paragraph
             if line == '':
@@ -1316,6 +1351,10 @@ class Formatter(object):
             if not line.startswith(' '):
                 self._tabstops = []
 
+            # Handle end of indentation
+            if not line.startswith(' ') and self._quote_stack:
+                self.close_indentation()
+
             self.in_list_item = False
             self.in_quote = False
             # Throw a bunch of regexps on the problem
@@ -1335,7 +1374,7 @@ class Formatter(object):
                 self.close_table()
             self.continue_table = 0
 
-            sep = os.linesep
+            sep = '\n'
             if not(self.in_list_item or self.in_def_list or self.in_table):
                 if len(result):
                     self.open_paragraph()
@@ -1409,15 +1448,15 @@ class OneLinerFormatter(Formatter):
                     in_code_block -= 1
                     if in_code_block == 0:
                         if processor != 'comment':
-                            buf.write(' [...]' + os.linesep)
+                            buf.write(' [...]\n')
                         processor = None
             elif in_code_block:
                 if not processor:
                     if line.startswith('#!'):
                         processor = line[2:].strip()
             else:
-                buf.write(line + os.linesep)
-        result = buf.getvalue()[:-len(os.linesep)]
+                buf.write(line + '\n')
+        result = buf.getvalue()[:-len('\n')]
 
         if shorten:
             result = shorten_line(result)
