@@ -1,16 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2013 Edgewall Software
+# Copyright (C) 2008-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import unittest
@@ -67,7 +67,7 @@ class TestWikiAddAttachment(FunctionalTwillTestCaseSetup):
         """Add attachment to a wiki page. Test that the attachment
         button reads 'Attach file' when no files have been attached, and
         'Attach another file' when there are existing attachments.
-        Feature added in http://trac.edgewall.org/ticket/10281"""
+        Feature added in https://trac.edgewall.org/ticket/10281"""
         name = self._tester.create_wiki_page()
         self._tester.go_to_wiki(name)
         tc.find("Attach file")
@@ -81,13 +81,7 @@ class TestWikiAddAttachment(FunctionalTwillTestCaseSetup):
                 'href="/zip-attachment/wiki/%s/">.zip</a>' % name)
 
 
-class TestWikiPageManipulator(FunctionalTwillTestCaseSetup):
-    def runTest(self):
-        plugin_name = self.__class__.__name__
-        env = self._testenv.get_trac_environment()
-        env.config.set('components', plugin_name + '.*', 'enabled')
-        env.config.save()
-        create_file(os.path.join(env.plugins_dir, plugin_name + '.py'), """\
+_plugin_py = """\
 from genshi.builder import tag
 from trac.core import Component, implements
 from trac.util.translation import tag_
@@ -106,7 +100,17 @@ class WikiPageManipulator(Component):
                          " line %(number)s.", number=tag.strong('10'))
         yield field, tag_("The field %(field)s cannot be empty.",
                           field=tag.strong(field))
-""")
+"""
+
+
+class TestWikiPageManipulator(FunctionalTwillTestCaseSetup):
+    def runTest(self):
+        plugin_name = self.__class__.__name__
+        env = self._testenv.get_trac_environment()
+        env.config.set('components', plugin_name + '.*', 'enabled')
+        env.config.save()
+        create_file(os.path.join(env.plugins_dir, plugin_name + '.py'),
+                    _plugin_py)
         self._testenv.restart()
 
         try:
@@ -280,7 +284,7 @@ class TestWikiRename(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket4812(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/4812"""
+        """Test for regression of https://trac.edgewall.org/ticket/4812"""
         pagename = random_unique_camel() + '/' + random_unique_camel()
         self._tester.create_wiki_page(pagename)
         self._tester.attach_file_to_wiki(pagename)
@@ -326,7 +330,7 @@ class ReStructuredTextCodeBlockTest(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket8976(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/8976
+        """Test for regression of https://trac.edgewall.org/ticket/8976
         Test fine grained permissions policy on wiki for specific page
         versions."""
         name = self._tester.create_wiki_page()
@@ -374,7 +378,7 @@ class RegressionTestTicket8976(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket10274(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/10274"""
+        """Test for regression of https://trac.edgewall.org/ticket/10274"""
         self._tester.go_to_wiki('WikiStart/..')
         tc.find("Invalid Wiki page name 'WikiStart/..'")
         self._tester.go_to_wiki('../WikiStart')
@@ -385,7 +389,7 @@ class RegressionTestTicket10274(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket10850(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/10850"""
+        """Test for regression of https://trac.edgewall.org/ticket/10850"""
         pagename = self._tester.create_wiki_page()
         # colon characters
         self._tester.attach_file_to_wiki(
@@ -416,7 +420,7 @@ class RegressionTestTicket10850(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket10957(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/10957"""
+        """Test for regression of https://trac.edgewall.org/ticket/10957"""
 
         self._tester.go_to_front()
         try:
@@ -483,7 +487,7 @@ class RegressionTestTicket10957(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket11302(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11302"""
+        """Test for regression of https://trac.edgewall.org/ticket/11302"""
         pagename = self._tester.create_wiki_page()
         self._tester.attach_file_to_wiki(
             pagename, description="illustrates [./@1#point1]")
@@ -494,15 +498,15 @@ class RegressionTestTicket11302(FunctionalTwillTestCaseSetup):
 
 class RegressionTestTicket11518(FunctionalTwillTestCaseSetup):
     def runTest(self):
-        """Test for regression of http://trac.edgewall.org/ticket/11518
+        """Test for regression of https://trac.edgewall.org/ticket/11518
         ResourceNotFound should be raised when version is invalid.
         """
         tc.go(self._tester.url + '/wiki/WikiStart?version=1abc')
-        tc.find(r"<h1>Trac Error</h1>")
-        tc.find('No version "1abc" for Wiki page "WikiStart')
+        tc.find(r"<h1>Error: Bad Request</h1>")
+        tc.find(r"Invalid value for request argument <em>version</em>.")
         tc.go(self._tester.url + '/wiki/WikiStart?version=')
-        tc.find(r"<h1>Trac Error</h1>")
-        tc.find('No version "" for Wiki page "WikiStart')
+        tc.find(r"<h1>Error: Bad Request</h1>")
+        tc.find(r"Invalid value for request argument <em>version</em>.")
 
 
 def functionalSuite(suite=None):
