@@ -1,18 +1,16 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012 Edgewall Software
+# Copyright (C) 2012-2019 Edgewall Software
 # Copyright (C) 2006-2011, Herbert Valerio Riedel <hvr@gnu.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import with_statement
+# history and logs, available at https://trac.edgewall.org/log/.
 
 from datetime import datetime
 import itertools
@@ -22,9 +20,10 @@ from genshi.builder import tag
 from genshi.core import Markup
 
 from trac.cache import cached
-from trac.config import BoolOption, IntOption, PathOption, Option
-from trac.core import *
-from trac.util import TracError, shorten_line
+from trac.config import BoolOption, IntOption, ListOption, PathOption, Option
+from trac.core import Component, TracError, implements
+from trac.env import ISystemInfoProvider
+from trac.util import shorten_line
 from trac.util.datefmt import FixedOffset, to_timestamp, format_datetime
 from trac.util.text import to_unicode, exception_to_unicode
 from trac.util.translation import _
@@ -100,6 +99,22 @@ class GitCachedRepository(CachedRepository):
                 return count > 0
             return False
 
+        def needs_sync():
+            max_holders = 999
+            revs = sorted(set(rev for refname, rev in repos.git.get_refs()))
+            step = max_holders - 1
+            for idx in xrange(0, len(revs), step):
+                revs_ = revs[idx:idx + step]
+                holders = ','.join(('%s',) * len(revs_))
+                args = [self.id]
+                args.extend(revs_)
+                query = """SELECT COUNT(*) FROM revision
+                           WHERE repos=%%s AND rev IN (%s)""" % holders
+                for count, in self.env.db_query(query, args):
+                    if count < len(revs_):
+                        return True
+            return False
+
         def traverse(rev, seen):
             revs = []
             merge_revs = []
@@ -121,9 +136,7 @@ class GitCachedRepository(CachedRepository):
                     revs[idx:idx] = traverse(rev, seen)
             return revs
 
-        while True:
-            repos.sync()
-            repos_youngest = repos.youngest_rev or ''
+        def sync_revs():
             updated = False
             seen = set()
 
@@ -136,41 +149,45 @@ class GitCachedRepository(CachedRepository):
                     rev = revs.pop()
                     self.log.info("Trying to sync revision [%s]", rev)
                     cset = repos.get_changeset(rev)
-                    with self.env.db_transaction as db:
-                        try:
-                            self._insert_changeset(db, rev, cset)
-                            updated = True
-                        except self.env.db_exc.IntegrityError, e:
-                            self.log.info('Revision %s already cached: %r',
-                                          rev, e)
-                            db.rollback()
-                            continue
+                    try:
+                        self.insert_changeset(rev, cset)
+                        updated = True
+                    except self.env.db_exc.IntegrityError as e:
+                        self.log.info('Revision %s already cached: %r', rev, e)
+                        continue
                     if feedback:
                         feedback(rev)
 
-            if updated:
-                continue  # sync again
+            return updated
 
-            if meta_youngest != repos_youngest:
-                with self.env.db_transaction as db:
-                    db("""
-                        UPDATE repository SET value=%s WHERE id=%s AND name=%s
-                        """, (repos_youngest, self.id, CACHE_YOUNGEST_REV))
-                    del self.metadata
-            return
+        with self.env.db_query:
+            while True:
+                repos.sync()
+                if needs_sync() and sync_revs():
+                    continue  # sync again
+                repos_youngest = repos.youngest_rev or ''
+                if meta_youngest != repos_youngest:
+                    with self.env.db_transaction as db:
+                        db("""
+                            UPDATE repository SET value=%s
+                            WHERE id=%s AND name=%s
+                            """, (repos_youngest, self.id, CACHE_YOUNGEST_REV))
+                        del self.metadata
+                return
 
 
 class GitCachedChangeset(CachedChangeset):
-    """Git-specific cached changeset.
+    """Git-specific cached changeset."""
 
-    Handles get_branches()
-    """
     def get_branches(self):
         _rev = self.rev
 
         return [(k, v == _rev) for k, v in
                  self.repos.repos.git.get_branch_contains(_rev, resolve=True)]
 
+    def get_tags(self):
+        return self.repos.repos.git.get_tags(self.rev)
+
 
 def _last_iterable(iterable):
     """helper for detecting last iteration in for-loop"""
@@ -204,26 +221,46 @@ def _parse_user_time(s):
     return user, time
 
 
+_file_type_mask = 0170000
+
+
+def _is_dir(mode):
+    if mode is None:
+        return False
+    return (mode & _file_type_mask) in (0040000, 0160000)
+
+
+def _is_submodule(mode):
+    if mode is None:
+        return False
+    return (mode & _file_type_mask) == 0160000
+
+
 class GitConnector(Component):
 
-    implements(IRepositoryConnector, IWikiSyntaxProvider)
+    implements(IRepositoryConnector, ISystemInfoProvider, IWikiSyntaxProvider)
 
     def __init__(self):
         self._version = None
 
         try:
             self._version = PyGIT.Storage.git_version(git_bin=self.git_bin)
-        except PyGIT.GitError, e:
+        except PyGIT.GitError as e:
             self.log.error("GitError: %s", e)
 
         if self._version:
             self.log.info("detected GIT version %s", self._version['v_str'])
-            self.env.systeminfo.append(('GIT', self._version['v_str']))
             if not self._version['v_compatible']:
                 self.log.error("GIT version %s installed not compatible"
                                "(need >= %s)", self._version['v_str'],
                                self._version['v_min_str'])
 
+    # ISystemInfoProvider methods
+
+    def get_system_info(self):
+        if self._version:
+            yield 'GIT', self._version['v_str']
+
     # IWikiSyntaxProvider methods
 
     def _format_sha_link(self, formatter, sha, label):
@@ -249,7 +286,7 @@ class GitConnector(Component):
             return tag.a(label, class_='changeset',
                          title=shorten_line(changeset.message),
                          href=formatter.href.changeset(sha, repos.reponame))
-        except Exception, e:
+        except Exception as e:
             return tag.a(label, class_='missing changeset',
                          title=to_unicode(e), rel='nofollow')
 
@@ -404,7 +441,7 @@ class CsetPropertyRenderer(Component):
                              title=shorten_line(cset.message),
                              href=context.href.changeset(sha, repos.reponame))
 
-            except Exception, e:
+            except Exception as e:
                 return tag.a(sha, class_='missing changeset',
                              title=to_unicode(e), rel='nofollow')
 
@@ -443,9 +480,10 @@ class CsetPropertyRenderer(Component):
                                              "itself.")),
                                     class_='hint'),
                            tag.br(),
-                           tag.span(Markup(_("Use the <tt>(diff)</tt> links "
-                                             "above to see all the changes "
-                                             "relative to each parent.")),
+                           tag.span(Markup(_("Use the <code>(diff)</code> "
+                                             "links above to see all the "
+                                             "changes relative to each "
+                                             "parent.")),
                                     class_='hint'))
 
             # simple non-merge commit
@@ -489,11 +527,11 @@ class GitRepository(Repository):
                                            git_bin=git_bin,
                                            git_fs_encoding=git_fs_encoding)
             self._git = factory.getInstance()
-        except PyGIT.GitError, e:
+        except PyGIT.GitError as e:
             log.error(exception_to_unicode(e))
             raise InvalidRepository(
-                _("%(path)s does not appear to be a Git repository.",
-                  path=path))
+                _('"%(name)s" is not readable or not a Git repository.',
+                  name=params.get('name') or '(default)'))
 
         Repository.__init__(self, 'git:' + path, self.params, log)
         self._cached_git_id = str(self.id)
@@ -516,6 +554,9 @@ class GitRepository(Repository):
     def get_youngest_rev(self):
         return self.git.youngest_rev()
 
+    def get_path_history(self, path, rev=None, limit=None):
+        raise TracError(_("Unsupported \"Show only adds and deletes\""))
+
     def get_oldest_rev(self):
         return self.git.oldest_rev()
 
@@ -537,8 +578,11 @@ class GitRepository(Repository):
         return self.git.shortrev(self.normalize_rev(rev),
                                  min_len=self.shortrev_len)
 
-    def get_node(self, path, rev=None, historian=None):
-        return GitNode(self, path, rev, self.log, None, historian)
+    def get_node(self, path, rev=None):
+        return self._get_node(path, rev)
+
+    def _get_node(self, path, rev, ls_tree_info=None, historian=None):
+        return GitNode(self, path, rev, self.log, ls_tree_info, historian)
 
     def get_quickjump_entries(self, rev):
         for bname, bsha in self.git.get_branches():
@@ -564,30 +608,41 @@ class GitRepository(Repository):
     def get_changes(self, old_path, old_rev, new_path, new_rev,
                     ignore_ancestry=0):
         # TODO: handle renames/copies, ignore_ancestry
+        old_path = self.normalize_path(old_path)
+        new_path = self.normalize_path(new_path)
         if old_path != new_path:
             raise TracError(_("Not supported in git_fs"))
 
-        with self.git.get_historian(old_rev,
-                                    old_path.strip('/')) as old_historian:
-            with self.git.get_historian(new_rev,
-                                        new_path.strip('/')) as new_historian:
-                for chg in self.git.diff_tree(old_rev, new_rev,
-                                              self.normalize_path(new_path)):
-                    mode1, mode2, obj1, obj2, action, path, path2 = chg
+        old_rev = self.normalize_rev(old_rev)
+        new_rev = self.normalize_rev(new_rev)
+        if old_rev == new_rev:
+            return
 
-                    kind = Node.FILE
-                    if mode2.startswith('04') or mode1.startswith('04'):
-                        kind = Node.DIRECTORY
+        def get_tree(rev):
+            results = self.git.ls_tree(rev, target_path, recursive=True)
+            return dict((result[4], result) for result in results)
+
+        target_path = old_path.strip('/')
+        old_tree = get_tree(old_rev)
+        new_tree = get_tree(new_rev)
+
+        with self.git.get_historian(old_rev, target_path) as old_historian:
+            with self.git.get_historian(new_rev, target_path) as new_historian:
+                for chg in self.git.diff_tree(old_rev, new_rev, target_path):
+                    mode1, mode2, obj1, obj2, action, path, path2 = chg
+                    kind = Node.DIRECTORY \
+                           if _is_dir(mode2) or _is_dir(mode1) \
+                           else Node.FILE
 
                     change = GitChangeset.action_map[action]
-
-                    old_node = None
-                    new_node = None
-
-                    if change != Changeset.ADD:
-                        old_node = self.get_node(path, old_rev, old_historian)
-                    if change != Changeset.DELETE:
-                        new_node = self.get_node(path, new_rev, new_historian)
+                    old_node = self._get_node(path, old_rev,
+                                              old_tree.get(path, False),
+                                              old_historian) \
+                               if change != Changeset.ADD else None
+                    new_node = self._get_node(path, new_rev,
+                                              new_tree.get(path, False),
+                                              new_historian) \
+                               if change != Changeset.DELETE else None
 
                     yield old_node, new_node, kind, change
 
@@ -639,44 +694,46 @@ class GitNode(Node):
         self.repos = repos
         self.fs_sha = None # points to either tree or blobs
         self.fs_perm = None
+        self.fs_type = None
         self.fs_size = None
         if rev:
             rev = repos.normalize_rev(to_unicode(rev))
         else:
             rev = repos.youngest_rev
+        created_rev = rev
 
         kind = Node.DIRECTORY
         p = path.strip('/')
         if p:  # ie. not the root-tree
             if not rev:
                 raise NoSuchNode(path, rev)
-            if not ls_tree_info:
-                ls_tree_info = repos.git.ls_tree(rev, p) or None
+            if ls_tree_info is None:
+                ls_tree_info = repos.git.ls_tree(rev, p)
                 if ls_tree_info:
-                    [ls_tree_info] = ls_tree_info
-
+                    ls_tree_info = ls_tree_info[0]
             if not ls_tree_info:
                 raise NoSuchNode(path, rev)
-
-            self.fs_perm, k, self.fs_sha, self.fs_size, _ = ls_tree_info
+            self.fs_perm, self.fs_type, self.fs_sha, self.fs_size, fname = \
+                ls_tree_info
 
             # fix-up to the last commit-rev that touched this node
-            rev = repos.git.last_change(rev, p, historian)
+            created_rev = repos.git.last_change(rev, p, historian)
 
-            if k == 'tree':
-                pass
-            elif k == 'commit':
+            if self.fs_type == 'tree':
+                kind = Node.DIRECTORY
+            elif self.fs_type == 'blob':
+                kind = Node.FILE
+            elif _is_submodule(self.fs_perm):
                 # FIXME: this is a workaround for missing git submodule
                 #        support in the plugin
-                pass
-            elif k == 'blob':
-                kind = Node.FILE
+                kind = Node.DIRECTORY
             else:
+                self.log.warning('Got unexpected object %r', ls_tree_info)
                 raise TracError(_("Internal error (got unexpected object "
-                                  "kind '%(kind)s')", kind=k))
+                                  "kind '%(kind)s')", kind=self.fs_type))
 
         self.created_path = path
-        self.created_rev = rev
+        self.created_rev = created_rev
 
         Node.__init__(self, repos, path, rev, kind)
 
@@ -694,11 +751,15 @@ class GitNode(Node):
     def get_content(self):
         if not self.isfile:
             return None
-
         return self.repos.git.get_file(self.fs_sha)
 
     def get_properties(self):
-        return self.fs_perm and {'mode': self.fs_perm } or {}
+        if self.fs_perm is None:
+            return {}
+        props = {'mode': '%06o' % self.fs_perm}
+        if _is_submodule(self.fs_perm):
+            props['commit'] = self.fs_sha
+        return props
 
     def get_annotations(self):
         if not self.isfile:
@@ -712,6 +773,8 @@ class GitNode(Node):
             return
         if not self.isdir:
             return
+        if _is_submodule(self.fs_perm):
+            return
 
         with self.repos.git.get_historian(self.rev,
                                           self.path.strip('/')) as historian:
@@ -846,16 +909,17 @@ class GitChangeset(Changeset):
         # Returns the differences against the first parent
         parent = self.props.get('parent')
         parent = parent[0] if parent else None
+
         for mode1, mode2, obj1, obj2, action, path1, path2 in \
                 self.repos.git.diff_tree(parent, self.rev, find_renames=True):
             path = path2 or path1
             p_path, p_rev = path1, parent
 
             kind = Node.DIRECTORY \
-                   if mode2.startswith('04') or mode1.startswith('04') \
-                   else Node.FILE
+                   if _is_dir(mode2) or _is_dir(mode1) else \
+                   Node.FILE
 
-            action = GitChangeset.action_map[action[0]]
+            action = GitChangeset.action_map[action]
 
             if action == Changeset.ADD:
                 p_path = p_rev = None
@@ -869,37 +933,54 @@ class GitChangeset(Changeset):
                 for k, v in self.repos.git.get_branch_contains(_rev,
                                                                resolve=True)]
 
+    def get_tags(self):
+        return self.repos.git.get_tags(self.rev)
+
 
 class GitwebProjectsRepositoryProvider(Component):
 
     implements(IRepositoryProvider)
 
-    projects_list = PathOption('git', 'projects_list', doc=
+    projects_list = PathOption('gitweb-repositories', 'projects_list', doc=
         """Path to a gitweb-formatted projects.list""")
 
-    projects_base = PathOption('git', 'projects_base', doc=
+    projects_base = PathOption('gitweb-repositories', 'projects_base', doc=
         """Path to the base of your git projects""")
 
-    projects_url = Option('git', 'projects_url', doc=
-        """Template for project URLs. %s will be replaced with the repo
+    projects_url = Option('gitweb-repositories', 'projects_url', doc=
+        """Template for project URLs. `%s` will be replaced with the repo
         name""")
 
+    sync_per_request = ListOption('gitweb-repositories',
+        'sync_per_request', '', doc="""Repositories to sync on every request
+        (not recommended).""")
+
     def get_repositories(self):
+        """Retrieve repositories specified in a `projects_list` file."""
         if not self.projects_list:
             return
 
-        for line in open(self.projects_list):
-            line = line.strip()
-            name = line
-            if name.endswith('.git'):
-                name = name[:-4]
-            repo = {
-                'dir': os.path.join(self.projects_base, line),
-                'type': 'git',
-            }
-            description_path = os.path.join(repo['dir'], 'description')
-            if os.path.exists(description_path):
-                repo['description'] = open(description_path).read().strip()
-            if self.projects_url:
-                repo['url'] = self.projects_url % name
-            yield name, repo
+        if not os.path.exists(self.projects_list):
+            self.log.warn("The [git] projects_list file was not found at "
+                          "'%s'", self.projects_list)
+            return
+
+        with open(self.projects_list, 'r') as fp:
+            for line in fp:
+                entries = line.strip().split()
+                if entries:
+                    name = entries[0]
+                    reponame = name.rstrip('.git')
+                    info = {
+                        'dir': os.path.join(self.projects_base, name),
+                        'sync_per_request': reponame in self.sync_per_request,
+                        'type': 'git',
+                    }
+                    description_path = \
+                        os.path.join(info['dir'], 'description')
+                    if os.path.exists(description_path):
+                        with open(description_path, 'r') as fd:
+                            info['description'] = fd.read().strip()
+                    if self.projects_url:
+                        info['url'] = self.projects_url % reponame
+                    yield reponame, info
