@@ -1,40 +1,32 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2013 Edgewall Software
+# Copyright (C) 2012-2019 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
-
-from __future__ import with_statement
+# history and logs, available at https://trac.edgewall.org/log/.
 
 import os
 import tempfile
 import unittest
 from datetime import datetime
-from subprocess import Popen, PIPE
 
 import trac.tests.compat
-from trac.test import locate, EnvironmentStub
-from trac.tests.compat import rmtree
+from trac.test import EnvironmentStub, rmtree
 from trac.util import create_file
-from trac.util.compat import close_fds
 from trac.versioncontrol.api import Changeset, DbRepositoryProvider, \
                                     RepositoryManager
-from tracopt.versioncontrol.git.git_fs import GitConnector
 from tracopt.versioncontrol.git.PyGIT import GitCore, GitError, Storage, \
-                                             StorageFactory, parse_commit
+                                             SizedDict, StorageFactory, \
+                                             parse_commit
 from tracopt.versioncontrol.git.tests.git_fs import GitCommandMixin
 
 
-git_bin = None
-
-
 class GitTestCase(unittest.TestCase):
 
     def test_is_sha(self):
@@ -186,7 +178,7 @@ class NormalTestCase(unittest.TestCase, GitCommandMixin):
     def _storage(self, path=None):
         if path is None:
             path = os.path.join(self.repos_path, '.git')
-        return Storage(path, self.env.log, git_bin, 'utf-8')
+        return Storage(path, self.env.log, self.git_bin, 'utf-8')
 
     def test_control_files_detection(self):
         # Exception not raised when path points to ctrl file dir
@@ -317,7 +309,7 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 
     def _storage(self):
         path = os.path.join(self.repos_path, '.git')
-        return Storage(path, self.env.log, git_bin, 'utf-8')
+        return Storage(path, self.env.log, self.git_bin, 'utf-8')
 
     def test_unicode_verifyrev(self):
         storage = self._storage()
@@ -340,34 +332,34 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
                          storage.read_commit(storage.head())[1]['author'][0])
 
     def test_unicode_branches(self):
-        self._git('checkout', '-b', 'tickét10980', 'master')
+        self._git('checkout', '-b', 'tickɇt10980', 'master')
         storage = self._storage()
         branches = sorted(storage.get_branches())
         self.assertEqual(unicode, type(branches[0][0]))
         self.assertEqual(unicode, type(branches[1][0]))
         self.assertEqual(u'master', branches[0][0])
-        self.assertEqual(u'tickét10980', branches[1][0])
+        self.assertEqual(u'tickɇt10980', branches[1][0])
 
         contains = sorted(storage.get_branch_contains(branches[1][1],
                                                       resolve=True))
         self.assertEqual(unicode, type(contains[0][0]))
         self.assertEqual(unicode, type(contains[1][0]))
         self.assertEqual(u'master', contains[0][0])
-        self.assertEqual(u'tickét10980', contains[1][0])
+        self.assertEqual(u'tickɇt10980', contains[1][0])
 
     def test_unicode_tags(self):
-        self._git('tag', 'täg-t10980', 'master')
+        self._git('tag', 'tɐg-t10980', 'master')
         self._git_commit('-m', 'blah', '--allow-empty')
         self._git('tag', 'v0.42.1', 'master')
         storage = self._storage()
 
         tags = storage.get_tags()
         self.assertEqual(unicode, type(tags[0]))
-        self.assertEqual([u'täg-t10980', 'v0.42.1'], tags)
+        self.assertEqual([u'tɐg-t10980', 'v0.42.1'], tags)
 
-        rev = storage.verifyrev(u'täg-t10980')
+        rev = storage.verifyrev(u'tɐg-t10980')
         self.assertNotEqual(None, rev)
-        self.assertEqual([u'täg-t10980'], storage.get_tags(rev))
+        self.assertEqual([u'tɐg-t10980'], storage.get_tags(rev))
 
         rev = storage.verifyrev('v0.42.1')
         self.assertNotEqual(None, rev)
@@ -411,7 +403,7 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
             rev = storage.head()
             with storage.get_historian('HEAD', path) as historian:
                 hrev = storage.last_change('HEAD', path, historian)
-                self.assertEquals(rev, hrev)
+                self.assertEqual(rev, hrev)
 
         validate(paths[0], 'true')
         validate(paths[0], 'false')
@@ -421,6 +413,13 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
         validate(paths[2], 'false')
 
 
+class SizedDictTestCase(unittest.TestCase):
+
+    def test_setdefault_raises(self):
+        """`setdefault` raises NotImplementedError."""
+        self.assertRaises(NotImplementedError, SizedDict().setdefault)
+
+
 #class GitPerformanceTestCase(unittest.TestCase):
 #    """Performance test. Not really a unit test.
 #    Not self-contained: Needs a git repository and prints performance result
@@ -445,7 +444,7 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 #        t = timeit.Timer("shortrev_test()",
 #                         "from __main__ import shortrev_test")
 #        usec_per_rev = (1000000 * t.timeit(number=iters)/len(revs))
-#        print "%.2f usec/rev" % usec_per_rev # Print instead of testing
+#        print("%.2f usec/rev" % usec_per_rev) # Print instead of testing
 
 #class GitMemoryUsageTestCase(unittest.TestCase):
 #    """Memory test. Not really a unit test.
@@ -473,18 +472,18 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 #                    raise RuntimeError("failed to get memory stats")
 #
 #        else: # not linux2
-#            print "WARNING - meminfo.proc_statm() not available"
+#            print("WARNING - meminfo.proc_statm() not available")
 #            def proc_statm():
 #                return (0,)*7
 #
-#        print "statm =", proc_statm()
+#        print("statm =", proc_statm())
 #        __data_size = proc_statm()[5]
 #        __data_size_last = [__data_size]
 #
 #        def print_data_usage():
 #            __tmp = proc_statm()[5]
-#            print "DATA: %6d %+6d" % (__tmp - __data_size,
-#                                    __tmp - __data_size_last[0])
+#            print("DATA: %6d %+6d" % (__tmp - __data_size,
+#                                      __tmp - __data_size_last[0]))
 #            __data_size_last[0] = __tmp
 #
 #        print_data_usage()
@@ -493,46 +492,46 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 #
 #        print_data_usage()
 #
-#        print "[%s]" % g.head()
-#        print g.ls_tree(g.head())
-#        print "--------------"
+#        print("[%s]" % g.head())
+#        print(g.ls_tree(g.head()))
+#        print("--------------")
 #        print_data_usage()
-#        print g.read_commit(g.head())
-#        print "--------------"
+#        print(g.read_commit(g.head()))
+#        print("--------------")
 #        print_data_usage()
 #        p = g.parents(g.head())
-#        print list(p)
-#        print "--------------"
-#        print list(g.children(list(p)[0]))
-#        print list(g.children(list(p)[0]))
-#        print "--------------"
-#        print g.get_commit_encoding()
-#        print "--------------"
-#        print g.get_branches()
-#        print "--------------"
-#        print g.hist_prev_revision(g.oldest_rev()), g.oldest_rev(), \
-#                                g.hist_next_revision(g.oldest_rev())
+#        print(list(p))
+#        print("--------------")
+#        print(list(g.children(list(p)[0])))
+#        print(list(g.children(list(p)[0])))
+#        print("--------------")
+#        print(g.get_commit_encoding())
+#        print("--------------")
+#        print(g.get_branches())
+#        print("--------------")
+#        print(g.hist_prev_revision(g.oldest_rev()), g.oldest_rev(),
+#                                   g.hist_next_revision(g.oldest_rev()))
 #        print_data_usage()
-#        print "--------------"
+#        print("--------------")
 #        p = g.youngest_rev()
-#        print g.hist_prev_revision(p), p, g.hist_next_revision(p)
-#        print "--------------"
+#        print(g.hist_prev_revision(p), p, g.hist_next_revision(p))
+#        print("--------------")
 #
 #        p = g.head()
 #        for i in range(-5, 5):
-#            print i, g.history_relative_rev(p, i)
+#            print(i, g.history_relative_rev(p, i))
 #
 #        # check for loops
 #        def check4loops(head):
-#            print "check4loops", head
+#            print("check4loops", head)
 #            seen = set([head])
 #            for _sha in g.children_recursive(head):
 #                if _sha in seen:
-#                    print "dupe detected :-/", _sha, len(seen)
+#                    print("dupe detected :-/", _sha, len(seen))
 #                seen.add(_sha)
 #            return seen
 #
-#        print len(check4loops(g.parents(g.head())[0]))
+#        print(len(check4loops(g.parents(g.head())[0])))
 #
 #        #p = g.head()
 #        #revs = [ g.history_relative_rev(p, i) for i in range(0,10) ]
@@ -540,40 +539,38 @@ class UnicodeNameTestCase(unittest.TestCase, GitCommandMixin):
 #        revs = g.get_commits().keys()
 #        print_data_usage()
 #
-#        #print len(check4loops(g.oldest_rev()))
-#        #print len(list(g.children_recursive(g.oldest_rev())))
+#        #print(len(check4loops(g.oldest_rev())))
+#        #print(len(list(g.children_recursive(g.oldest_rev()))))
 #
 #        print_data_usage()
 #
 #        # perform typical trac operations:
 #
 #        if 1:
-#            print "--------------"
+#            print("--------------")
 #            rev = g.head()
 #            for mode, _type, sha, _size, name in g.ls_tree(rev):
 #                [last_rev] = g.history(rev, name, limit=1)
 #                s = g.get_obj_size(sha) if _type == 'blob' else 0
 #                msg = g.read_commit(last_rev)
 #
-#                print "%s %s %10d [%s]" % (_type, last_rev, s, name)
+#                print("%s %s %10d [%s]" % (_type, last_rev, s, name))
 #
-#        print "allocating 2nd instance"
+#        print("allocating 2nd instance")
 #        print_data_usage()
 #        g2 = Storage(path_to_repo, logging) # Need a git repository path here
 #        g2.head()
 #        print_data_usage()
 #
-#        print "allocating 3rd instance"
+#        print("allocating 3rd instance")
 #        g3 = Storage(path_to_repo, logging) # Need a git repository path here
 #        g3.head()
 #        print_data_usage()
 
 
-def suite():
-    global git_bin
+def test_suite():
     suite = unittest.TestSuite()
-    git_bin = locate('git')
-    if git_bin:
+    if GitCommandMixin.git_bin:
         suite.addTest(unittest.makeSuite(GitTestCase))
         suite.addTest(unittest.makeSuite(TestParseCommit))
         suite.addTest(unittest.makeSuite(NormalTestCase))
@@ -583,8 +580,9 @@ def suite():
     else:
         print("SKIP: tracopt/versioncontrol/git/tests/PyGIT.py (git cli "
               "binary, 'git', not found)")
+    suite.addTest(unittest.makeSuite(SizedDictTestCase))
     return suite
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
