@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2009 Edgewall Software
+# Copyright (C) 2005-2019 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.org/wiki/TracLicense.
+# are also available at https://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://trac.edgewall.org/log/.
+# history and logs, available at https://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Christian Boos <cboos@edgewall.org>
@@ -59,8 +59,7 @@ class SubversionPropertyRenderer(Component):
         be mapped to `http://ourserver/tracs/tools/browser/tags/1.1/tools?rev=`
         (and `rev` will be set to the appropriate revision number if the
         external additionally specifies a revision, see the
-        [http://svnbook.red-bean.com/en/1.4/svn.advanced.externals.html SVN Book on externals]
-        for more details).
+        [%(svnbook)s SVN Book on externals] for more details).
 
         Note that the number used as a key in the above section is purely used
         as a place holder, as the URLs themselves can't be used as a key due to
@@ -69,8 +68,8 @@ class SubversionPropertyRenderer(Component):
         Finally, the relative URLs introduced in
         [http://subversion.apache.org/docs/release-notes/1.5.html#externals Subversion 1.5]
         are not yet supported.
-
-        (''since 0.11'')""")
+        """,
+        doc_args={'svnbook': 'http://svnbook.red-bean.com/en/1.7/svn.advanced.externals.html'})
 
     def __init__(self):
         self._externals_map = {}
